@@ -16,6 +16,7 @@ from ._pydantic_core import (
     ValidationError,
     __version__,
     to_json,
+    to_jsonable_python,
 )
 from .core_schema import CoreConfig, CoreSchema, CoreSchemaType
 
@@ -41,6 +42,7 @@ __all__ = (
     'ArgsKwargs',
     'SchemaError',
     'ErrorDetails',
+    'InitErrorDetails',
     'ValidationError',
     'PydanticCustomError',
     'PydanticKnownError',
@@ -48,6 +50,7 @@ __all__ = (
     'PydanticSerializationError',
     'PydanticSerializationUnexpectedValue',
     'to_json',
+    'to_jsonable_python',
 )
 
 
@@ -57,3 +60,10 @@ class ErrorDetails(_TypedDict):
     msg: str
     input: _Any
     ctx: _NotRequired['dict[str, str | int | float]']
+
+
+class InitErrorDetails(_TypedDict):
+    type: 'str | PydanticCustomError'
+    loc: _NotRequired['tuple[int | str, ...]']
+    input: _Any
+    ctx: _NotRequired['dict[str, str | int | float]']
