@@ -1,24 +1,27 @@
 use std::fmt;
 
-use pyo3::prelude::*;
-use pyo3::types::{PyString, PyType};
+use pyo3::types::{PyDict, PyString, PyType};
+use pyo3::{intern, prelude::*};
 
 use crate::errors::{InputValue, LocItem, ValResult};
 use crate::{PyMultiHostUrl, PyUrl};
 
 use super::datetime::{EitherDate, EitherDateTime, EitherTime, EitherTimedelta};
 use super::return_enums::{EitherBytes, EitherString};
-use super::{GenericArguments, GenericCollection, GenericIterator, GenericMapping, JsonInput};
+use super::{GenericArguments, GenericIterable, GenericIterator, GenericMapping, JsonInput};
 
+#[derive(Debug, Clone, Copy)]
 pub enum InputType {
     Python,
     Json,
-    String,
 }
 
-impl InputType {
-    pub fn is_json(&self) -> bool {
-        matches!(self, Self::Json)
+impl IntoPy<PyObject> for InputType {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self {
+            Self::Json => intern!(py, "json").into(),
+            Self::Python => intern!(py, "python").into(),
+        }
     }
 }
 
@@ -27,8 +30,6 @@ impl InputType {
 /// * `strict_*` & `lax_*` if they have different behavior
 /// * or, `validate_*` and `strict_*` to just call `validate_*` if the behavior for strict and lax is the same
 pub trait Input<'a>: fmt::Debug + ToPyObject {
-    fn get_type(&self) -> &'static InputType;
-
     fn as_loc_item(&self) -> LocItem;
 
     fn as_error_value(&'a self) -> InputValue<'a>;
@@ -44,9 +45,6 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         None
     }
 
-    // input_ prefix to differentiate from the function on PyAny
-    fn input_is_instance(&self, class: &PyAny, json_mask: u8) -> PyResult<bool>;
-
     fn is_exact_instance(&self, _class: &PyType) -> bool {
         false
     }
@@ -55,6 +53,8 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         false
     }
 
+    fn as_kwargs(&'a self, py: Python<'a>) -> Option<&'a PyDict>;
+
     fn input_is_subclass(&self, _class: &PyType) -> PyResult<bool> {
         Ok(false)
     }
@@ -90,6 +90,8 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         self.strict_str()
     }
 
+    fn as_str_strict(&self) -> Option<&str>;
+
     fn validate_bytes(&'a self, strict: bool) -> ValResult<EitherBytes<'a>> {
         if strict {
             self.strict_bytes()
@@ -129,13 +131,18 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         self.strict_int()
     }
 
-    fn validate_float(&self, strict: bool) -> ValResult<f64> {
-        if strict {
+    fn as_int_strict(&self) -> Option<i64>;
+
+    fn validate_float(&self, strict: bool, ultra_strict: bool) -> ValResult<f64> {
+        if ultra_strict {
+            self.ultra_strict_float()
+        } else if strict {
             self.strict_float()
         } else {
             self.lax_float()
         }
     }
+    fn ultra_strict_float(&self) -> ValResult<f64>;
     fn strict_float(&self) -> ValResult<f64>;
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn lax_float(&self) -> ValResult<f64> {
@@ -155,62 +162,64 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         self.strict_dict()
     }
 
-    fn validate_typed_dict(&'a self, strict: bool, _from_attributes: bool) -> ValResult<GenericMapping<'a>> {
+    fn validate_model_fields(&'a self, strict: bool, _from_attributes: bool) -> ValResult<GenericMapping<'a>> {
         self.validate_dict(strict)
     }
 
-    fn validate_list(&'a self, strict: bool, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
-        if strict && !allow_any_iter {
+    fn validate_list(&'a self, strict: bool) -> ValResult<GenericIterable<'a>> {
+        if strict {
             self.strict_list()
         } else {
-            self.lax_list(allow_any_iter)
+            self.lax_list()
         }
     }
-    fn strict_list(&'a self) -> ValResult<GenericCollection<'a>>;
+    fn strict_list(&'a self) -> ValResult<GenericIterable<'a>>;
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn lax_list(&'a self, _allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
+    fn lax_list(&'a self) -> ValResult<GenericIterable<'a>> {
         self.strict_list()
     }
 
-    fn validate_tuple(&'a self, strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_tuple(&'a self, strict: bool) -> ValResult<GenericIterable<'a>> {
         if strict {
             self.strict_tuple()
         } else {
             self.lax_tuple()
         }
     }
-    fn strict_tuple(&'a self) -> ValResult<GenericCollection<'a>>;
+    fn strict_tuple(&'a self) -> ValResult<GenericIterable<'a>>;
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn lax_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn lax_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
         self.strict_tuple()
     }
 
-    fn validate_set(&'a self, strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_set(&'a self, strict: bool) -> ValResult<GenericIterable<'a>> {
         if strict {
             self.strict_set()
         } else {
             self.lax_set()
         }
     }
-    fn strict_set(&'a self) -> ValResult<GenericCollection<'a>>;
+    fn strict_set(&'a self) -> ValResult<GenericIterable<'a>>;
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn lax_set(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn lax_set(&'a self) -> ValResult<GenericIterable<'a>> {
         self.strict_set()
     }
 
-    fn validate_frozenset(&'a self, strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_frozenset(&'a self, strict: bool) -> ValResult<GenericIterable<'a>> {
         if strict {
             self.strict_frozenset()
         } else {
             self.lax_frozenset()
         }
     }
-    fn strict_frozenset(&'a self) -> ValResult<GenericCollection<'a>>;
+    fn strict_frozenset(&'a self) -> ValResult<GenericIterable<'a>>;
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn lax_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn lax_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
         self.strict_frozenset()
     }
 
+    fn extract_generic_iterable(&'a self) -> ValResult<GenericIterable<'a>>;
+
     fn validate_iter(&self) -> ValResult<GenericIterator>;
 
     fn validate_date(&self, strict: bool) -> ValResult<EitherDate> {
