@@ -1,24 +1,20 @@
 use pyo3::prelude::*;
+use pyo3::types::PyDict;
 
-use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValLineError, ValResult};
+use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValResult};
 
 use super::datetime::{
     bytes_as_date, bytes_as_datetime, bytes_as_time, bytes_as_timedelta, float_as_datetime, float_as_duration,
     float_as_time, int_as_datetime, int_as_duration, int_as_time, EitherDate, EitherDateTime, EitherTime,
 };
-use super::input_abstract::InputType;
 use super::parse_json::JsonArray;
 use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_int};
 use super::{
-    EitherBytes, EitherString, EitherTimedelta, GenericArguments, GenericCollection, GenericIterator, GenericMapping,
-    Input, JsonArgs, JsonInput, JsonType,
+    EitherBytes, EitherString, EitherTimedelta, GenericArguments, GenericIterable, GenericIterator, GenericMapping,
+    Input, JsonArgs, JsonInput,
 };
 
 impl<'a> Input<'a> for JsonInput {
-    fn get_type(&self) -> &'static InputType {
-        &InputType::Json
-    }
-
     /// This is required by since JSON object keys are always strings, I don't think it can be called
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn as_loc_item(&self) -> LocItem {
@@ -37,62 +33,22 @@ impl<'a> Input<'a> for JsonInput {
         matches!(self, JsonInput::Null)
     }
 
-    fn input_is_instance(&self, _class: &PyAny, json_mask: u8) -> PyResult<bool> {
-        if json_mask == 0 {
-            Ok(false)
-        } else {
-            let json_type: JsonType = match self {
-                JsonInput::Null => JsonType::Null,
-                JsonInput::Bool(_) => JsonType::Bool,
-                JsonInput::Int(_) => JsonType::Int,
-                JsonInput::Float(_) => JsonType::Float,
-                JsonInput::String(_) => JsonType::String,
-                JsonInput::Array(_) => JsonType::Array,
-                JsonInput::Object(_) => JsonType::Object,
-            };
-            Ok(json_type.matches(json_mask))
+    fn as_kwargs(&'a self, py: Python<'a>) -> Option<&'a PyDict> {
+        match self {
+            JsonInput::Object(object) => {
+                let dict = PyDict::new(py);
+                for (k, v) in object.iter() {
+                    dict.set_item(k, v.to_object(py)).unwrap();
+                }
+                Some(dict)
+            }
+            _ => None,
         }
     }
 
     fn validate_args(&'a self) -> ValResult<'a, GenericArguments<'a>> {
         match self {
-            JsonInput::Object(object) => {
-                if let Some(args) = object.get("__args__") {
-                    if let Some(kwargs) = object.get("__kwargs__") {
-                        // we only try this logic if there are only these two items in the dict
-                        if object.len() == 2 {
-                            let args = match args {
-                                JsonInput::Null => Ok(None),
-                                JsonInput::Array(args) => Ok(Some(args.as_slice())),
-                                _ => Err(ValLineError::new_with_loc(
-                                    ErrorType::PositionalArgumentsType,
-                                    args,
-                                    "__args__",
-                                )),
-                            };
-                            let kwargs = match kwargs {
-                                JsonInput::Null => Ok(None),
-                                JsonInput::Object(kwargs) => Ok(Some(kwargs)),
-                                _ => Err(ValLineError::new_with_loc(
-                                    ErrorType::KeywordArgumentsType,
-                                    kwargs,
-                                    "__kwargs__",
-                                )),
-                            };
-
-                            return match (args, kwargs) {
-                                (Ok(args), Ok(kwargs)) => Ok(JsonArgs::new(args, kwargs).into()),
-                                (Err(args_error), Err(kwargs_error)) => {
-                                    return Err(ValError::LineErrors(vec![args_error, kwargs_error]))
-                                }
-                                (Err(error), _) => Err(ValError::LineErrors(vec![error])),
-                                (_, Err(error)) => Err(ValError::LineErrors(vec![error])),
-                            };
-                        }
-                    }
-                }
-                Ok(JsonArgs::new(None, Some(object)).into())
-            }
+            JsonInput::Object(object) => Ok(JsonArgs::new(None, Some(object)).into()),
             JsonInput::Array(array) => Ok(JsonArgs::new(Some(array), None).into()),
             _ => Err(ValError::new(ErrorType::ArgumentsType, self)),
         }
@@ -128,6 +84,13 @@ impl<'a> Input<'a> for JsonInput {
         }
     }
 
+    fn as_str_strict(&self) -> Option<&str> {
+        match self {
+            JsonInput::String(s) => Some(s.as_str()),
+            _ => None,
+        }
+    }
+
     fn validate_bytes(&'a self, _strict: bool) -> ValResult<EitherBytes<'a>> {
         match self {
             JsonInput::String(s) => Ok(s.as_bytes().into()),
@@ -177,6 +140,19 @@ impl<'a> Input<'a> for JsonInput {
         }
     }
 
+    fn as_int_strict(&self) -> Option<i64> {
+        match self {
+            JsonInput::Int(i) => Some(*i),
+            _ => None,
+        }
+    }
+
+    fn ultra_strict_float(&self) -> ValResult<f64> {
+        match self {
+            JsonInput::Float(f) => Ok(*f),
+            _ => Err(ValError::new(ErrorType::FloatType, self)),
+        }
+    }
     fn strict_float(&self) -> ValResult<f64> {
         match self {
             JsonInput::Float(f) => Ok(*f),
@@ -211,53 +187,62 @@ impl<'a> Input<'a> for JsonInput {
         self.validate_dict(false)
     }
 
-    fn validate_list(&'a self, _strict: bool, _allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_list(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         match self {
-            JsonInput::Array(a) => Ok(a.into()),
+            JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
             _ => Err(ValError::new(ErrorType::ListType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_list(&'a self) -> ValResult<GenericCollection<'a>> {
-        self.validate_list(false, false)
+    fn strict_list(&'a self) -> ValResult<GenericIterable<'a>> {
+        self.validate_list(false)
     }
 
-    fn validate_tuple(&'a self, _strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_tuple(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         // just as in set's case, List has to be allowed
         match self {
-            JsonInput::Array(a) => Ok(a.into()),
+            JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
             _ => Err(ValError::new(ErrorType::TupleType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn strict_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
         self.validate_tuple(false)
     }
 
-    fn validate_set(&'a self, _strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_set(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         // we allow a list here since otherwise it would be impossible to create a set from JSON
         match self {
-            JsonInput::Array(a) => Ok(a.into()),
+            JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
             _ => Err(ValError::new(ErrorType::SetType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_set(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn strict_set(&'a self) -> ValResult<GenericIterable<'a>> {
         self.validate_set(false)
     }
 
-    fn validate_frozenset(&'a self, _strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_frozenset(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         // we allow a list here since otherwise it would be impossible to create a frozenset from JSON
         match self {
-            JsonInput::Array(a) => Ok(a.into()),
+            JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
             _ => Err(ValError::new(ErrorType::FrozenSetType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn strict_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
         self.validate_frozenset(false)
     }
 
+    fn extract_generic_iterable(&self) -> ValResult<GenericIterable> {
+        match self {
+            JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
+            JsonInput::String(s) => Ok(GenericIterable::JsonString(s)),
+            JsonInput::Object(object) => Ok(GenericIterable::JsonObject(object)),
+            _ => Err(ValError::new(ErrorType::IterableType, self)),
+        }
+    }
+
     fn validate_iter(&self) -> ValResult<GenericIterator> {
         match self {
             JsonInput::Array(a) => Ok(a.clone().into()),
@@ -332,10 +317,6 @@ impl<'a> Input<'a> for JsonInput {
 
 /// Required for Dict keys so the string can behave like an Input
 impl<'a> Input<'a> for String {
-    fn get_type(&self) -> &'static InputType {
-        &InputType::String
-    }
-
     fn as_loc_item(&self) -> LocItem {
         self.to_string().into()
     }
@@ -349,12 +330,8 @@ impl<'a> Input<'a> for String {
         false
     }
 
-    fn input_is_instance(&self, _class: &PyAny, json_mask: u8) -> PyResult<bool> {
-        if json_mask == 0 {
-            Ok(false)
-        } else {
-            Ok(JsonType::String.matches(json_mask))
-        }
+    fn as_kwargs(&'a self, _py: Python<'a>) -> Option<&'a PyDict> {
+        None
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -379,6 +356,10 @@ impl<'a> Input<'a> for String {
         self.validate_str(false)
     }
 
+    fn as_str_strict(&self) -> Option<&str> {
+        Some(self.as_str())
+    }
+
     fn validate_bytes(&'a self, _strict: bool) -> ValResult<EitherBytes<'a>> {
         Ok(self.as_bytes().into())
     }
@@ -404,6 +385,14 @@ impl<'a> Input<'a> for String {
         }
     }
 
+    fn as_int_strict(&self) -> Option<i64> {
+        None
+    }
+
+    #[cfg_attr(has_no_coverage, no_coverage)]
+    fn ultra_strict_float(&self) -> ValResult<f64> {
+        self.strict_float()
+    }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_float(&self) -> ValResult<f64> {
         Err(ValError::new(ErrorType::FloatType, self))
@@ -425,41 +414,45 @@ impl<'a> Input<'a> for String {
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn validate_list(&'a self, _strict: bool, _allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_list(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         Err(ValError::new(ErrorType::ListType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_list(&'a self) -> ValResult<GenericCollection<'a>> {
-        self.validate_list(false, false)
+    fn strict_list(&'a self) -> ValResult<GenericIterable<'a>> {
+        self.validate_list(false)
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn validate_tuple(&'a self, _strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_tuple(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         Err(ValError::new(ErrorType::TupleType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn strict_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
         self.validate_tuple(false)
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn validate_set(&'a self, _strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_set(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         Err(ValError::new(ErrorType::SetType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_set(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn strict_set(&'a self) -> ValResult<GenericIterable<'a>> {
         self.validate_set(false)
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn validate_frozenset(&'a self, _strict: bool) -> ValResult<GenericCollection<'a>> {
+    fn validate_frozenset(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         Err(ValError::new(ErrorType::FrozenSetType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn strict_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
         self.validate_frozenset(false)
     }
 
+    fn extract_generic_iterable(&'a self) -> ValResult<GenericIterable<'a>> {
+        Ok(GenericIterable::JsonString(self))
+    }
+
     fn validate_iter(&self) -> ValResult<GenericIterator> {
         Ok(string_to_vec(self).into())
     }
