@@ -4,15 +4,15 @@ use std::str::from_utf8;
 use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::{
-    PyBool, PyByteArray, PyBytes, PyDate, PyDateTime, PyDelta, PyDict, PyFrozenSet, PyIterator, PyList, PyMapping,
-    PySet, PyString, PyTime, PyTuple, PyType,
+    PyBool, PyByteArray, PyBytes, PyDate, PyDateTime, PyDelta, PyDict, PyFrozenSet, PyInt, PyIterator, PyList,
+    PyMapping, PySequence, PySet, PyString, PyTime, PyTuple, PyType,
 };
 #[cfg(not(PyPy))]
 use pyo3::types::{PyDictItems, PyDictKeys, PyDictValues};
 use pyo3::{ffi, intern, AsPyPointer, PyTypeInfo};
 
 use crate::build_tools::safe_repr;
-use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValLineError, ValResult};
+use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValResult};
 use crate::{ArgsKwargs, PyMultiHostUrl, PyUrl};
 
 use super::datetime::{
@@ -20,51 +20,66 @@ use super::datetime::{
     float_as_duration, float_as_time, int_as_datetime, int_as_duration, int_as_time, EitherDate, EitherDateTime,
     EitherTime,
 };
-use super::input_abstract::InputType;
 use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_int};
 use super::{
-    py_error_on_minusone, py_string_str, EitherBytes, EitherString, EitherTimedelta, GenericArguments,
-    GenericCollection, GenericIterator, GenericMapping, Input, JsonInput, PyArgs,
+    py_string_str, EitherBytes, EitherString, EitherTimedelta, GenericArguments, GenericIterable, GenericIterator,
+    GenericMapping, Input, JsonInput, PyArgs,
 };
 
-/// Extract generators and deques into a `GenericCollection`
-macro_rules! extract_shared_iter {
-    ($type:ty, $obj:ident) => {
-        if $obj.downcast::<PyIterator>().is_ok() {
-            Some($obj.into())
-        } else if is_deque($obj) {
-            Some($obj.into())
-        } else {
-            None
-        }
-    };
-}
-
-/// Extract dict keys, values and items into a `GenericCollection`
 #[cfg(not(PyPy))]
-macro_rules! extract_dict_iter {
-    ($obj:ident) => {
-        if $obj.is_instance_of::<PyDictKeys>().unwrap_or(false) {
-            Some($obj.into())
-        } else if $obj.is_instance_of::<PyDictValues>().unwrap_or(false) {
-            Some($obj.into())
-        } else if $obj.is_instance_of::<PyDictItems>().unwrap_or(false) {
-            Some($obj.into())
-        } else {
-            None
-        }
+macro_rules! extract_dict_keys {
+    ($py:expr, $obj:ident) => {
+        $obj.downcast::<PyDictKeys>()
+            .ok()
+            .map(|v| PyIterator::from_object($py, v).unwrap())
     };
 }
 
 #[cfg(PyPy)]
-macro_rules! extract_dict_iter {
-    ($obj:ident) => {
+macro_rules! extract_dict_keys {
+    ($py:expr, $obj:ident) => {
         if is_dict_keys_type($obj) {
-            Some($obj.into())
-        } else if is_dict_values_type($obj) {
-            Some($obj.into())
-        } else if is_dict_items_type($obj) {
-            Some($obj.into())
+            Some(PyIterator::from_object($py, $obj).unwrap())
+        } else {
+            None
+        }
+    };
+}
+
+#[cfg(not(PyPy))]
+macro_rules! extract_dict_values {
+    ($py:expr, $obj:ident) => {
+        $obj.downcast::<PyDictValues>()
+            .ok()
+            .map(|v| PyIterator::from_object($py, v).unwrap())
+    };
+}
+
+#[cfg(PyPy)]
+macro_rules! extract_dict_values {
+    ($py:expr, $obj:ident) => {
+        if is_dict_values_type($obj) {
+            Some(PyIterator::from_object($py, $obj).unwrap())
+        } else {
+            None
+        }
+    };
+}
+
+#[cfg(not(PyPy))]
+macro_rules! extract_dict_items {
+    ($py:expr, $obj:ident) => {
+        $obj.downcast::<PyDictItems>()
+            .ok()
+            .map(|v| PyIterator::from_object($py, v).unwrap())
+    };
+}
+
+#[cfg(PyPy)]
+macro_rules! extract_dict_items {
+    ($py:expr, $obj:ident) => {
+        if is_dict_items_type($obj) {
+            Some(PyIterator::from_object($py, $obj).unwrap())
         } else {
             None
         }
@@ -72,10 +87,6 @@ macro_rules! extract_dict_iter {
 }
 
 impl<'a> Input<'a> for PyAny {
-    fn get_type(&self) -> &'static InputType {
-        &InputType::Python
-    }
-
     fn as_loc_item(&self) -> LocItem {
         if let Ok(py_str) = self.downcast::<PyString>() {
             py_str.to_string_lossy().as_ref().into()
@@ -102,14 +113,6 @@ impl<'a> Input<'a> for PyAny {
         Some(self.getattr(name))
     }
 
-    fn input_is_instance(&self, class: &PyAny, _json_mask: u8) -> PyResult<bool> {
-        // See PyO3/pyo3#2694 - we can't use `is_instance` here since it requires PyType,
-        // and some check objects are not types, this logic is lifted from `is_instance` in PyO3
-        let result = unsafe { ffi::PyObject_IsInstance(self.as_ptr(), class.as_ptr()) };
-        py_error_on_minusone(self.py(), result)?;
-        Ok(result == 1)
-    }
-
     fn is_exact_instance(&self, class: &PyType) -> bool {
         self.get_type().is(class)
     }
@@ -118,6 +121,10 @@ impl<'a> Input<'a> for PyAny {
         true
     }
 
+    fn as_kwargs(&'a self, _py: Python<'a>) -> Option<&'a PyDict> {
+        self.downcast::<PyDict>().ok()
+    }
+
     fn input_is_subclass(&self, class: &PyType) -> PyResult<bool> {
         match self.downcast::<PyType>() {
             Ok(py_type) => py_type.is_subclass(class),
@@ -139,48 +146,11 @@ impl<'a> Input<'a> for PyAny {
 
     fn validate_args(&'a self) -> ValResult<'a, GenericArguments<'a>> {
         if let Ok(dict) = self.downcast::<PyDict>() {
-            if let Some(args) = dict.get_item(intern!(self.py(), "__args__")) {
-                if let Some(kwargs) = dict.get_item(intern!(self.py(), "__kwargs__")) {
-                    // we only try this logic if there are only these two items in the dict
-                    if dict.len() == 2 {
-                        let args = if let Ok(tuple) = args.downcast::<PyTuple>() {
-                            Ok(Some(tuple))
-                        } else if args.is_none() {
-                            Ok(None)
-                        } else if let Ok(list) = args.downcast::<PyList>() {
-                            Ok(Some(list_as_tuple(list)))
-                        } else {
-                            Err(ValLineError::new_with_loc(
-                                ErrorType::PositionalArgumentsType,
-                                args,
-                                "__args__",
-                            ))
-                        };
-
-                        let kwargs = if let Ok(dict) = kwargs.downcast::<PyDict>() {
-                            Ok(Some(dict))
-                        } else if kwargs.is_none() {
-                            Ok(None)
-                        } else {
-                            Err(ValLineError::new_with_loc(
-                                ErrorType::KeywordArgumentsType,
-                                kwargs,
-                                "__kwargs__",
-                            ))
-                        };
-
-                        return match (args, kwargs) {
-                            (Ok(args), Ok(kwargs)) => Ok(PyArgs::new(args, kwargs).into()),
-                            (Err(args_error), Err(kwargs_error)) => {
-                                Err(ValError::LineErrors(vec![args_error, kwargs_error]))
-                            }
-                            (Err(error), _) => Err(ValError::LineErrors(vec![error])),
-                            (_, Err(error)) => Err(ValError::LineErrors(vec![error])),
-                        };
-                    }
-                }
-            }
             Ok(PyArgs::new(None, Some(dict)).into())
+        } else if let Ok(args_kwargs) = self.extract::<ArgsKwargs>() {
+            let args = args_kwargs.args.into_ref(self.py());
+            let kwargs = args_kwargs.kwargs.map(|d| d.into_ref(self.py()));
+            Ok(PyArgs::new(Some(args), kwargs).into())
         } else if let Ok(tuple) = self.downcast::<PyTuple>() {
             Ok(PyArgs::new(Some(tuple), None).into())
         } else if let Ok(list) = self.downcast::<PyList>() {
@@ -257,6 +227,14 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
+    fn as_str_strict(&self) -> Option<&str> {
+        if self.get_type().is(get_py_str_type(self.py())) {
+            self.extract().ok()
+        } else {
+            None
+        }
+    }
+
     fn strict_bytes(&'a self) -> ValResult<EitherBytes<'a>> {
         if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             Ok(py_bytes.into())
@@ -326,6 +304,23 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
+    fn as_int_strict(&self) -> Option<i64> {
+        if self.get_type().is(get_py_int_type(self.py())) {
+            self.extract().ok()
+        } else {
+            None
+        }
+    }
+
+    fn ultra_strict_float(&self) -> ValResult<f64> {
+        if matches!(self.is_instance_of::<PyInt>(), Ok(true)) {
+            Err(ValError::new(ErrorType::FloatType, self))
+        } else if let Ok(float) = self.extract::<f64>() {
+            Ok(float)
+        } else {
+            Err(ValError::new(ErrorType::FloatType, self))
+        }
+    }
     fn strict_float(&self) -> ValResult<f64> {
         if self.extract::<bool>().is_ok() {
             Err(ValError::new(ErrorType::FloatType, self))
@@ -335,7 +330,6 @@ impl<'a> Input<'a> for PyAny {
             Err(ValError::new(ErrorType::FloatType, self))
         }
     }
-
     fn lax_float(&self) -> ValResult<f64> {
         if let Ok(float) = self.extract::<f64>() {
             Ok(float)
@@ -367,7 +361,7 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn validate_typed_dict(&'a self, strict: bool, from_attributes: bool) -> ValResult<GenericMapping<'a>> {
+    fn validate_model_fields(&'a self, strict: bool, from_attributes: bool) -> ValResult<GenericMapping<'a>> {
         if from_attributes {
             // if from_attributes, first try a dict, then mapping then from_attributes
             if let Ok(dict) = self.downcast::<PyDict>() {
@@ -391,107 +385,124 @@ impl<'a> Input<'a> for PyAny {
                 Err(ValError::new(ErrorType::DictAttributesType, self))
             }
         } else {
-            // otherwise we just call back to lax_dict if from_mapping is allowed, not there error in this
+            // otherwise we just call back to validate_dict if from_mapping is allowed, note that errors in this
             // case (correctly) won't hint about from_attributes
             self.validate_dict(strict)
         }
     }
 
-    fn strict_list(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else {
-            Err(ValError::new(ErrorType::ListType, self))
+    fn strict_list(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self.lax_list()? {
+            GenericIterable::List(iter) => Ok(GenericIterable::List(iter)),
+            _ => Err(ValError::new(ErrorType::ListType, self)),
         }
     }
 
-    fn lax_list(&'a self, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
-        if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Some(collection) = extract_dict_iter!(self) {
-            Ok(collection)
-        } else if allow_any_iter && self.iter().is_ok() {
-            Ok(self.into())
-        } else if let Some(collection) = extract_shared_iter!(PyList, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::ListType, self))
+    fn lax_list(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self
+            .extract_generic_iterable()
+            .map_err(|_| ValError::new(ErrorType::ListType, self))?
+        {
+            GenericIterable::PyString(_)
+            | GenericIterable::Bytes(_)
+            | GenericIterable::Dict(_)
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::ListType, self)),
+            other => Ok(other),
         }
     }
 
-    fn strict_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else {
-            Err(ValError::new(ErrorType::TupleType, self))
+    fn strict_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self.lax_tuple()? {
+            GenericIterable::Tuple(iter) => Ok(GenericIterable::Tuple(iter)),
+            _ => Err(ValError::new(ErrorType::TupleType, self)),
         }
     }
 
-    fn lax_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Some(collection) = extract_dict_iter!(self) {
-            Ok(collection)
-        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::TupleType, self))
+    fn lax_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self
+            .extract_generic_iterable()
+            .map_err(|_| ValError::new(ErrorType::TupleType, self))?
+        {
+            GenericIterable::PyString(_)
+            | GenericIterable::Bytes(_)
+            | GenericIterable::Dict(_)
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::TupleType, self)),
+            other => Ok(other),
         }
     }
 
-    fn strict_set(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(set) = self.downcast::<PySet>() {
-            Ok(set.into())
-        } else {
-            Err(ValError::new(ErrorType::SetType, self))
+    fn strict_set(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self.lax_set()? {
+            GenericIterable::Set(iter) => Ok(GenericIterable::Set(iter)),
+            _ => Err(ValError::new(ErrorType::SetType, self)),
         }
     }
 
-    fn lax_set(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(set) = self.downcast::<PySet>() {
-            Ok(set.into())
-        } else if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Ok(frozen_set) = self.downcast::<PyFrozenSet>() {
-            Ok(frozen_set.into())
-        } else if let Some(collection) = extract_dict_iter!(self) {
-            Ok(collection)
-        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::SetType, self))
+    fn lax_set(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self
+            .extract_generic_iterable()
+            .map_err(|_| ValError::new(ErrorType::SetType, self))?
+        {
+            GenericIterable::PyString(_)
+            | GenericIterable::Bytes(_)
+            | GenericIterable::Dict(_)
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::SetType, self)),
+            other => Ok(other),
         }
     }
 
-    fn strict_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(set) = self.downcast::<PyFrozenSet>() {
-            Ok(set.into())
-        } else {
-            Err(ValError::new(ErrorType::FrozenSetType, self))
+    fn strict_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self.lax_frozenset()? {
+            GenericIterable::FrozenSet(iter) => Ok(GenericIterable::FrozenSet(iter)),
+            _ => Err(ValError::new(ErrorType::FrozenSetType, self)),
         }
     }
 
-    fn lax_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(frozen_set) = self.downcast::<PyFrozenSet>() {
-            Ok(frozen_set.into())
-        } else if let Ok(set) = self.downcast::<PySet>() {
-            Ok(set.into())
-        } else if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Some(collection) = extract_dict_iter!(self) {
-            Ok(collection)
-        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
-            Ok(collection)
+    fn lax_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
+        match self
+            .extract_generic_iterable()
+            .map_err(|_| ValError::new(ErrorType::FrozenSetType, self))?
+        {
+            GenericIterable::PyString(_)
+            | GenericIterable::Bytes(_)
+            | GenericIterable::Dict(_)
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::FrozenSetType, self)),
+            other => Ok(other),
+        }
+    }
+
+    fn extract_generic_iterable(&'a self) -> ValResult<GenericIterable<'a>> {
+        // Handle concrete non-overlapping types first, then abstract types
+        if let Ok(iterable) = self.downcast::<PyList>() {
+            Ok(GenericIterable::List(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyTuple>() {
+            Ok(GenericIterable::Tuple(iterable))
+        } else if let Ok(iterable) = self.downcast::<PySet>() {
+            Ok(GenericIterable::Set(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyFrozenSet>() {
+            Ok(GenericIterable::FrozenSet(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyDict>() {
+            Ok(GenericIterable::Dict(iterable))
+        } else if let Some(iterable) = extract_dict_keys!(self.py(), self) {
+            Ok(GenericIterable::DictKeys(iterable))
+        } else if let Some(iterable) = extract_dict_values!(self.py(), self) {
+            Ok(GenericIterable::DictValues(iterable))
+        } else if let Some(iterable) = extract_dict_items!(self.py(), self) {
+            Ok(GenericIterable::DictItems(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyMapping>() {
+            Ok(GenericIterable::Mapping(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyString>() {
+            Ok(GenericIterable::PyString(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyBytes>() {
+            Ok(GenericIterable::Bytes(iterable))
+        } else if let Ok(iterable) = self.downcast::<PyByteArray>() {
+            Ok(GenericIterable::PyByteArray(iterable))
+        } else if let Ok(iterable) = self.downcast::<PySequence>() {
+            Ok(GenericIterable::Sequence(iterable))
+        } else if let Ok(iterable) = self.iter() {
+            Ok(GenericIterable::Iterator(iterable))
         } else {
-            Err(ValError::new(ErrorType::FrozenSetType, self))
+            Err(ValError::new(ErrorType::IterableType, self))
         }
     }
 
@@ -645,21 +656,6 @@ fn maybe_as_string(v: &PyAny, unicode_error: ErrorType) -> ValResult<Option<Cow<
     }
 }
 
-static DEQUE_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
-
-fn is_deque(v: &PyAny) -> bool {
-    let py = v.py();
-    let deque_type = DEQUE_TYPE
-        .get_or_init(py, || import_type(py, "collections", "deque").unwrap())
-        .as_ref(py);
-    v.is_instance(deque_type).unwrap_or(false)
-}
-
-fn import_type(py: Python, module: &str, attr: &str) -> PyResult<Py<PyType>> {
-    let obj = py.import(module)?.getattr(attr)?;
-    Ok(obj.downcast::<PyType>()?.into())
-}
-
 fn is_builtin_str(py_str: &PyString) -> bool {
     py_str.get_type().is(PyString::type_object(py_str.py()))
 }
@@ -726,3 +722,15 @@ pub fn list_as_tuple(list: &PyList) -> &PyTuple {
     };
     py_tuple.into_ref(list.py())
 }
+
+static PY_INT_TYPE: GILOnceCell<PyObject> = GILOnceCell::new();
+
+fn get_py_int_type(py: Python) -> &PyObject {
+    PY_INT_TYPE.get_or_init(py, || PyInt::type_object(py).into())
+}
+
+static PY_STR_TYPE: GILOnceCell<PyObject> = GILOnceCell::new();
+
+fn get_py_str_type(py: Python) -> &PyObject {
+    PY_STR_TYPE.get_or_init(py, || PyString::type_object(py).into())
+}
