@@ -9,13 +9,13 @@ use pyo3::prelude::*;
 #[global_allocator]
 static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
 
-mod args_kwargs;
-mod build_context;
+mod argument_markers;
 mod build_tools;
+mod definitions;
 mod errors;
 mod input;
+mod lazy_index_map;
 mod lookup_key;
-mod questions;
 mod recursion_guard;
 mod serializers;
 mod url;
@@ -23,7 +23,7 @@ mod validators;
 
 // required for benchmarks
 pub use self::url::{PyMultiHostUrl, PyUrl};
-pub use args_kwargs::ArgsKwargs;
+pub use argument_markers::ArgsKwargs;
 pub use build_tools::SchemaError;
 pub use errors::{list_all_errors, PydanticCustomError, PydanticKnownError, PydanticOmit, ValidationError};
 pub use serializers::{
@@ -32,7 +32,7 @@ pub use serializers::{
 pub use validators::SchemaValidator;
 
 pub fn get_version() -> String {
-    let version = env!("CARGO_PKG_VERSION").to_string();
+    let version = env!("CARGO_PKG_VERSION");
     // cargo uses "1.0-alpha1" etc. while python uses "1.0.0a1", this is not full compatibility,
     // but it's good enough for now
     // see https://docs.rs/semver/1.0.9/semver/struct.Version.html#method.parse for rust spec
