@@ -12,6 +12,7 @@ use super::config::SerializationConfig;
 use super::errors::{PydanticSerializationUnexpectedValue, UNEXPECTED_TYPE_SER_MARKER};
 use super::ob_type::ObTypeLookup;
 use super::shared::CombinedSerializer;
+use crate::definitions::Definitions;
 
 /// this is ugly, would be much better if extra could be stored in `SerializationState`
 /// then `SerializationState` got a `serialize_infer` method, but I couldn't get it to work
@@ -23,7 +24,7 @@ pub(crate) struct SerializationState {
 
 impl SerializationState {
     pub fn new(timedelta_mode: Option<&str>, bytes_mode: Option<&str>) -> Self {
-        let warnings = CollectWarnings::new(None);
+        let warnings = CollectWarnings::new(false);
         let rec_guard = SerRecursionGuard::default();
         let config = SerializationConfig::from_args(timedelta_mode, bytes_mode).unwrap();
         Self {
@@ -33,23 +34,25 @@ impl SerializationState {
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn extra<'py>(
         &'py self,
         py: Python<'py>,
         mode: &'py SerMode,
-        exclude_none: Option<bool>,
-        round_trip: Option<bool>,
-        serialize_unknown: Option<bool>,
+        by_alias: bool,
+        exclude_none: bool,
+        round_trip: bool,
+        serialize_unknown: bool,
         fallback: Option<&'py PyAny>,
     ) -> Extra<'py> {
         Extra::new(
             py,
             mode,
             &[],
-            None,
+            by_alias,
             &self.warnings,
-            None,
-            None,
+            false,
+            false,
             exclude_none,
             round_trip,
             &self.config,
@@ -69,7 +72,7 @@ impl SerializationState {
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub(crate) struct Extra<'a> {
     pub mode: &'a SerMode,
-    pub slots: &'a [CombinedSerializer],
+    pub definitions: &'a Definitions<CombinedSerializer>,
     pub ob_type_lookup: &'a ObTypeLookup,
     pub warnings: &'a CollectWarnings,
     pub by_alias: bool,
@@ -95,34 +98,34 @@ impl<'a> Extra<'a> {
     pub fn new(
         py: Python<'a>,
         mode: &'a SerMode,
-        slots: &'a [CombinedSerializer],
-        by_alias: Option<bool>,
+        definitions: &'a Definitions<CombinedSerializer>,
+        by_alias: bool,
         warnings: &'a CollectWarnings,
-        exclude_unset: Option<bool>,
-        exclude_defaults: Option<bool>,
-        exclude_none: Option<bool>,
-        round_trip: Option<bool>,
+        exclude_unset: bool,
+        exclude_defaults: bool,
+        exclude_none: bool,
+        round_trip: bool,
         config: &'a SerializationConfig,
         rec_guard: &'a SerRecursionGuard,
-        serialize_unknown: Option<bool>,
+        serialize_unknown: bool,
         fallback: Option<&'a PyAny>,
     ) -> Self {
         Self {
             mode,
-            slots,
+            definitions,
             ob_type_lookup: ObTypeLookup::cached(py),
             warnings,
-            by_alias: by_alias.unwrap_or(true),
-            exclude_unset: exclude_unset.unwrap_or(false),
-            exclude_defaults: exclude_defaults.unwrap_or(false),
-            exclude_none: exclude_none.unwrap_or(false),
-            round_trip: round_trip.unwrap_or(false),
+            by_alias,
+            exclude_unset,
+            exclude_defaults,
+            exclude_none,
+            round_trip,
             config,
             rec_guard,
             check: SerCheck::None,
             model: None,
             field_name: None,
-            serialize_unknown: serialize_unknown.unwrap_or(false),
+            serialize_unknown,
             fallback,
         }
     }
@@ -153,7 +156,7 @@ impl SerCheck {
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub(crate) struct ExtraOwned {
     mode: SerMode,
-    slots: Vec<CombinedSerializer>,
+    definitions: Vec<CombinedSerializer>,
     warnings: CollectWarnings,
     by_alias: bool,
     exclude_unset: bool,
@@ -173,7 +176,7 @@ impl ExtraOwned {
     pub fn new(extra: &Extra) -> Self {
         Self {
             mode: extra.mode.clone(),
-            slots: extra.slots.to_vec(),
+            definitions: extra.definitions.to_vec(),
             warnings: extra.warnings.clone(),
             by_alias: extra.by_alias,
             exclude_unset: extra.exclude_unset,
@@ -193,7 +196,7 @@ impl ExtraOwned {
     pub fn to_extra<'py>(&'py self, py: Python<'py>) -> Extra<'py> {
         Extra {
             mode: &self.mode,
-            slots: &self.slots,
+            definitions: &self.definitions,
             ob_type_lookup: ObTypeLookup::cached(py),
             warnings: &self.warnings,
             by_alias: self.by_alias,
@@ -265,9 +268,9 @@ pub(crate) struct CollectWarnings {
 }
 
 impl CollectWarnings {
-    pub(crate) fn new(active: Option<bool>) -> Self {
+    pub(crate) fn new(active: bool) -> Self {
         Self {
-            active: active.unwrap_or(true),
+            active,
             warnings: RefCell::new(None),
         }
     }
@@ -349,7 +352,7 @@ impl CollectWarnings {
 #[derive(Default, Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub struct RecursionInfo {
-    ids: AHashSet<usize>,
+    ids: AHashSet<(usize, usize)>, // first element is the object's id, the second is the serializer's id
     /// as with `src/recursion_guard.rs` this is used as a backup in case the identity check recursion guard fails
     /// see #143
     depth: u16,
@@ -364,12 +367,12 @@ pub struct SerRecursionGuard {
 impl SerRecursionGuard {
     const MAX_DEPTH: u16 = 200;
 
-    pub fn add(&self, value: &PyAny) -> PyResult<usize> {
+    pub fn add(&self, value: &PyAny, def_ref_id: usize) -> PyResult<usize> {
         // https://doc.rust-lang.org/std/collections/struct.HashSet.html#method.insert
         // "If the set did not have this value present, `true` is returned."
         let id = value.as_ptr() as usize;
         let mut info = self.info.borrow_mut();
-        if !info.ids.insert(id) {
+        if !info.ids.insert((id, def_ref_id)) {
             Err(PyValueError::new_err("Circular reference detected (id repeated)"))
         } else if info.depth > Self::MAX_DEPTH {
             Err(PyValueError::new_err("Circular reference detected (depth exceeded)"))
@@ -379,9 +382,9 @@ impl SerRecursionGuard {
         }
     }
 
-    pub fn pop(&self, id: usize) {
+    pub fn pop(&self, id: usize, def_ref_id: usize) {
         let mut info = self.info.borrow_mut();
         info.depth -= 1;
-        info.ids.remove(&id);
+        info.ids.remove(&(id, def_ref_id));
     }
 }
