@@ -13,6 +13,8 @@ use serde::ser::{Error, Serialize, SerializeMap, SerializeSeq, Serializer};
 use crate::build_tools::{py_err, safe_repr};
 use crate::serializers::errors::SERIALIZATION_ERR_MARKER;
 use crate::serializers::filter::SchemaFilter;
+use crate::serializers::shared::{PydanticSerializer, TypeSerializer};
+use crate::serializers::SchemaSerializer;
 use crate::url::{PyMultiHostUrl, PyUrl};
 
 use super::errors::{py_err_se_err, PydanticSerializationError};
@@ -30,6 +32,10 @@ pub(crate) fn infer_to_python(
     infer_to_python_known(&extra.ob_type_lookup.get_type(value), value, include, exclude, extra)
 }
 
+// arbitrary ids to identify that we recursed through infer_to_{python,json}_known
+// We just need them to be different from definition ref slot ids, which start at 0
+const INFER_DEF_REF_ID: usize = usize::MAX;
+
 pub(crate) fn infer_to_python_known(
     ob_type: &ObType,
     value: &PyAny,
@@ -38,7 +44,7 @@ pub(crate) fn infer_to_python_known(
     extra: &Extra,
 ) -> PyResult<PyObject> {
     let py = value.py();
-    let value_id = match extra.rec_guard.add(value) {
+    let value_id = match extra.rec_guard.add(value, INFER_DEF_REF_ID) {
         Ok(id) => id,
         Err(e) => {
             return match extra.mode {
@@ -91,6 +97,31 @@ pub(crate) fn infer_to_python_known(
         Ok::<PyObject, PyErr>(new_dict.into_py(py))
     };
 
+    let serialize_with_serializer = |value: &PyAny, is_model: bool| {
+        if let Ok(py_serializer) = value.getattr(intern!(py, "__pydantic_serializer__")) {
+            if let Ok(serializer) = py_serializer.extract::<SchemaSerializer>() {
+                let extra = serializer.build_extra(
+                    py,
+                    extra.mode,
+                    extra.by_alias,
+                    extra.warnings,
+                    extra.exclude_unset,
+                    extra.exclude_defaults,
+                    extra.exclude_none,
+                    extra.round_trip,
+                    extra.rec_guard,
+                    extra.serialize_unknown,
+                    extra.fallback,
+                );
+                return serializer.serializer.to_python(value, include, exclude, &extra);
+            }
+        }
+        // Fallback to dict serialization if `__pydantic_serializer__` is not set.
+        // This currently only affects non-pydantic dataclasses.
+        let dict = object_to_dict(value, is_model, extra)?;
+        serialize_dict(dict)
+    };
+
     let value = match extra.mode {
         SerMode::Json => match ob_type {
             // `bool` and `None` can't be subclasses, `ObType::Int`, `ObType::Float`, `ObType::Str` refer to exact types
@@ -160,8 +191,8 @@ pub(crate) fn infer_to_python_known(
                 let py_url: PyMultiHostUrl = value.extract()?;
                 py_url.__str__().into_py(py)
             }
-            ObType::Dataclass => serialize_dict(object_to_dict(value, false, extra)?)?,
-            ObType::Model => serialize_dict(object_to_dict(value, true, extra)?)?,
+            ObType::PydanticSerializable => serialize_with_serializer(value, true)?,
+            ObType::Dataclass => serialize_with_serializer(value, false)?,
             ObType::Enum => {
                 let v = value.getattr(intern!(py, "value"))?;
                 infer_to_python(v, include, exclude, extra)?.into_py(py)
@@ -185,7 +216,7 @@ pub(crate) fn infer_to_python_known(
                 if let Some(fallback) = extra.fallback {
                     let next_value = fallback.call1((value,))?;
                     let next_result = infer_to_python(next_value, include, exclude, extra);
-                    extra.rec_guard.pop(value_id);
+                    extra.rec_guard.pop(value_id, INFER_DEF_REF_ID);
                     return next_result;
                 } else if extra.serialize_unknown {
                     serialize_unknown(value).into_py(py)
@@ -226,8 +257,8 @@ pub(crate) fn infer_to_python_known(
                 }
                 new_dict.into_py(py)
             }
-            ObType::Dataclass => serialize_dict(object_to_dict(value, false, extra)?)?,
-            ObType::Model => serialize_dict(object_to_dict(value, true, extra)?)?,
+            ObType::PydanticSerializable => serialize_with_serializer(value, true)?,
+            ObType::Dataclass => serialize_with_serializer(value, false)?,
             ObType::Generator => {
                 let iter = super::type_serializers::generator::SerializationIterator::new(
                     value.downcast()?,
@@ -243,7 +274,7 @@ pub(crate) fn infer_to_python_known(
                 if let Some(fallback) = extra.fallback {
                     let next_value = fallback.call1((value,))?;
                     let next_result = infer_to_python(next_value, include, exclude, extra);
-                    extra.rec_guard.pop(value_id);
+                    extra.rec_guard.pop(value_id, INFER_DEF_REF_ID);
                     return next_result;
                 } else {
                     value.into_py(py)
@@ -252,7 +283,7 @@ pub(crate) fn infer_to_python_known(
             _ => value.into_py(py),
         },
     };
-    extra.rec_guard.pop(value_id);
+    extra.rec_guard.pop(value_id, INFER_DEF_REF_ID);
     Ok(value)
 }
 
@@ -311,7 +342,7 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
     exclude: Option<&PyAny>,
     extra: &Extra,
 ) -> Result<S::Ok, S::Error> {
-    let value_id = match extra.rec_guard.add(value).map_err(py_err_se_err) {
+    let value_id = match extra.rec_guard.add(value, INFER_DEF_REF_ID).map_err(py_err_se_err) {
         Ok(v) => v,
         Err(e) => {
             return if extra.serialize_unknown {
@@ -375,6 +406,36 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
         }};
     }
 
+    macro_rules! serialize_with_serializer {
+        ($py_serializable:expr, $is_model:expr) => {{
+            let py = $py_serializable.py();
+            if let Ok(py_serializer) = value.getattr(intern!(py, "__pydantic_serializer__")) {
+                if let Ok(extracted_serializer) = py_serializer.extract::<SchemaSerializer>() {
+                    let extra = extracted_serializer.build_extra(
+                        py,
+                        extra.mode,
+                        extra.by_alias,
+                        extra.warnings,
+                        extra.exclude_unset,
+                        extra.exclude_defaults,
+                        extra.exclude_none,
+                        extra.round_trip,
+                        extra.rec_guard,
+                        extra.serialize_unknown,
+                        extra.fallback,
+                    );
+                    let pydantic_serializer =
+                        PydanticSerializer::new(value, &extracted_serializer.serializer, include, exclude, &extra);
+                    return pydantic_serializer.serialize(serializer);
+                }
+            }
+            // Fallback to dict serialization if `__pydantic_serializer__` is not set.
+            // This currently only affects non-pydantic dataclasses.
+            let dict = object_to_dict(value, $is_model, extra).map_err(py_err_se_err)?;
+            serialize_dict!(dict)
+        }};
+    }
+
     let ser_result = match ob_type {
         ObType::None => serializer.serialize_none(),
         ObType::Int | ObType::IntSubclass => serialize!(i64),
@@ -429,8 +490,8 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
             let py_url: PyMultiHostUrl = value.extract().map_err(py_err_se_err)?;
             serializer.serialize_str(&py_url.__str__())
         }
-        ObType::Dataclass => serialize_dict!(object_to_dict(value, false, extra).map_err(py_err_se_err)?),
-        ObType::Model => serialize_dict!(object_to_dict(value, true, extra).map_err(py_err_se_err)?),
+        ObType::Dataclass => serialize_with_serializer!(value, false),
+        ObType::PydanticSerializable => serialize_with_serializer!(value, true),
         ObType::Enum => {
             let v = value.getattr(intern!(value.py(), "value")).map_err(py_err_se_err)?;
             infer_serialize(v, serializer, include, exclude, extra)
@@ -457,7 +518,7 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
             if let Some(fallback) = extra.fallback {
                 let next_value = fallback.call1((value,)).map_err(py_err_se_err)?;
                 let next_result = infer_serialize(next_value, serializer, include, exclude, extra);
-                extra.rec_guard.pop(value_id);
+                extra.rec_guard.pop(value_id, INFER_DEF_REF_ID);
                 return next_result;
             } else if extra.serialize_unknown {
                 serializer.serialize_str(&serialize_unknown(value))
@@ -471,7 +532,7 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
             }
         }
     };
-    extra.rec_guard.pop(value_id);
+    extra.rec_guard.pop(value_id, INFER_DEF_REF_ID);
     ser_result
 }
 
@@ -552,7 +613,7 @@ pub(crate) fn infer_json_key_known<'py>(ob_type: &ObType, key: &'py PyAny, extra
         ObType::List | ObType::Set | ObType::Frozenset | ObType::Dict | ObType::Generator => {
             py_err!(PyTypeError; "`{}` not valid as object key", ob_type)
         }
-        ObType::Dataclass | ObType::Model => {
+        ObType::Dataclass | ObType::PydanticSerializable => {
             // check that the instance is hashable
             key.hash()?;
             let key = key.str()?.to_string();
