@@ -10,13 +10,14 @@ use enum_dispatch::enum_dispatch;
 use serde::Serialize;
 use serde_json::ser::PrettyFormatter;
 
-use crate::build_context::BuildContext;
 use crate::build_tools::{py_err, py_error_type, SchemaDict};
+use crate::definitions::DefinitionsBuilder;
 
 use super::errors::se_err_py_err;
 use super::extra::Extra;
 use super::infer::infer_json_key;
 use super::ob_type::{IsType, ObType};
+use super::type_serializers::definitions::DefinitionRefSerializer;
 
 pub(crate) trait BuildSerializer: Sized {
     const EXPECTED_TYPE: &'static str;
@@ -24,7 +25,7 @@ pub(crate) trait BuildSerializer: Sized {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedSerializer>,
+        definitions: &mut DefinitionsBuilder<CombinedSerializer>,
     ) -> PyResult<CombinedSerializer>;
 }
 
@@ -47,17 +48,17 @@ macro_rules! combined_serializer {
                 lookup_type: &str,
                 schema: &PyDict,
                 config: Option<&PyDict>,
-                build_context: &mut BuildContext<CombinedSerializer>
+                definitions: &mut DefinitionsBuilder<CombinedSerializer>
             ) -> PyResult<CombinedSerializer> {
                 match lookup_type {
                     $(
-                        <$b_serializer>::EXPECTED_TYPE => match <$b_serializer>::build(schema, config, build_context) {
+                        <$b_serializer>::EXPECTED_TYPE => match <$b_serializer>::build(schema, config, definitions) {
                             Ok(serializer) => Ok(serializer),
                             Err(err) => py_err!("Error building `{}` serializer:\n  {}", lookup_type, err),
                         },
                     )*
                     $(
-                        <$builder>::EXPECTED_TYPE => match <$builder>::build(schema, config, build_context) {
+                        <$builder>::EXPECTED_TYPE => match <$builder>::build(schema, config, definitions) {
                             Ok(serializer) => Ok(serializer),
                             Err(err) => py_err!("Error building `{}` serializer:\n  {}", lookup_type, err),
                         },
@@ -78,6 +79,7 @@ combined_serializer! {
         // hence they're here.
         Function: super::type_serializers::function::FunctionPlainSerializer;
         FunctionWrap: super::type_serializers::function::FunctionWrapSerializer;
+        Fields: super::fields::GeneralFieldsSerializer;
     }
     // `find_only` is for type_serializers which are built directly via the `type` key and `find_serializer`
     // but aren't actually used for serialization, e.g. their `build` method must return another serializer
@@ -91,13 +93,15 @@ combined_serializer! {
         super::type_serializers::other::IsInstanceBuilder;
         super::type_serializers::other::IsSubclassBuilder;
         super::type_serializers::other::CallableBuilder;
-        super::type_serializers::definitions::DefinitionsBuilder;
+        super::type_serializers::definitions::DefinitionsSerializerBuilder;
         super::type_serializers::dataclass::DataclassArgsBuilder;
         super::type_serializers::dataclass::DataclassBuilder;
         super::type_serializers::function::FunctionBeforeSerializerBuilder;
         super::type_serializers::function::FunctionAfterSerializerBuilder;
         super::type_serializers::function::FunctionPlainSerializerBuilder;
         super::type_serializers::function::FunctionWrapSerializerBuilder;
+        super::type_serializers::model::ModelFieldsBuilder;
+        super::type_serializers::typed_dict::TypedDictBuilder;
     }
     // `both` means the struct is added to both the `CombinedSerializer` enum and the match statement in
     // `find_serializer` so they can be used via a `type` str.
@@ -118,7 +122,6 @@ combined_serializer! {
         FrozenSet: super::type_serializers::set_frozenset::FrozenSetSerializer;
         Generator: super::type_serializers::generator::GeneratorSerializer;
         Dict: super::type_serializers::dict::DictSerializer;
-        TypedDict: super::type_serializers::typed_dict::TypedDictSerializer;
         Model: super::type_serializers::model::ModelSerializer;
         Url: super::type_serializers::url::UrlSerializer;
         MultiHostUrl: super::type_serializers::url::MultiHostUrlSerializer;
@@ -127,6 +130,7 @@ combined_serializer! {
         ToString: super::type_serializers::format::ToStringSerializer;
         WithDefault: super::type_serializers::with_default::WithDefaultSerializer;
         Json: super::type_serializers::json::JsonSerializer;
+        JsonOrPython: super::type_serializers::json_or_python::JsonOrPythonSerializer;
         Union: super::type_serializers::union::UnionSerializer;
         Literal: super::type_serializers::literal::LiteralSerializer;
         Recursive: super::type_serializers::definitions::DefinitionRefSerializer;
@@ -139,7 +143,7 @@ impl CombinedSerializer {
     fn _build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedSerializer>,
+        definitions: &mut DefinitionsBuilder<CombinedSerializer>,
     ) -> PyResult<CombinedSerializer> {
         let py = schema.py();
         let type_key = intern!(py, "type");
@@ -154,7 +158,7 @@ impl CombinedSerializer {
                     return super::type_serializers::function::FunctionPlainSerializer::build(
                         schema,
                         config,
-                        build_context,
+                        definitions,
                     )
                     .map_err(|err| py_error_type!("Error building `function-plain` serializer:\n  {}", err));
                 }
@@ -165,7 +169,7 @@ impl CombinedSerializer {
                     return super::type_serializers::function::FunctionWrapSerializer::build(
                         schema,
                         config,
-                        build_context,
+                        definitions,
                     )
                     .map_err(|err| py_error_type!("Error building `function-wrap` serializer:\n  {}", err));
                 }
@@ -176,7 +180,7 @@ impl CombinedSerializer {
                 Some(ser_type) => {
                     // otherwise if `schema.serialization.type` is defined, use that with `find_serializer`
                     // instead of `schema.type`. In this case it's an error if a serializer isn't found.
-                    return Self::find_serializer(ser_type, ser_schema, config, build_context);
+                    return Self::find_serializer(ser_type, ser_schema, config, definitions);
                 }
                 // if `schema.serialization.type` is None, fall back to `schema.type`
                 None => (),
@@ -184,7 +188,7 @@ impl CombinedSerializer {
         }
 
         let type_: &str = schema.get_as_req(type_key)?;
-        Self::find_serializer(type_, schema, config, build_context)
+        Self::find_serializer(type_, schema, config, definitions)
     }
 }
 
@@ -195,41 +199,16 @@ impl BuildSerializer for CombinedSerializer {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedSerializer>,
+        definitions: &mut DefinitionsBuilder<CombinedSerializer>,
     ) -> PyResult<CombinedSerializer> {
-        if let Some(schema_ref) = schema.get_as::<String>(intern!(schema.py(), "ref"))? {
-            // as with validators, if there's a ref,
-            // we **might** want to store the serializer in slots and return a DefinitionRefSerializer:
-            // * if the ref isn't used at all, we just want to return a normal serializer, and ignore the ref completely
-            // * if the ref is used inside itself, we have to store the serializer in slots,
-            //   and return a DefinitionRefSerializer - two step process with `prepare_slot` and `complete_slot`
-            // * if the ref is used elsewhere, we want to clone it each time it's used
-            if build_context.ref_used(&schema_ref) {
-                // the ref is used somewhere
-                // check the ref is unique
-                if build_context.ref_already_used(&schema_ref) {
-                    return py_err!("Duplicate ref: `{}`", schema_ref);
-                }
-
-                return if build_context.ref_used_within(schema, &schema_ref)? {
-                    // the ref is used within itself, so we have to store the serializer in slots
-                    // and return a DefinitionRefSerializer
-                    let slot_id = build_context.prepare_slot(schema_ref, None)?;
-                    let inner_ser = Self::_build(schema, config, build_context)?;
-                    build_context.complete_slot(slot_id, inner_ser)?;
-                    Ok(super::type_serializers::definitions::DefinitionRefSerializer::from_id(
-                        slot_id,
-                    ))
-                } else {
-                    // the ref is used elsewhere, so we want to clone it each time it's used
-                    let serializer = Self::_build(schema, config, build_context)?;
-                    build_context.store_reusable(schema_ref, serializer.clone());
-                    Ok(serializer)
-                };
-            }
+        let py: Python = schema.py();
+        if let Some(schema_ref) = schema.get_as::<String>(intern!(py, "ref"))? {
+            let inner_ser = Self::_build(schema, config, definitions)?;
+            let ser_id = definitions.add_definition(schema_ref, inner_ser)?;
+            return Ok(DefinitionRefSerializer::from_id(ser_id));
         }
 
-        Self::_build(schema, config, build_context)
+        Self::_build(schema, config, definitions)
     }
 }
 
@@ -353,7 +332,7 @@ pub(super) fn object_to_dict<'py>(value: &'py PyAny, is_model: bool, extra: &Ext
     let attr = value.getattr(intern!(py, "__dict__"))?;
     let attrs: &PyDict = attr.downcast()?;
     if is_model && extra.exclude_unset {
-        let fields_set: &PySet = value.getattr(intern!(py, "__fields_set__"))?.downcast()?;
+        let fields_set: &PySet = value.getattr(intern!(py, "__pydantic_fields_set__"))?.downcast()?;
 
         let new_attrs = attrs.copy()?;
         for key in new_attrs.keys() {
