@@ -8,6 +8,7 @@ pub mod format;
 pub mod function;
 pub mod generator;
 pub mod json;
+pub mod json_or_python;
 pub mod list;
 pub mod literal;
 pub mod model;
@@ -23,13 +24,15 @@ pub mod union;
 pub mod url;
 pub mod with_default;
 
+pub(self) use super::computed_fields::ComputedFields;
 pub(self) use super::config::utf8_py_error;
 pub(self) use super::errors::{py_err_se_err, PydanticSerializationError};
-pub(self) use super::extra::{Extra, ExtraOwned, SerMode};
-pub(self) use super::filter::SchemaFilter;
+pub(self) use super::extra::{Extra, ExtraOwned, SerCheck, SerMode};
+pub(self) use super::fields::{FieldsMode, GeneralFieldsSerializer, SerField};
+pub(self) use super::filter::{AnyFilter, SchemaFilter};
 pub(self) use super::infer::{
     infer_json_key, infer_json_key_known, infer_serialize, infer_serialize_known, infer_to_python,
-    infer_to_python_known, SerializeInfer,
+    infer_to_python_known,
 };
 pub(self) use super::ob_type::{IsType, ObType};
 pub(self) use super::shared::{
