@@ -5,10 +5,9 @@ use pyo3::types::{PyDict, PyList};
 use crate::build_tools::{py_err, SchemaDict};
 use crate::errors::ValResult;
 use crate::input::Input;
-use crate::questions::Question;
 use crate::recursion_guard::RecursionGuard;
 
-use super::{build_validator, BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
+use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
 
 #[derive(Debug, Clone)]
 pub struct ChainValidator {
@@ -22,12 +21,12 @@ impl BuildValidator for ChainValidator {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedValidator>,
+        definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let steps: Vec<CombinedValidator> = schema
             .get_as_req::<&PyList>(intern!(schema.py(), "steps"))?
             .iter()
-            .map(|step| build_validator_steps(step, config, build_context))
+            .map(|step| build_validator_steps(step, config, definitions))
             .collect::<PyResult<Vec<Vec<CombinedValidator>>>>()?
             .into_iter()
             .flatten()
@@ -57,9 +56,9 @@ impl BuildValidator for ChainValidator {
 fn build_validator_steps<'a>(
     step: &'a PyAny,
     config: Option<&'a PyDict>,
-    build_context: &mut BuildContext<CombinedValidator>,
+    definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<Vec<CombinedValidator>> {
-    let validator = build_validator(step, config, build_context)?;
+    let validator = build_validator(step, config, definitions)?;
     if let CombinedValidator::Chain(chain_validator) = validator {
         Ok(chain_validator.steps)
     } else {
@@ -73,29 +72,33 @@ impl Validator for ChainValidator {
         py: Python<'data>,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
         let mut steps_iter = self.steps.iter();
         let first_step = steps_iter.next().unwrap();
-        let value = first_step.validate(py, input, extra, slots, recursion_guard)?;
+        let value = first_step.validate(py, input, extra, definitions, recursion_guard)?;
 
         steps_iter.try_fold(value, |v, step| {
-            step.validate(py, v.into_ref(py), extra, slots, recursion_guard)
+            step.validate(py, v.into_ref(py), extra, definitions, recursion_guard)
         })
     }
 
+    fn different_strict_behavior(
+        &self,
+        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
+        ultra_strict: bool,
+    ) -> bool {
+        self.steps
+            .iter()
+            .any(|v| v.different_strict_behavior(definitions, ultra_strict))
+    }
+
     fn get_name(&self) -> &str {
         &self.name
     }
 
-    fn ask(&self, question: &Question) -> bool {
-        // any makes more sense since at the moment we only use ask for "return_fields_set", might need
-        // more complex logic in future
-        self.steps.iter().any(|v| v.ask(question))
-    }
-
-    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
-        self.steps.iter_mut().try_for_each(|v| v.complete(build_context))
+    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
+        self.steps.iter_mut().try_for_each(|v| v.complete(definitions))
     }
 }
