@@ -8,11 +8,10 @@ use pyo3::prelude::*;
 use pyo3::types::{PyAny, PyDict};
 use pyo3::{intern, PyTraverseError, PyVisit};
 
-use crate::build_context::BuildContext;
 use crate::build_tools::{py_err, py_error_type, SchemaDict, SchemaError};
+use crate::definitions::{Definitions, DefinitionsBuilder};
 use crate::errors::{ErrorMode, LocItem, ValError, ValResult, ValidationError};
-use crate::input::Input;
-use crate::questions::{Answers, Question};
+use crate::input::{Input, InputType};
 use crate::recursion_guard::RecursionGuard;
 
 mod any;
@@ -36,10 +35,12 @@ mod int;
 mod is_instance;
 mod is_subclass;
 mod json;
+mod json_or_python;
 mod lax_or_strict;
 mod list;
 mod literal;
 mod model;
+mod model_fields;
 mod none;
 mod nullable;
 mod set;
@@ -54,11 +55,13 @@ mod with_default;
 
 pub use with_default::DefaultType;
 
+use self::definitions::DefinitionRefValidator;
+
 #[pyclass(module = "pydantic_core._pydantic_core")]
 #[derive(Debug, Clone)]
 pub struct SchemaValidator {
     validator: CombinedValidator,
-    slots: Vec<CombinedValidator>,
+    definitions: Vec<CombinedValidator>,
     schema: PyObject,
     #[pyo3(get)]
     title: PyObject,
@@ -71,11 +74,14 @@ impl SchemaValidator {
         let self_validator = SelfValidator::new(py)?;
         let schema = self_validator.validate_schema(py, schema)?;
 
-        let mut build_context = BuildContext::new(schema)?;
+        let mut definitions_builder = DefinitionsBuilder::new();
 
-        let mut validator = build_validator(schema, config, &mut build_context)?;
-        validator.complete(&build_context)?;
-        let slots = build_context.into_slots_val()?;
+        let mut validator = build_validator(schema, config, &mut definitions_builder)?;
+        validator.complete(&definitions_builder)?;
+        let mut definitions = definitions_builder.clone().finish()?;
+        for val in definitions.iter_mut() {
+            val.complete(&definitions_builder)?;
+        }
         let config_title = match config {
             Some(c) => c.get_item("title"),
             None => None,
@@ -86,7 +92,7 @@ impl SchemaValidator {
         };
         Ok(Self {
             validator,
-            slots,
+            definitions,
             schema: schema.into_py(py),
             title,
         })
@@ -107,7 +113,7 @@ impl SchemaValidator {
         context: Option<&PyAny>,
         self_instance: Option<&PyAny>,
     ) -> PyResult<PyObject> {
-        let r = self._validate(py, input, strict, context, self_instance);
+        let r = self._validate(py, input, InputType::Python, strict, context, self_instance);
         r.map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
     }
 
@@ -120,7 +126,7 @@ impl SchemaValidator {
         context: Option<&PyAny>,
         self_instance: Option<&PyAny>,
     ) -> PyResult<bool> {
-        match self._validate(py, input, strict, context, self_instance) {
+        match self._validate(py, input, InputType::Python, strict, context, self_instance) {
             Ok(_) => Ok(true),
             Err(ValError::InternalErr(err)) => Err(err),
             Err(ValError::Omit) => Err(ValidationError::omit_error()),
@@ -139,33 +145,13 @@ impl SchemaValidator {
     ) -> PyResult<PyObject> {
         match input.parse_json() {
             Ok(input) => {
-                let r = self._validate(py, &input, strict, context, self_instance);
+                let r = self._validate(py, &input, InputType::Json, strict, context, self_instance);
                 r.map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Json))
             }
             Err(err) => Err(self.prepare_validation_err(py, err, ErrorMode::Json)),
         }
     }
 
-    #[pyo3(signature = (input, *, strict=None, context=None, self_instance=None))]
-    pub fn isinstance_json(
-        &self,
-        py: Python,
-        input: &PyAny,
-        strict: Option<bool>,
-        context: Option<&PyAny>,
-        self_instance: Option<&PyAny>,
-    ) -> PyResult<bool> {
-        match input.parse_json() {
-            Ok(input) => match self._validate(py, &input, strict, context, self_instance) {
-                Ok(_) => Ok(true),
-                Err(ValError::InternalErr(err)) => Err(err),
-                Err(ValError::Omit) => Err(ValidationError::omit_error()),
-                Err(ValError::LineErrors(_)) => Ok(false),
-            },
-            Err(_) => Ok(false),
-        }
-    }
-
     #[pyo3(signature = (obj, field_name, field_value, *, strict=None, context=None))]
     pub fn validate_assignment(
         &self,
@@ -177,8 +163,10 @@ impl SchemaValidator {
         context: Option<&PyAny>,
     ) -> PyResult<PyObject> {
         let extra = Extra {
+            mode: InputType::Python,
             data: None,
             strict,
+            ultra_strict: false,
             context,
             field_name: None,
             self_instance: None,
@@ -186,23 +174,23 @@ impl SchemaValidator {
 
         let guard = &mut RecursionGuard::default();
         self.validator
-            .validate_assignment(py, obj, field_name, field_value, &extra, &self.slots, guard)
+            .validate_assignment(py, obj, field_name, field_value, &extra, &self.definitions, guard)
             .map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
     }
 
     pub fn __repr__(&self, py: Python) -> String {
         format!(
-            "SchemaValidator(title={:?}, validator={:#?}, slots={:#?})",
+            "SchemaValidator(title={:?}, validator={:#?}, definitions={:#?})",
             self.title.extract::<&str>(py).unwrap(),
             self.validator,
-            self.slots,
+            self.definitions,
         )
     }
 
     fn __traverse__(&self, visit: PyVisit<'_>) -> Result<(), PyTraverseError> {
         self.validator.py_gc_traverse(&visit)?;
         visit.call(&self.schema)?;
-        for slot in self.slots.iter() {
+        for slot in self.definitions.iter() {
             slot.py_gc_traverse(&visit)?;
         }
         Ok(())
@@ -210,7 +198,7 @@ impl SchemaValidator {
 
     fn __clear__(&mut self) {
         self.validator.py_gc_clear();
-        for slot in self.slots.iter_mut() {
+        for slot in self.definitions.iter_mut() {
             slot.py_gc_clear();
         }
     }
@@ -221,6 +209,7 @@ impl SchemaValidator {
         &'data self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
+        mode: InputType,
         strict: Option<bool>,
         context: Option<&'data PyAny>,
         self_instance: Option<&PyAny>,
@@ -231,8 +220,8 @@ impl SchemaValidator {
         self.validator.validate(
             py,
             input,
-            &Extra::new(strict, context, self_instance),
-            &self.slots,
+            &Extra::new(strict, context, self_instance, mode),
+            &self.definitions,
             &mut RecursionGuard::default(),
         )
     }
@@ -244,6 +233,7 @@ impl SchemaValidator {
 
 static SCHEMA_DEFINITION: GILOnceCell<SchemaValidator> = GILOnceCell::new();
 
+#[derive(Debug, Clone)]
 pub struct SelfValidator<'py> {
     validator: &'py SchemaValidator,
 }
@@ -258,11 +248,12 @@ impl<'py> SelfValidator<'py> {
     }
 
     pub fn validate_schema(&self, py: Python<'py>, schema: &'py PyAny) -> PyResult<&'py PyAny> {
+        let extra = Extra::new(None, None, None, InputType::Python);
         match self.validator.validator.validate(
             py,
             schema,
-            &Extra::default(),
-            &self.validator.slots,
+            &extra,
+            &self.validator.definitions,
             &mut RecursionGuard::default(),
         ) {
             Ok(schema_obj) => Ok(schema_obj.into_ref(py)),
@@ -276,15 +267,20 @@ impl<'py> SelfValidator<'py> {
         py.run(code, None, Some(locals))?;
         let self_schema: &PyDict = locals.get_as_req(intern!(py, "self_schema"))?;
 
-        let mut build_context = BuildContext::for_self_schema();
+        let mut definitions_builder = DefinitionsBuilder::new();
 
-        let validator = match build_validator(self_schema, None, &mut build_context) {
+        let mut validator = match build_validator(self_schema, None, &mut definitions_builder) {
             Ok(v) => v,
             Err(err) => return py_err!("Error building self-schema:\n  {}", err),
         };
+        validator.complete(&definitions_builder)?;
+        let mut definitions = definitions_builder.clone().finish()?;
+        for val in definitions.iter_mut() {
+            val.complete(&definitions_builder)?;
+        }
         Ok(SchemaValidator {
             validator,
-            slots: build_context.into_slots_val()?,
+            definitions,
             schema: py.None(),
             title: "Self Schema".into_py(py),
         })
@@ -299,7 +295,7 @@ pub trait BuildValidator: Sized {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedValidator>,
+        definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator>;
 }
 
@@ -308,50 +304,25 @@ fn build_specific_validator<'a, T: BuildValidator>(
     val_type: &str,
     schema_dict: &'a PyDict,
     config: Option<&'a PyDict>,
-    build_context: &mut BuildContext<CombinedValidator>,
+    definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<CombinedValidator> {
     let py = schema_dict.py();
     if let Some(schema_ref) = schema_dict.get_as::<String>(intern!(py, "ref"))? {
-        // if there's a ref, we **might** want to store the validator in slots and return a DefinitionRefValidator:
-        // * if the ref isn't used at all, we just want to return a normal validator, and ignore the ref completely
-        // * if the ref is used inside itself, we have to store the validator in slots,
-        //   and return a DefinitionRefValidator - two step process with `prepare_slot` and `complete_slot`
-        // * if the ref is used elsewhere, we want to clone it each time it's used
-        if build_context.ref_used(&schema_ref) {
-            // the ref is used somewhere
-            // check the ref is unique
-            if build_context.ref_already_used(&schema_ref) {
-                return py_err!("Duplicate ref: `{}`", schema_ref);
-            }
-
-            return if build_context.ref_used_within(schema_dict, &schema_ref)? {
-                // the ref is used within itself, so we have to store the validator in slots
-                // and return a DefinitionRefValidator
-                let answers = Answers::new(schema_dict)?;
-                let slot_id = build_context.prepare_slot(schema_ref, Some(answers.clone()))?;
-                let inner_val = T::build(schema_dict, config, build_context)?;
-                let name = inner_val.get_name().to_string();
-                build_context.complete_slot(slot_id, inner_val)?;
-                Ok(definitions::DefinitionRefValidator::from_id(slot_id, name, answers))
-            } else {
-                // ref is used, but only out side itself, we want to clone it everywhere it's used
-                let validator = T::build(schema_dict, config, build_context)?;
-                build_context.store_reusable(schema_ref, validator.clone());
-                Ok(validator)
-            };
-        }
+        let inner_val = T::build(schema_dict, config, definitions)?;
+        let validator_id = definitions.add_definition(schema_ref, inner_val)?;
+        return Ok(DefinitionRefValidator::new(validator_id).into());
     }
 
-    T::build(schema_dict, config, build_context)
+    T::build(schema_dict, config, definitions)
         .map_err(|err| py_error_type!("Error building \"{}\" validator:\n  {}", val_type, err))
 }
 
 // macro to build the match statement for validator selection
 macro_rules! validator_match {
-    ($type:ident, $dict:ident, $config:ident, $build_context:ident, $($validator:path,)+) => {
+    ($type:ident, $dict:ident, $config:ident, $definitions:ident, $($validator:path,)+) => {
         match $type {
             $(
-                <$validator>::EXPECTED_TYPE => build_specific_validator::<$validator>($type, $dict, $config, $build_context),
+                <$validator>::EXPECTED_TYPE => build_specific_validator::<$validator>($type, $dict, $config, $definitions),
             )+
             _ => return py_err!(r#"Unknown schema type: "{}""#, $type),
         }
@@ -361,7 +332,7 @@ macro_rules! validator_match {
 pub fn build_validator<'a>(
     schema: &'a PyAny,
     config: Option<&'a PyDict>,
-    build_context: &mut BuildContext<CombinedValidator>,
+    definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<CombinedValidator> {
     let dict: &PyDict = schema.downcast()?;
     let type_: &str = dict.get_as_req(intern!(schema.py(), "type"))?;
@@ -369,7 +340,7 @@ pub fn build_validator<'a>(
         type_,
         dict,
         config,
-        build_context,
+        definitions,
         // typed dict e.g. heterogeneous dicts or simply a model
         typed_dict::TypedDictValidator,
         // unions
@@ -379,6 +350,7 @@ pub fn build_validator<'a>(
         nullable::NullableValidator,
         // model classes
         model::ModelValidator,
+        model_fields::ModelFieldsValidator,
         // dataclasses
         dataclass::DataclassArgsValidator,
         dataclass::DataclassValidator,
@@ -409,7 +381,7 @@ pub fn build_validator<'a>(
         // function call - validation around a function call
         call::CallValidator,
         // literals
-        literal::LiteralBuilder,
+        literal::LiteralValidator,
         // any
         any::AnyValidator,
         // bytes
@@ -436,6 +408,8 @@ pub fn build_validator<'a>(
         chain::ChainValidator,
         // lax or strict
         lax_or_strict::LaxOrStrictValidator,
+        // json or python
+        json_or_python::JsonOrPython,
         // generator validators
         generator::GeneratorValidator,
         // custom error
@@ -447,14 +421,16 @@ pub fn build_validator<'a>(
         url::MultiHostUrlValidator,
         // recursive (self-referencing) models
         definitions::DefinitionRefValidator,
-        definitions::DefinitionsBuilder,
+        definitions::DefinitionsValidatorBuilder,
     )
 }
 
 /// More (mostly immutable) data to pass between validators, should probably be class `Context`,
 /// but that would confuse it with context as per pydantic/pydantic#1549
-#[derive(Debug, Default)]
+#[derive(Debug)]
 pub struct Extra<'a> {
+    /// Validation mode
+    pub mode: InputType,
     /// This is used as the `data` kwargs to validator functions
     pub data: Option<&'a PyDict>,
     /// Represents the fields of the model we are currently validating
@@ -462,6 +438,8 @@ pub struct Extra<'a> {
     pub field_name: Option<&'a str>,
     /// whether we're in strict or lax mode
     pub strict: Option<bool>,
+    /// whether we're in ultra-strict mode, only used occasionally in unions
+    pub ultra_strict: bool,
     /// context used in validator functions
     pub context: Option<&'a PyAny>,
     /// This is an instance of the model or dataclass being validated, when validation is performed from `__init__`
@@ -469,21 +447,31 @@ pub struct Extra<'a> {
 }
 
 impl<'a> Extra<'a> {
-    pub fn new(strict: Option<bool>, context: Option<&'a PyAny>, self_instance: Option<&'a PyAny>) -> Self {
+    pub fn new(
+        strict: Option<bool>,
+        context: Option<&'a PyAny>,
+        self_instance: Option<&'a PyAny>,
+        mode: InputType,
+    ) -> Self {
         Extra {
             strict,
             context,
             self_instance,
-            ..Default::default()
+            mode,
+            data: None,
+            field_name: None,
+            ultra_strict: false,
         }
     }
 }
 
 impl<'a> Extra<'a> {
-    pub fn as_strict(&self) -> Self {
+    pub fn as_strict(&self, ultra_strict: bool) -> Self {
         Self {
+            mode: self.mode,
             data: self.data,
             strict: Some(true),
+            ultra_strict,
             context: self.context,
             field_name: self.field_name,
             self_instance: self.self_instance,
@@ -503,6 +491,7 @@ pub enum CombinedValidator {
     Nullable(nullable::NullableValidator),
     // create new model classes
     Model(model::ModelValidator),
+    ModelFields(model_fields::ModelFieldsValidator),
     // dataclasses
     DataclassArgs(dataclass::DataclassArgsValidator),
     Dataclass(dataclass::DataclassValidator),
@@ -536,11 +525,7 @@ pub enum CombinedValidator {
     // function call - validation around a function call
     FunctionCall(call::CallValidator),
     // literals
-    LiteralSingleString(literal::LiteralSingleStringValidator),
-    LiteralSingleInt(literal::LiteralSingleIntValidator),
-    LiteralMultipleStrings(literal::LiteralMultipleStringsValidator),
-    LiteralMultipleInts(literal::LiteralMultipleIntsValidator),
-    LiteralGeneral(literal::LiteralGeneralValidator),
+    Literal(literal::LiteralValidator),
     // any
     Any(any::AnyValidator),
     // bytes
@@ -579,6 +564,8 @@ pub enum CombinedValidator {
     MultiHostUrl(url::MultiHostUrlValidator),
     // reference to definition, useful for recursive (self-referencing) models
     DefinitionRef(definitions::DefinitionRefValidator),
+    // input dependent
+    JsonOrPython(json_or_python::JsonOrPython),
 }
 
 /// This trait must be implemented by all validators, it allows various validators to be accessed consistently,
@@ -596,7 +583,7 @@ pub trait Validator: Send + Sync + Clone + Debug {
         py: Python<'data>,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject>;
 
@@ -606,7 +593,7 @@ pub trait Validator: Send + Sync + Clone + Debug {
         _py: Python<'data>,
         _outer_loc: Option<impl Into<LocItem>>,
         _extra: &Extra,
-        _slots: &'data [CombinedValidator],
+        _definitions: &'data Definitions<CombinedValidator>,
         _recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, Option<PyObject>> {
         Ok(None)
@@ -621,28 +608,26 @@ pub trait Validator: Send + Sync + Clone + Debug {
         _field_name: &'data str,
         _field_value: &'data PyAny,
         _extra: &Extra,
-        _slots: &'data [CombinedValidator],
+        _definitions: &'data Definitions<CombinedValidator>,
         _recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
         let py_err = PyTypeError::new_err(format!("validate_assignment is not supported for {}", self.get_name()));
         Err(py_err.into())
     }
 
+    /// whether the validator behaves differently in strict mode, and in ultra strict mode
+    /// implementations should return true if any of their sub-validators return true
+    fn different_strict_behavior(
+        &self,
+        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
+        ultra_strict: bool,
+    ) -> bool;
+
     /// `get_name` generally returns `Self::EXPECTED_TYPE` or some other clear identifier of the validator
     /// this is used in the error location in unions, and in the top level message in `ValidationError`
     fn get_name(&self) -> &str;
 
-    /// allows validators to ask specific questions of sub-validators in a general way, could be extended
-    /// to do more, validators which don't know the question and have sub-validators
-    /// should return the result them in an `...iter().all(|v| v.ask(question))` way, ONLY
-    /// if they return the value of the sub-validator, e.g. functions, unions
-    fn ask(&self, _question: &Question) -> bool {
-        false
-    }
-
     /// this method must be implemented for any validator which holds references to other validators,
     /// it is used by `DefinitionRefValidator` to set its name
-    fn complete(&mut self, _build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
+    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()>;
 }
