@@ -1,10 +1,8 @@
 use pyo3::intern;
 use pyo3::prelude::*;
+use pyo3::types::{PyDict, PyString};
 
 use ahash::AHashSet;
-use pyo3::exceptions::PyKeyError;
-use pyo3::types::PyTuple;
-use pyo3::types::{PyDict, PySet, PyString, PyType};
 
 use crate::build_tools::{is_strict, py_err, schema_or_config, schema_or_config_same, ExtraBehavior, SchemaDict};
 use crate::errors::{py_err_string, ErrorType, ValError, ValLineError, ValResult};
@@ -13,10 +11,9 @@ use crate::input::{
     MappingGenericIterator,
 };
 use crate::lookup_key::LookupKey;
-use crate::questions::Question;
 use crate::recursion_guard::RecursionGuard;
 
-use super::{build_validator, BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
+use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
 
 #[derive(Debug, Clone)]
 struct TypedDictField {
@@ -25,7 +22,6 @@ struct TypedDictField {
     name_py: Py<PyString>,
     required: bool,
     validator: CombinedValidator,
-    frozen: bool,
 }
 
 #[derive(Debug, Clone)]
@@ -34,8 +30,6 @@ pub struct TypedDictValidator {
     extra_behavior: ExtraBehavior,
     extra_validator: Option<Box<CombinedValidator>>,
     strict: bool,
-    from_attributes: bool,
-    return_fields_set: bool,
     loc_by_alias: bool,
 }
 
@@ -45,22 +39,19 @@ impl BuildValidator for TypedDictValidator {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedValidator>,
+        definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let strict = is_strict(schema, config)?;
 
         let total =
             schema_or_config(schema, config, intern!(py, "total"), intern!(py, "typed_dict_total"))?.unwrap_or(true);
-        let from_attributes = schema_or_config_same(schema, config, intern!(py, "from_attributes"))?.unwrap_or(false);
         let populate_by_name = schema_or_config_same(schema, config, intern!(py, "populate_by_name"))?.unwrap_or(false);
 
-        let return_fields_set = schema.get_as(intern!(py, "return_fields_set"))?.unwrap_or(false);
-
         let extra_behavior = ExtraBehavior::from_schema_or_config(py, schema, config, ExtraBehavior::Ignore)?;
 
         let extra_validator = match (schema.get_item(intern!(py, "extra_validator")), &extra_behavior) {
-            (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(v, config, build_context)?)),
+            (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(v, config, definitions)?)),
             (Some(_), _) => return py_err!("extra_validator can only be used if extra_behavior=allow"),
             (_, _) => None,
         };
@@ -74,7 +65,7 @@ impl BuildValidator for TypedDictValidator {
 
             let schema = field_info.get_as_req(intern!(py, "schema"))?;
 
-            let validator = match build_validator(schema, config, build_context) {
+            let validator = match build_validator(schema, config, definitions) {
                 Ok(v) => v,
                 Err(err) => return py_err!("Field \"{}\":\n  {}", field_name, err),
             };
@@ -118,7 +109,6 @@ impl BuildValidator for TypedDictValidator {
                 name_py: PyString::intern(py, field_name).into(),
                 validator,
                 required,
-                frozen: field_info.get_as::<bool>(intern!(py, "frozen"))?.unwrap_or(false),
             });
         }
 
@@ -127,8 +117,6 @@ impl BuildValidator for TypedDictValidator {
             extra_behavior,
             extra_validator,
             strict,
-            from_attributes,
-            return_fields_set,
             loc_by_alias: config.get_as(intern!(py, "loc_by_alias"))?.unwrap_or(true),
         }
         .into())
@@ -141,23 +129,20 @@ impl Validator for TypedDictValidator {
         py: Python<'data>,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
         let strict = extra.strict.unwrap_or(self.strict);
-        let dict = input.validate_typed_dict(strict, self.from_attributes)?;
+        let dict = input.validate_dict(strict)?;
 
         let output_dict = PyDict::new(py);
         let mut errors: Vec<ValLineError> = Vec::with_capacity(self.fields.len());
-        let mut fields_set_vec: Option<Vec<Py<PyString>>> = match self.return_fields_set {
-            true => Some(Vec::with_capacity(self.fields.len())),
-            false => None,
-        };
 
         // we only care about which keys have been used if we're iterating over the object for extra after
         // the first pass
-        let mut used_keys: Option<AHashSet<&str>> = match self.extra_behavior {
-            ExtraBehavior::Allow | ExtraBehavior::Forbid => Some(AHashSet::with_capacity(self.fields.len())),
+        let mut used_keys: Option<AHashSet<&str>> = match (&self.extra_behavior, &dict) {
+            (_, GenericMapping::PyGetAttr(_, _)) => None,
+            (ExtraBehavior::Allow | ExtraBehavior::Forbid, _) => Some(AHashSet::with_capacity(self.fields.len())),
             _ => None,
         };
 
@@ -190,13 +175,10 @@ impl Validator for TypedDictValidator {
                         }
                         match field
                             .validator
-                            .validate(py, value, &extra, slots, recursion_guard)
+                            .validate(py, value, &extra, definitions, recursion_guard)
                         {
                             Ok(value) => {
                                 output_dict.set_item(&field.name_py, value)?;
-                                if let Some(ref mut fs) = fields_set_vec {
-                                    fs.push(field.name_py.clone_ref(py));
-                                }
                             }
                             Err(ValError::Omit) => continue,
                             Err(ValError::LineErrors(line_errors)) => {
@@ -207,7 +189,7 @@ impl Validator for TypedDictValidator {
                             Err(err) => return Err(err),
                         }
                         continue;
-                    } else if let Some(value) = field.validator.default_value(py, Some(field.name.as_str()), &extra, slots, recursion_guard)? {
+                    } else if let Some(value) = field.validator.default_value(py, Some(field.name.as_str()), &extra, definitions, recursion_guard)? {
                         output_dict.set_item(&field.name_py, value)?;
                     } else if field.required {
                         errors.push(field.lookup_key.error(
@@ -252,12 +234,9 @@ impl Validator for TypedDictValidator {
                             ExtraBehavior::Allow => {
                             let py_key = either_str.as_py_string(py);
                                 if let Some(ref validator) = self.extra_validator {
-                                    match validator.validate(py, value, &extra, slots, recursion_guard) {
+                                    match validator.validate(py, value, &extra, definitions, recursion_guard) {
                                         Ok(value) => {
                                             output_dict.set_item(py_key, value)?;
-                                            if let Some(ref mut fs) = fields_set_vec {
-                                                fs.push(py_key.into_py(py));
-                                            }
                                         }
                                         Err(ValError::LineErrors(line_errors)) => {
                                             for err in line_errors {
@@ -268,9 +247,6 @@ impl Validator for TypedDictValidator {
                                     }
                                 } else {
                                     output_dict.set_item(py_key, value.to_object(py))?;
-                                    if let Some(ref mut fs) = fields_set_vec {
-                                        fs.push(py_key.into_py(py));
-                                    }
                                 };
                             }
                         }
@@ -287,120 +263,31 @@ impl Validator for TypedDictValidator {
 
         if !errors.is_empty() {
             Err(ValError::LineErrors(errors))
-        } else if let Some(fs) = fields_set_vec {
-            let fields_set = PySet::new(py, &fs)?;
-            Ok((output_dict, fields_set).to_object(py))
         } else {
             Ok(output_dict.to_object(py))
         }
     }
 
-    fn validate_assignment<'s, 'data: 's>(
-        &'s self,
-        py: Python<'data>,
-        obj: &'data PyAny,
-        field_name: &'data str,
-        field_value: &'data PyAny,
-        extra: &Extra,
-        slots: &'data [CombinedValidator],
-        recursion_guard: &'s mut RecursionGuard,
-    ) -> ValResult<'data, PyObject> {
-        let dict: &PyDict = obj.downcast()?;
-
-        let ok = |output: PyObject| {
-            dict.set_item(field_name, output)?;
-            Ok(dict.to_object(py))
-        };
-
-        let prepare_result = |result: ValResult<'data, PyObject>| match result {
-            Ok(output) => ok(output),
-            Err(ValError::LineErrors(line_errors)) => {
-                let errors = line_errors
-                    .into_iter()
-                    .map(|e| e.with_outer_location(field_name.to_string().into()))
-                    .collect();
-                Err(ValError::LineErrors(errors))
-            }
-            Err(err) => Err(err),
-        };
-
-        // by using dict but removing the field in question, we match V1 behaviour
-        let data_dict = dict.copy()?;
-        if let Err(err) = data_dict.del_item(field_name) {
-            // KeyError is fine here as the field might not be in the dict
-            if !err.get_type(py).is(PyType::new::<PyKeyError>(py)) {
-                return Err(err.into());
-            }
-        }
-
-        let extra = Extra {
-            data: Some(data_dict),
-            field_name: Some(field_name),
-            ..*extra
-        };
-
-        let new_data = if let Some(field) = self.fields.iter().find(|f| f.name == field_name) {
-            if field.frozen {
-                Err(ValError::new_with_loc(
-                    ErrorType::FrozenField,
-                    field_value,
-                    field.name.to_string(),
-                ))
-            } else {
-                prepare_result(
-                    field
-                        .validator
-                        .validate(py, field_value, &extra, slots, recursion_guard),
-                )
-            }
-        } else {
-            // Handle extra (unknown) field
-            // We partially use the extra_behavior for initialization / validation
-            // to determine how to handle assignment
-            // For models / typed dicts we forbid assigning extra attributes
-            // unless the user explicitly set extra_behavior to 'allow'
-            match self.extra_behavior {
-                ExtraBehavior::Allow => match self.extra_validator {
-                    Some(ref validator) => {
-                        prepare_result(validator.validate(py, field_value, &extra, slots, recursion_guard))
-                    }
-                    None => ok(field_value.to_object(py)),
-                },
-                ExtraBehavior::Forbid | ExtraBehavior::Ignore => {
-                    return Err(ValError::new_with_loc(
-                        ErrorType::NoSuchAttribute {
-                            attribute: field_name.to_string(),
-                        },
-                        field_value,
-                        field_name.to_string(),
-                    ))
-                }
-            }
-        }?;
-        if self.return_fields_set {
-            let fields_set: &PySet = PySet::new(py, &[field_name.to_string()])?;
-            Ok(PyTuple::new(py, [new_data, fields_set.to_object(py)]).to_object(py))
-        } else {
-            Ok(new_data)
-        }
+    fn different_strict_behavior(
+        &self,
+        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
+        ultra_strict: bool,
+    ) -> bool {
+        self.fields
+            .iter()
+            .any(|f| f.validator.different_strict_behavior(definitions, ultra_strict))
     }
 
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
 
-    fn ask(&self, question: &Question) -> bool {
-        match question {
-            Question::ReturnFieldsSet => self.return_fields_set,
-        }
-    }
-
-    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
+    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
         self.fields
             .iter_mut()
-            .try_for_each(|f| f.validator.complete(build_context))?;
+            .try_for_each(|f| f.validator.complete(definitions))?;
         match &mut self.extra_validator {
-            Some(v) => v.complete(build_context),
+            Some(v) => v.complete(definitions),
             None => Ok(()),
         }
     }
