@@ -13,11 +13,10 @@ use crate::build_tools::{is_strict, py_err, schema_or_config, SchemaDict};
 use crate::errors::{ErrorType, LocItem, ValError, ValLineError, ValResult};
 use crate::input::{GenericMapping, Input};
 use crate::lookup_key::LookupKey;
-use crate::questions::Question;
 use crate::recursion_guard::RecursionGuard;
 
 use super::custom_error::CustomError;
-use super::{build_validator, BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
+use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
 
 #[derive(Debug, Clone)]
 pub struct UnionValidator {
@@ -25,6 +24,8 @@ pub struct UnionValidator {
     custom_error: Option<CustomError>,
     strict: bool,
     name: String,
+    strict_required: bool,
+    ultra_strict_required: bool,
 }
 
 impl BuildValidator for UnionValidator {
@@ -33,13 +34,13 @@ impl BuildValidator for UnionValidator {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedValidator>,
+        definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let choices: Vec<CombinedValidator> = schema
             .get_as_req::<&PyList>(intern!(py, "choices"))?
             .iter()
-            .map(|choice| build_validator(choice, config, build_context))
+            .map(|choice| build_validator(choice, config, definitions))
             .collect::<PyResult<Vec<CombinedValidator>>>()?;
 
         let auto_collapse = || schema.get_as_req(intern!(py, "auto_collapse")).unwrap_or(true);
@@ -51,9 +52,11 @@ impl BuildValidator for UnionValidator {
 
                 Ok(Self {
                     choices,
-                    custom_error: CustomError::build(schema, config, build_context)?,
+                    custom_error: CustomError::build(schema, config, definitions)?,
                     strict: is_strict(schema, config)?,
                     name: format!("{}[{descr}]", Self::EXPECTED_TYPE),
+                    strict_required: true,
+                    ultra_strict_required: false,
                 }
                 .into())
             }
@@ -81,18 +84,31 @@ impl Validator for UnionValidator {
         py: Python<'data>,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
+        if self.ultra_strict_required {
+            // do an ultra strict check first
+            let ultra_strict_extra = extra.as_strict(true);
+            if let Some(res) = self
+                .choices
+                .iter()
+                .map(|validator| validator.validate(py, input, &ultra_strict_extra, definitions, recursion_guard))
+                .find(ValResult::is_ok)
+            {
+                return res;
+            }
+        }
+
         if extra.strict.unwrap_or(self.strict) {
             let mut errors: Option<Vec<ValLineError>> = match self.custom_error {
                 None => Some(Vec::with_capacity(self.choices.len())),
                 _ => None,
             };
-            let strict_extra = extra.as_strict();
+            let strict_extra = extra.as_strict(false);
 
             for validator in &self.choices {
-                let line_errors = match validator.validate(py, input, &strict_extra, slots, recursion_guard) {
+                let line_errors = match validator.validate(py, input, &strict_extra, definitions, recursion_guard) {
                     Err(ValError::LineErrors(line_errors)) => line_errors,
                     otherwise => return otherwise,
                 };
@@ -108,16 +124,18 @@ impl Validator for UnionValidator {
 
             Err(self.or_custom_error(errors, input))
         } else {
-            // 1st pass: check if the value is an exact instance of one of the Union types,
-            // e.g. use validate in strict mode
-            let strict_extra = extra.as_strict();
-            if let Some(res) = self
-                .choices
-                .iter()
-                .map(|validator| validator.validate(py, input, &strict_extra, slots, recursion_guard))
-                .find(ValResult::is_ok)
-            {
-                return res;
+            if self.strict_required {
+                // 1st pass: check if the value is an exact instance of one of the Union types,
+                // e.g. use validate in strict mode
+                let strict_extra = extra.as_strict(false);
+                if let Some(res) = self
+                    .choices
+                    .iter()
+                    .map(|validator| validator.validate(py, input, &strict_extra, definitions, recursion_guard))
+                    .find(ValResult::is_ok)
+                {
+                    return res;
+                }
             }
 
             let mut errors: Option<Vec<ValLineError>> = match self.custom_error {
@@ -127,7 +145,7 @@ impl Validator for UnionValidator {
 
             // 2nd pass: check if the value can be coerced into one of the Union types, e.g. use validate
             for validator in &self.choices {
-                let line_errors = match validator.validate(py, input, extra, slots, recursion_guard) {
+                let line_errors = match validator.validate(py, input, extra, definitions, recursion_guard) {
                     Err(ValError::LineErrors(line_errors)) => line_errors,
                     success => return success,
                 };
@@ -145,16 +163,25 @@ impl Validator for UnionValidator {
         }
     }
 
+    fn different_strict_behavior(
+        &self,
+        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
+        ultra_strict: bool,
+    ) -> bool {
+        self.choices
+            .iter()
+            .any(|v| v.different_strict_behavior(definitions, ultra_strict))
+    }
+
     fn get_name(&self) -> &str {
         &self.name
     }
 
-    fn ask(&self, question: &Question) -> bool {
-        self.choices.iter().all(|v| v.ask(question))
-    }
-
-    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
-        self.choices.iter_mut().try_for_each(|v| v.complete(build_context))
+    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
+        self.choices.iter_mut().try_for_each(|v| v.complete(definitions))?;
+        self.strict_required = self.different_strict_behavior(Some(definitions), false);
+        self.ultra_strict_required = self.different_strict_behavior(Some(definitions), true);
+        Ok(())
     }
 }
 
@@ -253,7 +280,7 @@ impl BuildValidator for TaggedUnionValidator {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedValidator>,
+        definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let discriminator = Discriminator::new(py, schema.get_as_req(intern!(py, "discriminator"))?)?;
@@ -274,7 +301,7 @@ impl BuildValidator for TaggedUnionValidator {
                 continue;
             }
 
-            let validator = build_validator(value, config, build_context)?;
+            let validator = build_validator(value, config, definitions)?;
             let tag_repr = tag.repr();
             if first {
                 first = false;
@@ -326,7 +353,7 @@ impl BuildValidator for TaggedUnionValidator {
             discriminator,
             from_attributes,
             strict: is_strict(schema, config)?,
-            custom_error: CustomError::build(schema, config, build_context)?,
+            custom_error: CustomError::build(schema, config, definitions)?,
             tags_repr,
             discriminator_repr,
             name: format!("{}[{descr}]", Self::EXPECTED_TYPE),
@@ -341,7 +368,7 @@ impl Validator for TaggedUnionValidator {
         py: Python<'data>,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
         match self.discriminator {
@@ -362,14 +389,14 @@ impl Validator for TaggedUnionValidator {
                         }
                     }};
                 }
-                let dict = input.validate_typed_dict(self.strict, self.from_attributes)?;
+                let dict = input.validate_model_fields(self.strict, self.from_attributes)?;
                 let tag = match dict {
                     GenericMapping::PyDict(dict) => find_validator!(py_get_dict_item, dict),
                     GenericMapping::PyGetAttr(obj, kwargs) => find_validator!(py_get_attr, obj, kwargs),
                     GenericMapping::PyMapping(mapping) => find_validator!(py_get_mapping_item, mapping),
                     GenericMapping::JsonObject(mapping) => find_validator!(json_get, mapping),
                 }?;
-                self.find_call_validator(py, &tag, input, extra, slots, recursion_guard)
+                self.find_call_validator(py, &tag, input, extra, definitions, recursion_guard)
             }
             Discriminator::Function(ref func) => {
                 let tag = func.call1(py, (input.to_object(py),))?;
@@ -377,7 +404,14 @@ impl Validator for TaggedUnionValidator {
                     Err(self.tag_not_found(input))
                 } else {
                     let tag: &PyAny = tag.downcast(py)?;
-                    self.find_call_validator(py, &(ChoiceKey::from_py(tag)?), input, extra, slots, recursion_guard)
+                    self.find_call_validator(
+                        py,
+                        &(ChoiceKey::from_py(tag)?),
+                        input,
+                        extra,
+                        definitions,
+                        recursion_guard,
+                    )
                 }
             }
             Discriminator::SelfSchema => self.find_call_validator(
@@ -385,24 +419,30 @@ impl Validator for TaggedUnionValidator {
                 &ChoiceKey::Str(self.self_schema_tag(py, input)?.into_owned()),
                 input,
                 extra,
-                slots,
+                definitions,
                 recursion_guard,
             ),
         }
     }
 
+    fn different_strict_behavior(
+        &self,
+        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
+        ultra_strict: bool,
+    ) -> bool {
+        self.choices
+            .values()
+            .any(|v| v.different_strict_behavior(definitions, ultra_strict))
+    }
+
     fn get_name(&self) -> &str {
         &self.name
     }
 
-    fn ask(&self, question: &Question) -> bool {
-        self.choices.values().all(|v| v.ask(question))
-    }
-
-    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
+    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
         self.choices
             .iter_mut()
-            .try_for_each(|(_, validator)| validator.complete(build_context))
+            .try_for_each(|(_, validator)| validator.complete(definitions))
     }
 }
 
@@ -458,18 +498,18 @@ impl TaggedUnionValidator {
         tag: &ChoiceKey,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
         if let Some(validator) = self.choices.get(tag) {
-            return match validator.validate(py, input, extra, slots, recursion_guard) {
+            return match validator.validate(py, input, extra, definitions, recursion_guard) {
                 Ok(res) => Ok(res),
                 Err(err) => Err(err.with_outer_location(tag.into())),
             };
         } else if let Some(ref repeat_choices) = self.repeat_choices {
             if let Some(choice_tag) = repeat_choices.get(tag) {
                 let validator = &self.choices[choice_tag];
-                return match validator.validate(py, input, extra, slots, recursion_guard) {
+                return match validator.validate(py, input, extra, definitions, recursion_guard) {
                     Ok(res) => Ok(res),
                     Err(err) => Err(err.with_outer_location(tag.into())),
                 };
