@@ -1,14 +1,20 @@
 use pyo3::intern;
+use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
 
 use crate::build_tools::{py_err, schema_or_config_same, SchemaDict};
 use crate::errors::{LocItem, ValError, ValResult};
 use crate::input::Input;
-use crate::questions::Question;
 use crate::recursion_guard::RecursionGuard;
 
-use super::{build_validator, BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
+use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+
+static COPY_DEEPCOPY: GILOnceCell<PyObject> = GILOnceCell::new();
+
+fn get_deepcopy(py: Python) -> PyResult<PyObject> {
+    Ok(py.import("copy")?.getattr("deepcopy")?.into_py(py))
+}
 
 #[derive(Debug, Clone)]
 pub enum DefaultType {
@@ -53,6 +59,7 @@ pub struct WithDefaultValidator {
     on_error: OnError,
     validator: Box<CombinedValidator>,
     validate_default: bool,
+    copy_default: bool,
     name: String,
 }
 
@@ -62,7 +69,7 @@ impl BuildValidator for WithDefaultValidator {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext<CombinedValidator>,
+        definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let default = DefaultType::new(schema)?;
@@ -81,7 +88,14 @@ impl BuildValidator for WithDefaultValidator {
         };
 
         let sub_schema: &PyAny = schema.get_as_req(intern!(schema.py(), "schema"))?;
-        let validator = Box::new(build_validator(sub_schema, config, build_context)?);
+        let validator = Box::new(build_validator(sub_schema, config, definitions)?);
+
+        let copy_default = if let DefaultType::Default(default_obj) = &default {
+            default_obj.as_ref(py).hash().is_err()
+        } else {
+            false
+        };
+
         let name = format!("{}[{}]", Self::EXPECTED_TYPE, validator.get_name());
 
         Ok(Self {
@@ -89,6 +103,7 @@ impl BuildValidator for WithDefaultValidator {
             on_error,
             validator,
             validate_default: schema_or_config_same(schema, config, intern!(py, "validate_default"))?.unwrap_or(false),
+            copy_default,
             name,
         }
         .into())
@@ -101,15 +116,15 @@ impl Validator for WithDefaultValidator {
         py: Python<'data>,
         input: &'data impl Input<'data>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
-        match self.validator.validate(py, input, extra, slots, recursion_guard) {
+        match self.validator.validate(py, input, extra, definitions, recursion_guard) {
             Ok(v) => Ok(v),
             Err(e) => match self.on_error {
                 OnError::Raise => Err(e),
                 OnError::Default => Ok(self
-                    .default_value(py, None::<usize>, extra, slots, recursion_guard)?
+                    .default_value(py, None::<usize>, extra, definitions, recursion_guard)?
                     .unwrap()),
                 OnError::Omit => Err(ValError::Omit),
             },
@@ -121,13 +136,19 @@ impl Validator for WithDefaultValidator {
         py: Python<'data>,
         outer_loc: Option<impl Into<LocItem>>,
         extra: &Extra,
-        slots: &'data [CombinedValidator],
+        definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, Option<PyObject>> {
         match self.default.default_value(py)? {
-            Some(dft) => {
+            Some(stored_dft) => {
+                let dft: Py<PyAny> = if self.copy_default {
+                    let deepcopy_func = COPY_DEEPCOPY.get_or_init(py, || get_deepcopy(py).unwrap());
+                    deepcopy_func.call1(py, (&stored_dft,))?.into_py(py)
+                } else {
+                    stored_dft
+                };
                 if self.validate_default {
-                    match self.validate(py, dft.into_ref(py), extra, slots, recursion_guard) {
+                    match self.validate(py, dft.into_ref(py), extra, definitions, recursion_guard) {
                         Ok(v) => Ok(Some(v)),
                         Err(e) => {
                             if let Some(outer_loc) = outer_loc {
@@ -145,16 +166,20 @@ impl Validator for WithDefaultValidator {
         }
     }
 
+    fn different_strict_behavior(
+        &self,
+        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
+        ultra_strict: bool,
+    ) -> bool {
+        self.validator.different_strict_behavior(definitions, ultra_strict)
+    }
+
     fn get_name(&self) -> &str {
         &self.name
     }
 
-    fn ask(&self, question: &Question) -> bool {
-        self.validator.ask(question)
-    }
-
-    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
-        self.validator.complete(build_context)
+    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
+        self.validator.complete(definitions)
     }
 }
 
