@@ -6,11 +6,13 @@ import os
 import platform
 from datetime import date, datetime, timedelta, timezone
 from decimal import Decimal
-from typing import Dict, FrozenSet, List, Optional, Set, Union
+from enum import Enum
+from typing import Any, Dict, FrozenSet, List, Optional, Set, Union
 
 import pytest
+from dirty_equals import IsStr
 
-from pydantic_core import PydanticCustomError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import ArgsKwargs, PydanticCustomError, SchemaValidator, ValidationError, core_schema
 from pydantic_core import ValidationError as CoreValidationError
 
 if os.getenv('BENCHMARK_VS_PYDANTIC'):
@@ -39,55 +41,20 @@ class TestBenchmarkSimpleModel:
     @pytest.fixture(scope='class')
     def core_validator_fs(self):
         class CoreModel:
-            __slots__ = '__dict__', '__fields_set__'
+            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
         return SchemaValidator(
             {
                 'type': 'model',
                 'cls': CoreModel,
                 'schema': {
-                    'type': 'typed-dict',
-                    'return_fields_set': True,
+                    'type': 'model-fields',
                     'fields': {
-                        'name': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                        'age': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                        'friends': {
-                            'type': 'typed-dict-field',
-                            'schema': {'type': 'list', 'items_schema': {'type': 'int'}},
-                        },
+                        'name': {'type': 'model-field', 'schema': {'type': 'str'}},
+                        'age': {'type': 'model-field', 'schema': {'type': 'int'}},
+                        'friends': {'type': 'model-field', 'schema': {'type': 'list', 'items_schema': {'type': 'int'}}},
                         'settings': {
-                            'type': 'typed-dict-field',
-                            'schema': {
-                                'type': 'dict',
-                                'keys_schema': {'type': 'str'},
-                                'values_schema': {'type': 'float'},
-                            },
-                        },
-                    },
-                },
-            }
-        )
-
-    @pytest.fixture(scope='class')
-    def core_validator_not_fs(self):
-        class CoreModel:
-            __slots__ = '__dict__', '__fields_set__'
-
-        return SchemaValidator(
-            {
-                'type': 'model',
-                'cls': CoreModel,
-                'schema': {
-                    'type': 'typed-dict',
-                    'fields': {
-                        'name': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                        'age': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                        'friends': {
-                            'type': 'typed-dict-field',
-                            'schema': {'type': 'list', 'items_schema': {'type': 'int'}},
-                        },
-                        'settings': {
-                            'type': 'typed-dict-field',
+                            'type': 'model-field',
                             'schema': {
                                 'type': 'dict',
                                 'keys_schema': {'type': 'str'},
@@ -111,17 +78,9 @@ class TestBenchmarkSimpleModel:
         m = core_validator_fs.validate_python(self.data)
         assert m.name == 'John'
         assert m.__dict__.keys() == {'name', 'age', 'friends', 'settings'}
-        assert m.__fields_set__ == {'name', 'age', 'friends', 'settings'}
+        assert m.__pydantic_fields_set__ == {'name', 'age', 'friends', 'settings'}
         benchmark(core_validator_fs.validate_python, self.data)
 
-    @pytest.mark.benchmark(group='simple model - python')
-    def test_core_python_not_fs(self, core_validator_not_fs, benchmark):
-        m = core_validator_not_fs.validate_python(self.data)
-        assert m.name == 'John'
-        assert m.__dict__.keys() == {'name', 'age', 'friends', 'settings'}
-        assert not hasattr(m, '__fields_set__')
-        benchmark(core_validator_not_fs.validate_python, self.data)
-
     @skip_pydantic
     @pytest.mark.benchmark(group='simple model - JSON')
     def test_pyd_json(self, pydantic_model, benchmark):
@@ -137,54 +96,45 @@ class TestBenchmarkSimpleModel:
         json_data = json.dumps(self.data)
         benchmark(core_validator_fs.validate_json, json_data)
 
-    @pytest.mark.benchmark(group='simple model - JSON')
-    def test_core_json_not_fs(self, core_validator_not_fs, benchmark):
-        json_data = json.dumps(self.data)
-        benchmark(core_validator_not_fs.validate_json, json_data)
-
 
 class TestModelLarge:
     @pytest.fixture(scope='class')
     def core_model_validator(self):
         class CoreModel:
-            __slots__ = '__dict__', '__fields_set__'
+            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
         return SchemaValidator(
             {
                 'type': 'model',
                 'cls': CoreModel,
                 'schema': {
-                    'type': 'typed-dict',
-                    'return_fields_set': True,
+                    'type': 'model-fields',
                     'extra_behavior': 'allow',
-                    'total': False,
-                    'fields': {
-                        f'field_{i}': {'type': 'typed-dict-field', 'schema': {'type': 'int'}} for i in range(100)
-                    },
+                    'fields': {f'field_{i}': {'type': 'model-field', 'schema': {'type': 'int'}} for i in range(100)},
                 },
             }
         )
 
-    data = {f'field_{97 - i}': i for i in range(98)}
+    data = {f'field_{99 - i}': i for i in range(100)}
     data['more'] = 'more data'
 
     @pytest.mark.benchmark(group='large model - python')
     def test_core_python(self, core_model_validator, benchmark):
         m = core_model_validator.validate_python(self.data)
-        assert m.field_0 == 97
-        assert m.field_1 == 96
-        assert m.field_97 == 0
-        assert m.more == 'more data'
+        assert m.field_0 == 99
+        assert m.field_1 == 98
+        assert m.field_97 == 2
+        assert m.__pydantic_extra__ == {'more': 'more data'}
         benchmark(core_model_validator.validate_python, self.data)
 
     @pytest.mark.benchmark(group='large model - JSON')
     def test_core_json_fs(self, core_model_validator, benchmark):
         json_data = json.dumps(self.data)
         m = core_model_validator.validate_json(json_data)
-        assert m.field_0 == 97
-        assert m.field_1 == 96
-        assert m.field_97 == 0
-        assert m.more == 'more data'
+        assert m.field_0 == 99
+        assert m.field_1 == 98
+        assert m.field_97 == 2
+        assert m.__pydantic_extra__ == {'more': 'more data'}
         benchmark(core_model_validator.validate_json, json_data)
 
 
@@ -242,8 +192,8 @@ def test_small_class_core_dict(benchmark):
 @pytest.mark.benchmark(group='create small model')
 def test_small_class_core_model(benchmark):
     class MyCoreModel:
-        # this is not required, but it avoids `__fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__fields_set__'
+        # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
         # these are here just as decoration
         name: str
         age: int
@@ -253,11 +203,10 @@ def test_small_class_core_model(benchmark):
             'type': 'model',
             'cls': MyCoreModel,
             'schema': {
-                'type': 'typed-dict',
-                'return_fields_set': True,
+                'type': 'model-fields',
                 'fields': {
-                    'name': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                    'age': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
+                    'name': {'type': 'model-field', 'schema': {'type': 'str'}},
+                    'age': {'type': 'model-field', 'schema': {'type': 'int'}},
                 },
             },
         }
@@ -315,6 +264,21 @@ def test_core_string_strict_wrong(benchmark):
             pass
 
 
+@pytest.mark.benchmark(group='string')
+def test_core_string_strict_wrong_str_e(benchmark):
+    validator = SchemaValidator(core_schema.str_schema(strict=True))
+
+    with pytest.raises(ValidationError, match='Input should be a valid string'):
+        validator.validate_python(123)
+
+    @benchmark
+    def t():
+        try:
+            validator.validate_python(123)
+        except ValidationError as e:
+            str(e)
+
+
 @pytest.mark.benchmark(group='isinstance-string')
 def test_isinstance_string_lax_true(benchmark):
     validator = SchemaValidator(core_schema.str_schema())
@@ -379,8 +343,8 @@ def test_definition_model_pyd(definition_model_data, benchmark):
 @pytest.mark.benchmark(group='recursive model')
 def test_definition_model_core(definition_model_data, benchmark):
     class CoreBranch:
-        # this is not required, but it avoids `__fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__fields_set__'
+        # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
     v = SchemaValidator(
         {
@@ -388,12 +352,11 @@ def test_definition_model_core(definition_model_data, benchmark):
             'type': 'model',
             'cls': CoreBranch,
             'schema': {
-                'type': 'typed-dict',
-                'return_fields_set': True,
+                'type': 'model-fields',
                 'fields': {
-                    'width': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
+                    'width': {'type': 'model-field', 'schema': {'type': 'int'}},
                     'branch': {
-                        'type': 'typed-dict-field',
+                        'type': 'model-field',
                         'schema': {
                             'type': 'default',
                             'schema': {
@@ -513,6 +476,7 @@ def test_list_of_any_core_py(benchmark):
 
 
 set_of_ints_data = ({i for i in range(1000)}, {str(i) for i in range(1000)})
+set_of_ints_duplicates = ([i for i in range(100)] * 10, [str(i) for i in range(100)] * 10)
 
 
 @skip_pydantic
@@ -537,6 +501,26 @@ def test_set_of_ints_core(benchmark):
         v.validate_python(set_of_ints_data[1])
 
 
+@pytest.mark.benchmark(group='Set[int]')
+def test_set_of_ints_core_duplicates(benchmark):
+    v = SchemaValidator({'type': 'set', 'items_schema': {'type': 'int'}})
+
+    @benchmark
+    def t():
+        v.validate_python(set_of_ints_duplicates[0])
+        v.validate_python(set_of_ints_duplicates[1])
+
+
+@pytest.mark.benchmark(group='Set[int]')
+def test_set_of_ints_core_length(benchmark):
+    v = SchemaValidator({'type': 'set', 'items_schema': {'type': 'int'}, 'max_length': 2000})
+
+    @benchmark
+    def t():
+        v.validate_python(set_of_ints_data[0])
+        v.validate_python(set_of_ints_data[1])
+
+
 @skip_pydantic
 @pytest.mark.benchmark(group='Set[int] JSON')
 def test_set_of_ints_pyd_json(benchmark):
@@ -563,7 +547,20 @@ def test_set_of_ints_core_json(benchmark):
         v.validate_json(json_data[1])
 
 
+@pytest.mark.benchmark(group='Set[int] JSON')
+def test_set_of_ints_core_json_duplicates(benchmark):
+    v = SchemaValidator({'type': 'set', 'items_schema': {'type': 'int'}})
+
+    json_data = [json.dumps(list(d)) for d in set_of_ints_duplicates]
+
+    @benchmark
+    def t():
+        v.validate_json(json_data[0])
+        v.validate_json(json_data[1])
+
+
 frozenset_of_ints = frozenset({i for i in range(1000)})
+frozenset_of_ints_duplicates = [i for i in range(100)] * 10
 
 
 @skip_pydantic
@@ -582,6 +579,13 @@ def test_frozenset_of_ints_core(benchmark):
     benchmark(v.validate_python, frozenset_of_ints)
 
 
+@pytest.mark.benchmark(group='FrozenSet[int]')
+def test_frozenset_of_ints_duplicates_core(benchmark):
+    v = SchemaValidator({'type': 'frozenset', 'items_schema': {'type': 'int'}})
+
+    benchmark(v.validate_python, frozenset_of_ints_duplicates)
+
+
 dict_of_ints_data = ({str(i): i for i in range(1000)}, {str(i): str(i) for i in range(1000)})
 
 
@@ -674,7 +678,7 @@ def test_many_models_core_dict(benchmark):
 @pytest.mark.benchmark(group='List[SimpleMode]')
 def test_many_models_core_model(benchmark):
     class MyCoreModel:
-        __slots__ = '__dict__', '__fields_set__'
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
     v = SchemaValidator(
         {
@@ -683,9 +687,8 @@ def test_many_models_core_model(benchmark):
                 'type': 'model',
                 'cls': MyCoreModel,
                 'schema': {
-                    'type': 'typed-dict',
-                    'return_fields_set': True,
-                    'fields': {'age': {'type': 'typed-dict-field', 'schema': {'type': 'int'}}},
+                    'type': 'model-fields',
+                    'fields': {'age': {'type': 'model-field', 'schema': {'type': 'int'}}},
                 },
             },
         }
@@ -742,16 +745,15 @@ class TestBenchmarkDateTime:
     @pytest.fixture(scope='class')
     def core_validator(self):
         class CoreModel:
-            __slots__ = '__dict__', '__fields_set__'
+            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
         return SchemaValidator(
             {
                 'type': 'model',
                 'cls': CoreModel,
                 'schema': {
-                    'type': 'typed-dict',
-                    'return_fields_set': True,
-                    'fields': {'dt': {'type': 'typed-dict-field', 'schema': {'type': 'datetime'}}},
+                    'type': 'model-fields',
+                    'fields': {'dt': {'type': 'model-field', 'schema': {'type': 'datetime'}}},
                 },
             }
         )
@@ -939,6 +941,18 @@ def test_dont_raise_error(benchmark):
         v.validate_python(42)
 
 
+@pytest.mark.benchmark(group='raise-error')
+def test_dont_raise_error_no_info(benchmark):
+    def f(input_value):
+        return input_value
+
+    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'no-info', 'function': f}})
+
+    @benchmark
+    def t():
+        v.validate_python(42)
+
+
 @pytest.mark.benchmark(group='raise-error')
 def test_raise_error_value_error(benchmark):
     def f(input_value, info):
@@ -1024,12 +1038,9 @@ def test_arguments(benchmark):
             ],
         }
     )
-    assert v.validate_python({'__args__': (1, 'a', 'true'), '__kwargs__': {'b': 'bb', 'c': 3}}) == (
-        (1, 'a', True),
-        {'b': 'bb', 'c': 3},
-    )
+    assert v.validate_python(ArgsKwargs((1, 'a', 'true'), {'b': 'bb', 'c': 3})) == ((1, 'a', True), {'b': 'bb', 'c': 3})
 
-    benchmark(v.validate_python, {'__args__': (1, 'a', 'true'), '__kwargs__': {'b': 'bb', 'c': 3}})
+    benchmark(v.validate_python, ArgsKwargs((1, 'a', 'true'), {'b': 'bb', 'c': 3}))
 
 
 @pytest.mark.benchmark(group='defaults')
@@ -1164,14 +1175,14 @@ def test_generator_rust(benchmark):
 
 @pytest.mark.benchmark(group='isinstance-json')
 def test_isinstance_json(benchmark):
-    validator = SchemaValidator(core_schema.is_instance_schema(str, json_types={'str'}))
-    assert validator.isinstance_json('"foo"') is True
-    assert validator.isinstance_json('123') is False
+    validator = SchemaValidator(core_schema.json_or_python_schema(core_schema.str_schema(), core_schema.int_schema()))
+    assert validator.validate_json('"foo"') == 'foo'
+    with pytest.raises(ValidationError):
+        validator.validate_json('123')
 
     @benchmark
     def t():
-        validator.isinstance_json('"foo"')
-        validator.isinstance_json('123')
+        validator.validate_json('"foo"')
 
 
 @pytest.mark.benchmark(group='error')
@@ -1187,6 +1198,7 @@ def test_int_error(benchmark):
                 'loc': (),
                 'msg': 'Input should be a valid integer, unable to parse string as an integer',
                 'input': 'bar',
+                'url': IsStr(regex=r'https://errors.pydantic.dev/.*?/v/int_parsing'),
             }
         ]
     else:
@@ -1222,19 +1234,20 @@ def test_definition_out_of_tree(benchmark):
 @pytest.mark.benchmark(group='model_instance')
 def test_model_instance(benchmark):
     class MyModel:
-        __slots__ = '__dict__', '__fields_set__'
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
         def __init__(self, **d):
             self.__dict__ = d
-            self.__fields_set__ = set(d)
+            self.__pydantic_extra__ = {}
+            self.__pydantic_fields_set__ = set(d)
 
     validator = SchemaValidator(
         core_schema.model_schema(
             MyModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.int_schema()),
                 }
             ),
             revalidate_instances='always',
@@ -1258,12 +1271,13 @@ def test_model_instance_abc(benchmark):
             return hasattr(instance, '__pydantic_validator__') and super().__instancecheck__(instance)
 
     class BaseModel(metaclass=MyMeta):
-        __slots__ = '__dict__', '__fields_set__'
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
         __pydantic_validator__ = True
 
         def __init__(self, **d):
             self.__dict__ = d
-            self.__fields_set__ = set(d)
+            self.__pydantic_extra__ = {}
+            self.__pydantic_fields_set__ = set(d)
 
     class MyModel(BaseModel):
         pass
@@ -1271,10 +1285,10 @@ def test_model_instance_abc(benchmark):
     validator = SchemaValidator(
         core_schema.model_schema(
             MyModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.int_schema()),
                 }
             ),
             revalidate_instances='always',
@@ -1289,3 +1303,80 @@ def test_model_instance_abc(benchmark):
     assert validator.isinstance_python(m1)
 
     benchmark(validator.validate_python, m1)
+
+
+class SomeStrEnum(str, Enum):
+    foo = 'foo_val'
+    bar = 'bar_val'
+    baz = 'baz_val'
+
+
+LARGE_STR_PREFIX = 'a' * 50
+
+
+@pytest.mark.parametrize(
+    'allowed_values,input,expected_val_res',
+    [
+        (list(range(5)), 4, 4),
+        ([f'abc{i}' for i in range(5)], 'abc4', 'abc4'),
+        ([LARGE_STR_PREFIX + f'{i}' for i in range(5)], f'{LARGE_STR_PREFIX}4', f'{LARGE_STR_PREFIX}4'),
+        ([SomeStrEnum.foo, SomeStrEnum.bar], SomeStrEnum.bar, SomeStrEnum.bar),
+        (list(range(100)), 5, 5),
+        ([f'abc{i}' for i in range(100)], 'abc99', 'abc99'),
+        ([LARGE_STR_PREFIX + f'{i}' for i in range(100)], f'{LARGE_STR_PREFIX}99', f'{LARGE_STR_PREFIX}99'),
+        (['null', None, -1, SomeStrEnum.baz], None, None),
+    ],
+    ids=[
+        'few_ints',
+        'few_small_strings',
+        'few_large_strings',
+        'few_str_enum',
+        'many_ints',
+        'many_small_strings',
+        'many_large_strings',
+        'few_mixed',
+    ],
+)
+@pytest.mark.parametrize('py_or_json', ['python', 'json'])
+def test_validate_literal(
+    benchmark: Any, allowed_values: List[Any], input: Any, expected_val_res: Any, py_or_json: str
+) -> None:
+    validator = SchemaValidator(core_schema.literal_schema(expected=allowed_values))
+
+    if py_or_json == 'python':
+        res = validator.validate_python(input)
+        assert res == expected_val_res
+
+        benchmark(validator.validate_python, input)
+    else:
+        input_json = json.dumps(input)
+        res = validator.validate_json(input_json)
+        assert res == expected_val_res
+
+        benchmark(validator.validate_json, input_json)
+
+
+@pytest.mark.benchmark(group='root_model')
+def test_core_root_model(benchmark):
+    class MyModel:
+        __slots__ = 'root'
+        root: List[int]
+
+    v = SchemaValidator(
+        core_schema.model_schema(MyModel, core_schema.list_schema(core_schema.int_schema()), root_model=True)
+    )
+    assert v.validate_python([1, 2, '3']).root == [1, 2, 3]
+    input_data = list(range(100))
+    benchmark(v.validate_python, input_data)
+
+
+@skip_pydantic
+@pytest.mark.benchmark(group='root_model')
+def test_v1_root_model(benchmark):
+    class MyModel(BaseModel):
+        __root__: List[int]
+
+    assert MyModel.parse_obj([1, 2, '3']).__root__ == [1, 2, 3]
+    input_data = list(range(100))
+
+    benchmark(MyModel.parse_obj, input_data)
