@@ -23,23 +23,19 @@ class TestBenchmarkSimpleModel:
     @pytest.fixture(scope='class')
     def core_schema(self):
         class CoreModel:
-            __slots__ = '__dict__', '__fields_set__'
+            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
         return {
             'type': 'model',
             'cls': CoreModel,
             'schema': {
-                'type': 'typed-dict',
-                'return_fields_set': True,
+                'type': 'model-fields',
                 'fields': {
-                    'name': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                    'age': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                    'friends': {
-                        'type': 'typed-dict-field',
-                        'schema': {'type': 'list', 'items_schema': {'type': 'int'}},
-                    },
+                    'name': {'type': 'model-field', 'schema': {'type': 'str'}},
+                    'age': {'type': 'model-field', 'schema': {'type': 'int'}},
+                    'friends': {'type': 'model-field', 'schema': {'type': 'list', 'items_schema': {'type': 'int'}}},
                     'settings': {
-                        'type': 'typed-dict-field',
+                        'type': 'model-field',
                         'schema': {'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'float'}},
                     },
                 },
@@ -194,7 +190,23 @@ def test_date_format_function(benchmark):
 
     serializer = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(fmt, json_return_type='str')
+            serialization=core_schema.plain_serializer_function_ser_schema(fmt, info_arg=True, json_return_type='str')
+        )
+    )
+    d = date(2022, 11, 20)
+    assert serializer.to_python(d) == '2022-11-20'
+
+    benchmark(serializer.to_python, d)
+
+
+@pytest.mark.benchmark(group='date-format')
+def test_date_format_function_no_info(benchmark):
+    def fmt(value):
+        return value.strftime('%Y-%m-%d')
+
+    serializer = SchemaSerializer(
+        core_schema.any_schema(
+            serialization=core_schema.plain_serializer_function_ser_schema(fmt, json_return_type='str')
         )
     )
     d = date(2022, 11, 20)
@@ -245,16 +257,16 @@ def basic_model_serializer_fixture():
     return SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'a': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'b': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'c': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'd': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'e': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'f': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'g': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'h': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'a': core_schema.model_field(core_schema.int_schema()),
+                    'b': core_schema.model_field(core_schema.int_schema()),
+                    'c': core_schema.model_field(core_schema.int_schema()),
+                    'd': core_schema.model_field(core_schema.int_schema()),
+                    'e': core_schema.model_field(core_schema.int_schema()),
+                    'f': core_schema.model_field(core_schema.int_schema()),
+                    'g': core_schema.model_field(core_schema.int_schema()),
+                    'h': core_schema.model_field(core_schema.int_schema()),
                 }
             ),
         )
@@ -268,6 +280,46 @@ def test_core_model_py(benchmark, basic_model_serializer):
     benchmark(basic_model_serializer.to_python, m)
 
 
+@pytest.fixture(scope='module', name='basic_model_serializer_extra')
+def basic_model_serializer_extra_fixture():
+    return SchemaSerializer(
+        core_schema.model_schema(
+            BasicModel,
+            core_schema.model_fields_schema(
+                {
+                    'a': core_schema.model_field(core_schema.int_schema()),
+                    'b': core_schema.model_field(core_schema.int_schema()),
+                    'c': core_schema.model_field(core_schema.int_schema()),
+                    'd': core_schema.model_field(core_schema.int_schema()),
+                    'e': core_schema.model_field(core_schema.int_schema()),
+                    'f': core_schema.model_field(core_schema.int_schema()),
+                    'g': core_schema.model_field(core_schema.int_schema()),
+                    'h': core_schema.model_field(core_schema.int_schema()),
+                },
+                extra_behavior='allow',
+            ),
+            extra_behavior='allow',
+        )
+    )
+
+
+@pytest.mark.benchmark(group='model-python')
+def test_core_model_py_extra(benchmark, basic_model_serializer_extra):
+    m = BasicModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, __pydantic_extra__={'i': 9})
+    assert basic_model_serializer_extra.to_python(m) == {
+        'a': 1,
+        'b': 2,
+        'c': 3,
+        'd': 4,
+        'e': 5,
+        'f': 6,
+        'g': 7,
+        'h': 8,
+        'i': 9,
+    }
+    benchmark(basic_model_serializer_extra.to_python, m)
+
+
 @pytest.mark.benchmark(group='model-json')
 def test_core_model_json(benchmark, basic_model_serializer):
     m = BasicModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8)
@@ -275,8 +327,15 @@ def test_core_model_json(benchmark, basic_model_serializer):
     benchmark(basic_model_serializer.to_json, m)
 
 
+@pytest.mark.benchmark(group='model-json')
+def test_core_model_json_extra(benchmark, basic_model_serializer_extra):
+    m = BasicModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, __pydantic_extra__={'i': 9})
+    assert basic_model_serializer_extra.to_json(m) == b'{"a":1,"b":2,"c":3,"d":4,"e":5,"f":6,"g":7,"h":8,"i":9}'
+    benchmark(basic_model_serializer_extra.to_json, m)
+
+
 class FieldsSetModel:
-    __slots__ = '__dict__', '__fields_set__'
+    __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
     def __init__(self, **kwargs):
         for key, value in kwargs.items():
@@ -288,16 +347,16 @@ def fs_model_serializer_fixture():
     return SchemaSerializer(
         core_schema.model_schema(
             FieldsSetModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'a': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'b': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'c': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'd': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'e': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'f': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'g': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'h': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'a': core_schema.model_field(core_schema.int_schema()),
+                    'b': core_schema.model_field(core_schema.int_schema()),
+                    'c': core_schema.model_field(core_schema.int_schema()),
+                    'd': core_schema.model_field(core_schema.int_schema()),
+                    'e': core_schema.model_field(core_schema.int_schema()),
+                    'f': core_schema.model_field(core_schema.int_schema()),
+                    'g': core_schema.model_field(core_schema.int_schema()),
+                    'h': core_schema.model_field(core_schema.int_schema()),
                 }
             ),
         )
@@ -306,7 +365,7 @@ def fs_model_serializer_fixture():
 
 @pytest.mark.benchmark(group='model-exclude-unset')
 def test_model_exclude_unset_false(benchmark, fs_model_serializer):
-    m = FieldsSetModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, __fields_set__={'a', 'b', 'c', 'd', 'e', 'f'})
+    m = FieldsSetModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, __pydantic_fields_set__={'a', 'b', 'c', 'd', 'e', 'f'})
     assert fs_model_serializer.to_python(m) == {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'e': 5, 'f': 6, 'g': 7, 'h': 8}
     assert fs_model_serializer.to_python(m, exclude_unset=True) == {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'e': 5, 'f': 6}
 
@@ -317,7 +376,7 @@ def test_model_exclude_unset_false(benchmark, fs_model_serializer):
 
 @pytest.mark.benchmark(group='model-exclude-unset')
 def test_model_exclude_unset_true(benchmark, fs_model_serializer):
-    m = FieldsSetModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, __fields_set__={'a', 'b', 'c', 'd', 'e', 'f'})
+    m = FieldsSetModel(a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, __pydantic_fields_set__={'a', 'b', 'c', 'd', 'e', 'f'})
     assert fs_model_serializer.to_python(m) == {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'e': 5, 'f': 6, 'g': 7, 'h': 8}
     assert fs_model_serializer.to_python(m, exclude_unset=True) == {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'e': 5, 'f': 6}
 
@@ -350,9 +409,9 @@ def test_model_list_core_json(benchmark):
     s = SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'a': core_schema.typed_dict_field(
+                    'a': core_schema.model_field(
                         core_schema.list_schema(
                             core_schema.int_schema(), serialization=core_schema.filter_seq_schema(exclude={1, 2})
                         )
