@@ -11,9 +11,10 @@ from typing import ClassVar
 import pytest
 from dirty_equals import HasRepr, IsList
 
-from pydantic_core import PydanticSerializationError, SchemaSerializer, core_schema, to_json
+from pydantic_core import PydanticSerializationError, SchemaSerializer, SchemaValidator, core_schema, to_json
 
 from ..conftest import plain_repr
+from .test_dataclasses import IsStrictDict, on_pypy
 from .test_list_tuple import as_list, as_tuple
 
 
@@ -23,7 +24,7 @@ def any_serializer():
 
 
 def test_repr(any_serializer):
-    assert plain_repr(any_serializer) == 'SchemaSerializer(serializer=Any(AnySerializer),slots=[])'
+    assert plain_repr(any_serializer) == 'SchemaSerializer(serializer=Any(AnySerializer),definitions=[])'
 
 
 @dataclasses.dataclass(frozen=True)
@@ -35,7 +36,7 @@ class MyDataclass:
 
 
 class MyModel:
-    __pydantic_validator__ = 42
+    __pydantic_serializer__ = 42
 
     def __init__(self, **kwargs):
         for key, value in kwargs.items():
@@ -115,7 +116,7 @@ def test_any_json(any_serializer, value, expected_json):
 def test_other_type():
     """Types with no serializer, fall back to any serializer"""
     v = SchemaSerializer(core_schema.is_instance_schema(int))
-    assert plain_repr(v) == 'SchemaSerializer(serializer=Any(AnySerializer),slots=[])'
+    assert plain_repr(v) == 'SchemaSerializer(serializer=Any(AnySerializer),definitions=[])'
     assert v.to_json('foobar') == b'"foobar"'
 
 
@@ -242,8 +243,8 @@ def test_include_dict(any_serializer):
 
 
 class FieldsSetModel:
-    __pydantic_validator__ = 42
-    __slots__ = '__dict__', '__fields_set__'
+    __pydantic_serializer__ = 42
+    __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
     def __init__(self, **kwargs):
         for key, value in kwargs.items():
@@ -252,7 +253,7 @@ class FieldsSetModel:
 
 def test_exclude_unset(any_serializer):
     # copied from test of the same name in test_model.py
-    m = FieldsSetModel(foo=1, bar=2, spam=3, __fields_set__={'bar', 'spam'})
+    m = FieldsSetModel(foo=1, bar=2, spam=3, __pydantic_fields_set__={'bar', 'spam'})
     assert any_serializer.to_python(m) == {'foo': 1, 'bar': 2, 'spam': 3}
     assert any_serializer.to_python(m, exclude_unset=True) == {'bar': 2, 'spam': 3}
     assert any_serializer.to_python(m, exclude=None, exclude_unset=True) == {'bar': 2, 'spam': 3}
@@ -265,7 +266,7 @@ def test_exclude_unset(any_serializer):
     assert any_serializer.to_json(m, exclude={'bar': ...}, exclude_unset=True) == b'{"spam":3}'
     assert any_serializer.to_json(m, exclude={'bar': {}}, exclude_unset=True) == b'{"bar":2,"spam":3}'
 
-    m2 = FieldsSetModel(foo=1, bar=2, spam=3, __fields_set__={'bar', 'spam', 'missing'})
+    m2 = FieldsSetModel(foo=1, bar=2, spam=3, __pydantic_fields_set__={'bar', 'spam', 'missing'})
     assert any_serializer.to_python(m2) == {'foo': 1, 'bar': 2, 'spam': 3}
     assert any_serializer.to_python(m2, exclude_unset=True) == {'bar': 2, 'spam': 3}
 
@@ -412,3 +413,63 @@ def test_encoding(any_serializer, gen_input, kwargs, expected_json):
     assert to_json(gen_input(), **kwargs) == expected_json
     if not kwargs:
         assert any_serializer.to_python(gen_input(), mode='json') == json.loads(expected_json)
+
+
+def test_any_dataclass():
+    @dataclasses.dataclass
+    class Foo:
+        a: str
+        b: bytes
+
+    # Build a schema that does not include the field 'b', to test that it is not serialized
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo', [core_schema.dataclass_field(name='a', schema=core_schema.str_schema())]
+        ),
+    )
+    Foo.__pydantic_serializer__ = SchemaSerializer(schema)
+
+    s = SchemaSerializer(core_schema.any_schema())
+    assert s.to_python(Foo(a='hello', b=b'more')) == IsStrictDict(a='hello')
+    assert s.to_python(Foo(a='hello', b=b'more'), mode='json') == IsStrictDict(a='hello')
+    j = s.to_json(Foo(a='hello', b=b'more'))
+
+    if on_pypy:
+        assert json.loads(j) == {'a': 'hello'}
+    else:
+        assert j == b'{"a":"hello"}'
+
+    assert s.to_python(Foo(a='hello', b=b'more'), exclude={'a'}) == IsStrictDict()
+
+
+def test_any_model():
+    class Foo:
+        a: str
+        b: bytes
+
+        def __init__(self, a: str, b: bytes):
+            self.a = a
+            self.b = b
+
+    # Build a schema that does not include the field 'b', to test that it is not serialized
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo', [core_schema.dataclass_field(name='a', schema=core_schema.str_schema())]
+        ),
+    )
+    Foo.__pydantic_validator__ = SchemaValidator(schema)
+    Foo.__pydantic_serializer__ = SchemaSerializer(schema)
+
+    s = SchemaSerializer(core_schema.any_schema())
+    assert s.to_python(Foo(a='hello', b=b'more')) == IsStrictDict(a='hello')
+    assert s.to_python(Foo(a='hello', b=b'more'), mode='json') == IsStrictDict(a='hello')
+    j = s.to_json(Foo(a='hello', b=b'more'))
+
+    if on_pypy:
+        assert json.loads(j) == {'a': 'hello'}
+    else:
+        assert j == b'{"a":"hello"}'
+
+    assert s.to_python(Foo(a='hello', b=b'more'), exclude={'a'}) == IsStrictDict()
