@@ -5,7 +5,14 @@ from pathlib import Path
 
 import pytest
 
-from pydantic_core import PydanticOmit, PydanticSerializationError, SchemaError, SchemaSerializer, core_schema
+from pydantic_core import (
+    PydanticOmit,
+    PydanticSerializationError,
+    PydanticSerializationUnexpectedValue,
+    SchemaError,
+    SchemaSerializer,
+    core_schema,
+)
 
 
 def repr_function(value, _info):
@@ -16,9 +23,28 @@ def repr_function(value, _info):
     'value,expected_python,expected_json',
     [(None, 'None', b'"None"'), (1, '1', b'"1"'), ([1, 2, 3], '[1, 2, 3]', b'"[1, 2, 3]"')],
 )
-def test_function(value, expected_python, expected_json):
+def test_function_general(value, expected_python, expected_json):
     s = SchemaSerializer(
-        core_schema.any_schema(serialization=core_schema.general_plain_serializer_function_ser_schema(repr_function))
+        core_schema.any_schema(
+            serialization=core_schema.plain_serializer_function_ser_schema(repr_function, info_arg=True)
+        )
+    )
+    assert s.to_python(value) == expected_python
+    assert s.to_json(value) == expected_json
+    assert s.to_python(value, mode='json') == json.loads(expected_json)
+
+
+def repr_function_no_info(value):
+    return repr(value)
+
+
+@pytest.mark.parametrize(
+    'value,expected_python,expected_json',
+    [(None, 'None', b'"None"'), (1, '1', b'"1"'), ([1, 2, 3], '[1, 2, 3]', b'"[1, 2, 3]"')],
+)
+def test_function_no_info(value, expected_python, expected_json):
+    s = SchemaSerializer(
+        core_schema.any_schema(serialization=core_schema.plain_serializer_function_ser_schema(repr_function_no_info))
     )
     assert s.to_python(value) == expected_python
     assert s.to_json(value) == expected_json
@@ -34,7 +60,7 @@ def test_function_args():
         return value * 2
 
     s = SchemaSerializer(
-        core_schema.any_schema(serialization=core_schema.general_plain_serializer_function_ser_schema(double))
+        core_schema.any_schema(serialization=core_schema.plain_serializer_function_ser_schema(double, info_arg=True))
     )
     assert s.to_python(4) == 8
     # insert_assert(f_info)
@@ -100,7 +126,9 @@ def test_function_error():
         raise TypeError('foo')
 
     s = SchemaSerializer(
-        core_schema.any_schema(serialization=core_schema.general_plain_serializer_function_ser_schema(raise_error))
+        core_schema.any_schema(
+            serialization=core_schema.plain_serializer_function_ser_schema(raise_error, info_arg=True)
+        )
     )
 
     msg = 'Error calling function `raise_error`: TypeError: foo$'
@@ -122,7 +150,9 @@ def test_function_error_keys():
 
     s = SchemaSerializer(
         core_schema.dict_schema(
-            core_schema.any_schema(serialization=core_schema.general_plain_serializer_function_ser_schema(raise_error)),
+            core_schema.any_schema(
+                serialization=core_schema.plain_serializer_function_ser_schema(raise_error, info_arg=True)
+            ),
             core_schema.int_schema(),
         )
     )
@@ -148,7 +178,9 @@ def test_function_known_type():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(append_42, json_return_type='list')
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                append_42, info_arg=True, json_return_type='list'
+            )
         )
     )
     assert s.to_python([1, 2, 3]) == [1, 2, 3, 42]
@@ -171,7 +203,9 @@ def test_function_args_str():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(append_args, json_return_type='str')
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                append_args, info_arg=True, json_return_type='str'
+            )
         )
     )
     assert s.to_python(123) == (
@@ -197,11 +231,11 @@ def test_function_args_str():
 
 
 def test_invalid_return_type():
-    with pytest.raises(SchemaError, match='function-plain -> json_return_type\n  Input should be'):
+    with pytest.raises(SchemaError, match=r'function-plain\.json_return_type\n  Input should be'):
         SchemaSerializer(
             core_schema.any_schema(
-                serialization=core_schema.general_plain_serializer_function_ser_schema(
-                    lambda _: 1, json_return_type='different'
+                serialization=core_schema.plain_serializer_function_ser_schema(
+                    lambda _: 1, info_arg=True, json_return_type='different'
                 )
             )
         )
@@ -213,7 +247,7 @@ def test_dict_keys():
 
     s = SchemaSerializer(
         core_schema.dict_schema(
-            core_schema.int_schema(serialization=core_schema.general_plain_serializer_function_ser_schema(fmt))
+            core_schema.int_schema(serialization=core_schema.plain_serializer_function_ser_schema(fmt, info_arg=True))
         )
     )
     assert s.to_python({1: True}) == {'<1>': True}
@@ -223,7 +257,7 @@ def test_function_as_key():
     s = SchemaSerializer(
         core_schema.dict_schema(
             core_schema.any_schema(
-                serialization=core_schema.general_plain_serializer_function_ser_schema(repr_function)
+                serialization=core_schema.plain_serializer_function_ser_schema(repr_function, info_arg=True)
             ),
             core_schema.any_schema(),
         )
@@ -239,7 +273,7 @@ def test_function_only_json():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(double, when_used='json')
+            serialization=core_schema.plain_serializer_function_ser_schema(double, info_arg=True, when_used='json')
         )
     )
     assert s.to_python(4) == 4
@@ -252,8 +286,8 @@ def test_function_only_json():
 def test_function_unless_none():
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(
-                repr_function, when_used='unless-none'
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                repr_function, info_arg=True, when_used='unless-none'
             )
         )
     )
@@ -269,8 +303,8 @@ def test_function_unless_none():
 def test_wrong_return_type():
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(
-                repr_function, json_return_type='int'
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                repr_function, info_arg=True, json_return_type='int'
             )
         )
     )
@@ -291,7 +325,9 @@ def test_wrong_return_type_str():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(f, json_return_type='str_subclass')
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                f, info_arg=True, json_return_type='str_subclass'
+            )
         )
     )
     assert s.to_python(123) == '123'
@@ -313,20 +349,32 @@ def test_function_wrap():
         return f'result={serializer(len(value))} repr={serializer!r}'
 
     s = SchemaSerializer(
-        core_schema.int_schema(serialization=core_schema.general_wrap_serializer_function_ser_schema(f))
+        core_schema.int_schema(serialization=core_schema.wrap_serializer_function_ser_schema(f, info_arg=True))
     )
     assert s.to_python('foo') == 'result=3 repr=SerializationCallable(serializer=int)'
     assert s.to_python('foo', mode='json') == 'result=3 repr=SerializationCallable(serializer=int)'
     assert s.to_json('foo') == b'"result=3 repr=SerializationCallable(serializer=int)"'
 
 
+def test_function_wrap_no_info():
+    def f(value, serializer):
+        return f'result={serializer(len(value))} repr={serializer!r}'
+
+    s = SchemaSerializer(core_schema.int_schema(serialization=core_schema.wrap_serializer_function_ser_schema(f)))
+    assert s.to_python('foo') == 'result=3 repr=SerializationCallable(serializer=int)'
+    assert s.to_python('foo', mode='json') == 'result=3 repr=SerializationCallable(serializer=int)'
+    assert s.to_json('foo') == b'"result=3 repr=SerializationCallable(serializer=int)"'
+
+
 def test_function_wrap_custom_schema():
     def f(value, serializer, _info):
         return f'result={serializer(len(value))} repr={serializer!r}'
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(f, schema=core_schema.int_schema())
+            serialization=core_schema.wrap_serializer_function_ser_schema(
+                f, info_arg=True, schema=core_schema.int_schema()
+            )
         )
     )
     assert s.to_python('foo') == 'result=3 repr=SerializationCallable(serializer=int)'
@@ -348,7 +396,9 @@ def test_function_wrap_fallback():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(f, schema=core_schema.any_schema())
+            serialization=core_schema.wrap_serializer_function_ser_schema(
+                f, info_arg=True, schema=core_schema.any_schema()
+            )
         )
     )
     assert s.to_python('foo') == 'result=foo'
@@ -383,8 +433,8 @@ def test_deque():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(
-                serialize_deque, schema=core_schema.any_schema()
+            serialization=core_schema.wrap_serializer_function_ser_schema(
+                serialize_deque, info_arg=True, schema=core_schema.any_schema()
             )
         )
     )
@@ -411,8 +461,8 @@ def test_custom_mapping():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(
-                serialize_custom_mapping, schema=core_schema.int_schema()
+            serialization=core_schema.wrap_serializer_function_ser_schema(
+                serialize_custom_mapping, info_arg=True, schema=core_schema.int_schema()
             )
         )
     )
@@ -446,7 +496,7 @@ def test_function_wrap_model():
                     'c': core_schema.typed_dict_field(core_schema.any_schema(), serialization_exclude=True),
                 }
             ),
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(wrap_function),
+            serialization=core_schema.wrap_serializer_function_ser_schema(wrap_function, info_arg=True),
         )
     )
     m = MyModel(a=1, b=b'foobar', c='excluded')
@@ -488,7 +538,7 @@ def test_function_plain_model():
                     'c': core_schema.typed_dict_field(core_schema.any_schema(), serialization_exclude=True),
                 }
             ),
-            serialization=core_schema.general_plain_serializer_function_ser_schema(wrap_function),
+            serialization=core_schema.plain_serializer_function_ser_schema(wrap_function, info_arg=True),
         )
     )
     m = MyModel(a=1, b=b'foobar', c='not excluded')
@@ -515,9 +565,68 @@ def test_wrap_return_type():
 
     s = SchemaSerializer(
         core_schema.str_schema(
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(to_path, json_return_type='path')
+            serialization=core_schema.wrap_serializer_function_ser_schema(
+                to_path, info_arg=True, json_return_type='path'
+            )
         )
     )
     assert s.to_python('foobar') == Path('foobar.new')
     assert s.to_python('foobar', mode='json') == 'foobar.new'
     assert s.to_json('foobar') == b'"foobar.new"'
+
+
+def test_unexpected_value():
+    def return_xxx(_value):
+        return 'xxx'
+
+    s = SchemaSerializer(
+        core_schema.any_schema(
+            serialization=core_schema.plain_serializer_function_ser_schema(return_xxx, json_return_type='int')
+        )
+    )
+    assert s.to_python('foo') == 'xxx'
+
+    m = "Error serializing to JSON: TypeError: 'str' object cannot be interpreted as an integer"
+    with pytest.raises(PydanticSerializationError, match=m):
+        s.to_json('foo')
+
+
+def test_raise_unexpected():
+    def raise_unexpected(_value):
+        raise PydanticSerializationUnexpectedValue('unexpected')
+
+    s = SchemaSerializer(
+        core_schema.any_schema(serialization=core_schema.plain_serializer_function_ser_schema(raise_unexpected))
+    )
+    with pytest.warns(UserWarning, match=r'PydanticSerializationUnexpectedValue\(unexpected\)'):
+        assert s.to_python('foo') == 'foo'
+
+    with pytest.warns(UserWarning, match=r'PydanticSerializationUnexpectedValue\(unexpected\)'):
+        assert s.to_json('foo') == b'"foo"'
+
+
+def test_pydantic_serialization_unexpected_value():
+    v = PydanticSerializationUnexpectedValue('abc')
+    assert str(v) == 'abc'
+    assert repr(v) == 'PydanticSerializationUnexpectedValue(abc)'
+    v = PydanticSerializationUnexpectedValue()
+    assert str(v) == 'Unexpected Value'
+    assert repr(v) == 'PydanticSerializationUnexpectedValue(Unexpected Value)'
+
+
+def test_function_after_preserves_wrapped_serialization():
+    def f(value, _info):
+        return value
+
+    s = SchemaSerializer(core_schema.general_after_validator_function(f, core_schema.int_schema()))
+    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+        assert s.to_python('abc') == 'abc'
+
+
+def test_function_wrap_preserves_wrapped_serialization():
+    def f(value, handler, _info):
+        return handler(value)
+
+    s = SchemaSerializer(core_schema.general_wrap_validator_function(f, core_schema.int_schema()))
+    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+        assert s.to_python('abc') == 'abc'
