@@ -159,9 +159,6 @@ class RemovedContains(ImplicitContains):
         ({'a': 'dict'}, 'Input should be a valid set'),
         ({4.2}, 'Input should be a valid integer, got a number with a fractional part'),
         ({'a'}, 'Input should be a valid integer, unable to parse string as an integer'),
-        (ImplicitContains(), 'Input should be a valid set'),
-        (ExplicitContains(), re.compile('.*Invalid Schema:.*Input should be a valid set.*', re.DOTALL)),
-        (RemovedContains(), re.compile('.*Invalid Schema:.*Input should be a valid set.*', re.DOTALL)),
     ],
 )
 @pytest.mark.parametrize('schema_func', [core_schema.list_schema, core_schema.tuple_variable_schema])
@@ -333,14 +330,14 @@ def test_function_positional_tuple():
             'type': 'tuple-positional',
             'items_schema': [
                 core_schema.any_schema(
-                    serialization=core_schema.general_plain_serializer_function_ser_schema(partial(f, 'a'))
+                    serialization=core_schema.plain_serializer_function_ser_schema(partial(f, 'a'), info_arg=True)
                 ),
                 core_schema.any_schema(
-                    serialization=core_schema.general_plain_serializer_function_ser_schema(partial(f, 'b'))
+                    serialization=core_schema.plain_serializer_function_ser_schema(partial(f, 'b'), info_arg=True)
                 ),
             ],
             'extra_schema': core_schema.any_schema(
-                serialization=core_schema.general_plain_serializer_function_ser_schema(partial(f, 'extra'))
+                serialization=core_schema.plain_serializer_function_ser_schema(partial(f, 'extra'), info_arg=True)
             ),
         }
     )
