@@ -1,11 +1,20 @@
 import dataclasses
 import json
 import platform
+from random import randint
 from typing import Any, ClassVar
 
-import pytest
+try:
+    from functools import cached_property
+except ImportError:
+    cached_property = None
 
-from pydantic_core import SchemaSerializer, SchemaValidator, core_schema
+import pytest
+from dirty_equals import IsJson
+
+from pydantic_core import PydanticSerializationError, SchemaSerializer, SchemaValidator, core_schema
+
+from ..conftest import plain_repr
 
 on_pypy = platform.python_implementation() == 'PyPy'
 # pypy doesn't seem to maintain order of `__dict__`
@@ -29,14 +38,16 @@ def test_model():
     s = SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.bytes_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.bytes_schema()),
                 }
             ),
         )
     )
+    assert 'mode:SimpleDict' in plain_repr(s)
+    assert 'has_extra:false' in plain_repr(s)
     assert s.to_python(BasicModel(foo=1, bar=b'more')) == IsStrictDict(foo=1, bar=b'more')
     assert s.to_python(BasicSubModel(foo=1, bar=b'more')) == IsStrictDict(foo=1, bar=b'more')
     assert s.to_python(BasicModel(bar=b'more', foo=1)) == IsStrictDict(bar=b'more', foo=1)
@@ -75,11 +86,11 @@ def test_dataclass():
         DataClass,
         serialization=core_schema.model_ser_schema(
             DataClass,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.str_schema()),
-                    'spam': core_schema.typed_dict_field(core_schema.bytes_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.str_schema()),
+                    'spam': core_schema.model_field(core_schema.bytes_schema()),
                 }
             ),
         ),
@@ -104,23 +115,28 @@ def test_model_allow_extra():
     s = SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.bytes_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.bytes_schema()),
                 },
                 extra_behavior='allow',
             ),
+            extra_behavior='allow',
         )
     )
-    assert s.to_python(BasicModel(foo=1, bar=b'more')) == IsStrictDict(foo=1, bar=b'more')
-    assert s.to_python(BasicModel(bar=b'more', foo=1)) == IsStrictDict(bar=b'more', foo=1)
-    assert s.to_python(BasicModel(foo=1, c=3, bar=b'more')) == IsStrictDict(foo=1, c=3, bar=b'more')
-    assert s.to_python(BasicModel(bar=b'more', c=3, foo=1), mode='json') == IsStrictDict(bar='more', c=3, foo=1)
+    assert s.to_python(BasicModel(foo=1, bar=b'more', __pydantic_extra__={})) == IsStrictDict(foo=1, bar=b'more')
+    assert s.to_python(BasicModel(bar=b'more', foo=1, __pydantic_extra__={})) == IsStrictDict(bar=b'more', foo=1)
+    assert s.to_python(BasicModel(foo=1, __pydantic_extra__=dict(c=3), bar=b'more')) == IsStrictDict(
+        foo=1, bar=b'more', c=3
+    )
+    assert s.to_python(BasicModel(bar=b'more', __pydantic_extra__=dict(c=3, foo=1)), mode='json') == IsStrictDict(
+        bar='more', c=3, foo=1
+    )
 
-    j = s.to_json(BasicModel(bar=b'more', foo=1, c=3))
+    j = s.to_json(BasicModel(bar=b'more', foo=1, __pydantic_extra__=dict(c=3)))
     if on_pypy:
-        assert json.loads(j) == {'bar': 'more', 'foo': 1, 'c': 3}
+        assert j == IsJson({'bar': 'more', 'foo': 1, 'c': 3})
     else:
         assert j == b'{"bar":"more","foo":1,"c":3}'
 
@@ -146,12 +162,12 @@ def test_include_exclude_args(params):
     s = SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'a': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'b': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'c': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'd': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'a': core_schema.model_field(core_schema.int_schema()),
+                    'b': core_schema.model_field(core_schema.int_schema()),
+                    'c': core_schema.model_field(core_schema.int_schema()),
+                    'd': core_schema.model_field(core_schema.int_schema()),
                 }
             ),
         )
@@ -169,11 +185,11 @@ def test_alias():
     s = SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'cat': core_schema.typed_dict_field(core_schema.int_schema(), serialization_alias='Meow'),
-                    'dog': core_schema.typed_dict_field(core_schema.int_schema(), serialization_alias='Woof'),
-                    'bird': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'cat': core_schema.model_field(core_schema.int_schema(), serialization_alias='Meow'),
+                    'dog': core_schema.model_field(core_schema.int_schema(), serialization_alias='Woof'),
+                    'bird': core_schema.model_field(core_schema.int_schema()),
                 }
             ),
         )
@@ -186,10 +202,10 @@ def test_model_wrong_warn():
     s = SchemaSerializer(
         core_schema.model_schema(
             type('MyModel', (), {}),
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.bytes_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.bytes_schema()),
                 }
             ),
         )
@@ -213,10 +229,10 @@ def test_exclude_none():
     s = SchemaSerializer(
         core_schema.model_schema(
             BasicModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.nullable_schema(core_schema.int_schema())),
-                    'bar': core_schema.typed_dict_field(core_schema.bytes_schema()),
+                    'foo': core_schema.model_field(core_schema.nullable_schema(core_schema.int_schema())),
+                    'bar': core_schema.model_field(core_schema.bytes_schema()),
                 },
                 extra_behavior='ignore',  # this is the default
             ),
@@ -235,7 +251,7 @@ def test_exclude_none():
 
 
 class FieldsSetModel:
-    __slots__ = '__dict__', '__fields_set__'
+    __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
     def __init__(self, **kwargs):
         for key, value in kwargs.items():
@@ -246,17 +262,17 @@ def test_exclude_unset():
     s = SchemaSerializer(
         core_schema.model_schema(
             FieldsSetModel,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'foo': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'bar': core_schema.typed_dict_field(core_schema.int_schema()),
-                    'spam': core_schema.typed_dict_field(core_schema.int_schema()),
+                    'foo': core_schema.model_field(core_schema.int_schema()),
+                    'bar': core_schema.model_field(core_schema.int_schema()),
+                    'spam': core_schema.model_field(core_schema.int_schema()),
                 },
                 extra_behavior='ignore',  # this is the default
             ),
         )
     )
-    m = FieldsSetModel(foo=1, bar=2, spam=3, __fields_set__={'bar', 'spam'})
+    m = FieldsSetModel(foo=1, bar=2, spam=3, __pydantic_fields_set__={'bar', 'spam'})
     assert s.to_python(m) == {'foo': 1, 'bar': 2, 'spam': 3}
     assert s.to_python(m, exclude_unset=True) == {'bar': 2, 'spam': 3}
     assert s.to_python(m, exclude=None, exclude_unset=True) == {'bar': 2, 'spam': 3}
@@ -269,7 +285,7 @@ def test_exclude_unset():
     assert s.to_json(m, exclude={'bar': ...}, exclude_unset=True) == b'{"spam":3}'
     assert s.to_json(m, exclude={'bar': {}}, exclude_unset=True) == b'{"bar":2,"spam":3}'
 
-    m2 = FieldsSetModel(foo=1, bar=2, spam=3, __fields_set__={'bar', 'spam', 'missing'})
+    m2 = FieldsSetModel(foo=1, bar=2, spam=3, __pydantic_fields_set__={'bar', 'spam', 'missing'})
     assert s.to_python(m2) == {'foo': 1, 'bar': 2, 'spam': 3}
     assert s.to_python(m2, exclude_unset=True) == {'bar': 2, 'spam': 3}
 
@@ -357,10 +373,9 @@ def test_advanced_exclude_nested_lists(exclude, expected):
 
     sub_sub_model_schema = core_schema.model_schema(
         type('SubSubModel', (), {}),
-        core_schema.typed_dict_schema(
+        core_schema.model_fields_schema(
             dict(
-                i=core_schema.typed_dict_field(core_schema.int_schema(), required=True),
-                j=core_schema.typed_dict_field(core_schema.int_schema(), required=True),
+                i=core_schema.model_field(core_schema.int_schema()), j=core_schema.model_field(core_schema.int_schema())
             )
         ),
     )
@@ -371,10 +386,10 @@ def test_advanced_exclude_nested_lists(exclude, expected):
 
     sub_model_schema = core_schema.model_schema(
         type('SubModel', (), {}),
-        core_schema.typed_dict_schema(
+        core_schema.model_fields_schema(
             dict(
-                k=core_schema.typed_dict_field(core_schema.int_schema(), required=True),
-                subsubs=core_schema.typed_dict_field(core_schema.list_schema(sub_sub_model_schema), required=True),
+                k=core_schema.model_field(core_schema.int_schema()),
+                subsubs=core_schema.model_field(core_schema.list_schema(sub_sub_model_schema)),
             )
         ),
     )
@@ -384,9 +399,7 @@ def test_advanced_exclude_nested_lists(exclude, expected):
 
     model_schema = core_schema.model_schema(
         BasicModel,
-        core_schema.typed_dict_schema(
-            dict(subs=core_schema.typed_dict_field(core_schema.list_schema(sub_model_schema), required=True))
-        ),
+        core_schema.model_fields_schema(dict(subs=core_schema.model_field(core_schema.list_schema(sub_model_schema)))),
     )
     v = SchemaValidator(model_schema)
 
@@ -411,11 +424,13 @@ def test_function_plain_field_serializer_to_python():
     s = SchemaSerializer(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'x': core_schema.typed_dict_field(
+                    'x': core_schema.model_field(
                         core_schema.int_schema(
-                            serialization=core_schema.field_plain_serializer_function_ser_schema(Model.ser_x)
+                            serialization=core_schema.plain_serializer_function_ser_schema(
+                                Model.ser_x, is_field_serializer=True, info_arg=True
+                            )
                         )
                     )
                 }
@@ -438,12 +453,12 @@ def test_function_wrap_field_serializer_to_python():
     s = SchemaSerializer(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'x': core_schema.typed_dict_field(
+                    'x': core_schema.model_field(
                         core_schema.int_schema(
-                            serialization=core_schema.field_wrap_serializer_function_ser_schema(
-                                Model.ser_x, schema=core_schema.any_schema()
+                            serialization=core_schema.wrap_serializer_function_ser_schema(
+                                Model.ser_x, is_field_serializer=True, info_arg=True, schema=core_schema.any_schema()
                             )
                         )
                     )
@@ -466,11 +481,13 @@ def test_function_plain_field_serializer_to_json():
     s = SchemaSerializer(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'x': core_schema.typed_dict_field(
+                    'x': core_schema.model_field(
                         core_schema.int_schema(
-                            serialization=core_schema.field_plain_serializer_function_ser_schema(Model.ser_x)
+                            serialization=core_schema.plain_serializer_function_ser_schema(
+                                Model.ser_x, is_field_serializer=True, info_arg=True
+                            )
                         )
                     )
                 }
@@ -493,12 +510,12 @@ def test_function_wrap_field_serializer_to_json():
     s = SchemaSerializer(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'x': core_schema.typed_dict_field(
+                    'x': core_schema.model_field(
                         core_schema.int_schema(
-                            serialization=core_schema.field_wrap_serializer_function_ser_schema(
-                                Model.ser_x, schema=core_schema.any_schema()
+                            serialization=core_schema.wrap_serializer_function_ser_schema(
+                                Model.ser_x, is_field_serializer=True, info_arg=True, schema=core_schema.any_schema()
                             )
                         )
                     )
@@ -507,3 +524,338 @@ def test_function_wrap_field_serializer_to_json():
         )
     )
     assert json.loads(s.to_json(Model(x=1000))) == {'x': '1_000'}
+
+
+def test_property():
+    @dataclasses.dataclass
+    class Model:
+        def __init__(self, **kwargs):
+            for key, value in kwargs.items():
+                setattr(self, key, value)
+
+        @property
+        def area(self) -> bytes:
+            a = self.width * self.height
+            return b'%d' % a
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {
+                    'width': core_schema.model_field(core_schema.int_schema()),
+                    'height': core_schema.model_field(core_schema.int_schema()),
+                },
+                computed_fields=[core_schema.computed_field('area', core_schema.bytes_schema())],
+            ),
+        )
+    )
+    assert s.to_python(Model(width=3, height=4)) == {'width': 3, 'height': 4, 'area': b'12'}
+    assert s.to_python(Model(width=3, height=4), mode='json') == {'width': 3, 'height': 4, 'area': '12'}
+    assert s.to_json(Model(width=3, height=4)) == b'{"width":3,"height":4,"area":"12"}'
+
+
+def test_property_alias():
+    @dataclasses.dataclass
+    class Model:
+        width: int
+        height: int
+
+        @property
+        def area(self) -> int:
+            return self.width * self.height
+
+        @property
+        def volume(self) -> int:
+            return self.area * self.height
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {
+                    'width': core_schema.model_field(core_schema.int_schema()),
+                    'height': core_schema.model_field(core_schema.int_schema()),
+                },
+                computed_fields=[
+                    core_schema.computed_field('area', core_schema.int_schema(), alias='Area'),
+                    core_schema.computed_field('volume', core_schema.int_schema()),
+                ],
+            ),
+        )
+    )
+    assert s.to_python(Model(3, 4)) == {'width': 3, 'height': 4, 'Area': 12, 'volume': 48}
+    assert s.to_python(Model(3, 4), mode='json') == {'width': 3, 'height': 4, 'Area': 12, 'volume': 48}
+    assert s.to_json(Model(3, 4)) == b'{"width":3,"height":4,"Area":12,"volume":48}'
+
+
+@pytest.mark.skipif(cached_property is None, reason='cached_property is not available')
+def test_cached_property_alias():
+    @dataclasses.dataclass
+    class Model:
+        width: int
+        height: int
+
+        @cached_property
+        def area(self) -> int:
+            return self.width * self.height
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {
+                    'width': core_schema.model_field(core_schema.int_schema()),
+                    'height': core_schema.model_field(core_schema.int_schema()),
+                },
+                computed_fields=[core_schema.computed_field('area', core_schema.int_schema())],
+            ),
+        )
+    )
+    assert s.to_python(Model(3, 4)) == {'width': 3, 'height': 4, 'area': 12}
+    assert s.to_python(Model(3, 4), mode='json') == {'width': 3, 'height': 4, 'area': 12}
+    assert s.to_json(Model(3, 4)) == b'{"width":3,"height":4,"area":12}'
+
+
+def test_property_attribute_error():
+    @dataclasses.dataclass
+    class Model:
+        width: int
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {'width': core_schema.model_field(core_schema.int_schema())},
+                computed_fields=[core_schema.computed_field('area', core_schema.bytes_schema())],
+            ),
+        )
+    )
+    with pytest.raises(AttributeError, match="^'Model' object has no attribute 'area'$"):
+        s.to_python(Model(3))
+    with pytest.raises(AttributeError, match="^'Model' object has no attribute 'area'$"):
+        s.to_python(Model(3), mode='json')
+
+    e = "^Error serializing to JSON: AttributeError: 'Model' object has no attribute 'area'$"
+    with pytest.raises(PydanticSerializationError, match=e):
+        s.to_json(Model(3))
+
+
+def test_property_other_error():
+    @dataclasses.dataclass
+    class Model:
+        width: int
+
+        @property
+        def area(self) -> int:
+            raise ValueError('xxx')
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {'width': core_schema.model_field(core_schema.int_schema())},
+                computed_fields=[core_schema.computed_field('area', core_schema.bytes_schema())],
+            ),
+        )
+    )
+    with pytest.raises(ValueError, match='^xxx$'):
+        s.to_python(Model(3))
+
+    with pytest.raises(ValueError, match='^xxx$'):
+        s.to_python(Model(3), mode='json')
+
+    e = '^Error serializing to JSON: ValueError: xxx$'
+    with pytest.raises(PydanticSerializationError, match=e):
+        s.to_json(Model(3))
+
+
+def test_property_include_exclude():
+    @dataclasses.dataclass
+    class Model:
+        a: int
+
+        @property
+        def b(self):
+            return [1, 2, b'3']
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {'a': core_schema.model_field(core_schema.int_schema())},
+                computed_fields=[core_schema.computed_field('b', core_schema.list_schema())],
+            ),
+        )
+    )
+    assert s.to_python(Model(1)) == {'a': 1, 'b': [1, 2, b'3']}
+    assert s.to_python(Model(1), exclude={'b'}) == {'a': 1}
+    assert s.to_python(Model(1), include={'a'}) == {'a': 1}
+    assert s.to_python(Model(1), exclude={'b': [0]}) == {'a': 1, 'b': [2, b'3']}
+
+    assert s.to_python(Model(1), mode='json') == {'a': 1, 'b': [1, 2, '3']}
+    assert s.to_python(Model(1), mode='json', exclude={'b'}) == {'a': 1}
+    assert s.to_python(Model(1), mode='json', include={'a'}) == {'a': 1}
+    assert s.to_python(Model(1), mode='json', exclude={'b': [0]}) == {'a': 1, 'b': [2, '3']}
+
+    assert s.to_json(Model(1)) == b'{"a":1,"b":[1,2,"3"]}'
+    assert s.to_json(Model(1), exclude={'b'}) == b'{"a":1}'
+    assert s.to_json(Model(1), include={'a'}) == b'{"a":1}'
+    assert s.to_json(Model(1), exclude={'b': [0]}) == b'{"a":1,"b":[2,"3"]}'
+
+
+@pytest.mark.skipif(cached_property is None, reason='cached_property is not available')
+def test_property_setter():
+    class Square:
+        side: float
+
+        def __init__(self, **kwargs):
+            self.__dict__ = kwargs
+
+        @property
+        def area(self) -> float:
+            return self.side**2
+
+        @area.setter
+        def area(self, area: float) -> None:
+            self.side = area**0.5
+
+        @area.deleter
+        def area(self) -> None:
+            self.side = 0.0
+
+        @cached_property
+        def random_n(self) -> int:
+            return randint(0, 1_000)
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Square,
+            core_schema.model_fields_schema(
+                {'side': core_schema.model_field(core_schema.float_schema())},
+                computed_fields=[
+                    core_schema.computed_field('area', core_schema.float_schema()),
+                    core_schema.computed_field('random_n', core_schema.int_schema(), alias='The random number'),
+                ],
+            ),
+        )
+    )
+
+    sq = Square(side=10.0)
+    the_random_n = sq.random_n
+    assert s.to_python(sq, by_alias=True) == {'side': 10.0, 'area': 100.0, 'The random number': the_random_n}
+    assert s.to_json(sq, by_alias=True) == b'{"side":10.0,"area":100.0,"The random number":%d}' % the_random_n
+    sq.area = 49.0
+    assert s.to_python(sq, by_alias=False) == {'side': 7, 'area': 49, 'random_n': the_random_n}
+    assert s.to_json(sq, by_alias=False) == b'{"side":7.0,"area":49.0,"random_n":%d}' % the_random_n
+    del sq.area
+    assert s.to_python(sq, by_alias=False) == {'side': 0, 'area': 0, 'random_n': the_random_n}
+    assert s.to_python(sq, exclude={'random_n'}) == {'side': 0, 'area': 0}
+
+
+def test_extra():
+    class MyModel:
+        # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        field_a: str
+        field_b: int
+
+    schema = core_schema.model_schema(
+        MyModel,
+        core_schema.model_fields_schema(
+            {
+                'field_a': core_schema.model_field(core_schema.bytes_schema()),
+                'field_b': core_schema.model_field(core_schema.int_schema()),
+            },
+            extra_behavior='allow',
+        ),
+        extra_behavior='allow',
+    )
+    v = SchemaValidator(schema)
+    m = v.validate_python({'field_a': b'test', 'field_b': 12, 'field_c': 'extra'})
+    assert isinstance(m, MyModel)
+    assert m.__dict__ == {'field_a': b'test', 'field_b': 12}
+    assert m.__pydantic_extra__ == {'field_c': 'extra'}
+    assert m.__pydantic_fields_set__ == {'field_a', 'field_b', 'field_c'}
+
+    s = SchemaSerializer(schema)
+    assert 'mode:ModelExtra' in plain_repr(s)
+    assert 'has_extra:true' in plain_repr(s)
+    assert s.to_python(m) == {'field_a': b'test', 'field_b': 12, 'field_c': 'extra'}
+    assert s.to_python(m, mode='json') == {'field_a': 'test', 'field_b': 12, 'field_c': 'extra'}
+    assert s.to_json(m) == b'{"field_a":"test","field_b":12,"field_c":"extra"}'
+
+    # test filtering
+    m = v.validate_python({'field_a': b'test', 'field_b': 12, 'field_c': None, 'field_d': [1, 2, 3]})
+    assert isinstance(m, MyModel)
+    assert m.__dict__ == {'field_a': b'test', 'field_b': 12}
+    assert m.__pydantic_extra__ == {'field_c': None, 'field_d': [1, 2, 3]}
+    assert m.__pydantic_fields_set__ == {'field_a', 'field_b', 'field_c', 'field_d'}
+
+    assert s.to_python(m) == {'field_a': b'test', 'field_b': 12, 'field_c': None, 'field_d': [1, 2, 3]}
+    assert s.to_json(m) == b'{"field_a":"test","field_b":12,"field_c":null,"field_d":[1,2,3]}'
+
+    assert s.to_python(m, exclude_none=True) == {'field_a': b'test', 'field_b': 12, 'field_d': [1, 2, 3]}
+    assert s.to_json(m, exclude_none=True) == b'{"field_a":"test","field_b":12,"field_d":[1,2,3]}'
+
+    assert s.to_python(m, exclude={'field_c'}) == {'field_a': b'test', 'field_b': 12, 'field_d': [1, 2, 3]}
+    assert s.to_json(m, exclude={'field_c'}) == b'{"field_a":"test","field_b":12,"field_d":[1,2,3]}'
+
+    assert s.to_python(m, exclude={'field_d': [0]}) == {
+        'field_a': b'test',
+        'field_b': 12,
+        'field_c': None,
+        'field_d': [2, 3],
+    }
+    assert s.to_json(m, exclude={'field_d': [0]}) == b'{"field_a":"test","field_b":12,"field_c":null,"field_d":[2,3]}'
+
+
+def test_extra_config():
+    class MyModel:
+        # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        field_a: str
+        field_b: int
+
+    schema = core_schema.model_schema(
+        MyModel,
+        core_schema.model_fields_schema(
+            {
+                'field_a': core_schema.model_field(core_schema.bytes_schema()),
+                'field_b': core_schema.model_field(core_schema.int_schema()),
+            }
+        ),
+        config=core_schema.CoreConfig(extra_fields_behavior='allow'),
+    )
+    s = SchemaSerializer(schema)
+    assert 'mode:ModelExtra' in plain_repr(s)
+    assert 'has_extra:true' in plain_repr(s)
+
+
+def test_extra_config_nested_model():
+    class OuterModel:
+        pass
+
+    class InnerModel:
+        pass
+
+    schema = core_schema.model_schema(
+        OuterModel,
+        core_schema.model_fields_schema(
+            {
+                'sub_model': core_schema.model_field(
+                    core_schema.model_schema(
+                        InnerModel,
+                        core_schema.model_fields_schema({'int': core_schema.model_field(core_schema.int_schema())}),
+                        config=core_schema.CoreConfig(extra_fields_behavior='allow'),
+                    )
+                )
+            }
+        ),
+        config={},
+    )
+    s = SchemaSerializer(schema)
+    # debug(s)
+    s_repr = plain_repr(s)
+    assert 'has_extra:true,root_model:false,name:"InnerModel"' in s_repr
+    assert 'has_extra:false,root_model:false,name:"OuterModel"' in s_repr
