@@ -12,7 +12,7 @@ except ImportError:
         return []
 
 
-@pytest.mark.skipif(sys.platform not in {'linux', 'darwin'}, reason='Only both on linux and macos')
+@pytest.mark.skipif(sys.platform not in {'linux', 'darwin'}, reason='Only on linux and macos')
 @pytest.mark.parametrize('example', find_examples('pydantic_core/core_schema.py'), ids=str)
 def test_docstrings(example: CodeExample, eval_example: EvalExample):
     eval_example.set_config(quotes='single')
@@ -25,7 +25,7 @@ def test_docstrings(example: CodeExample, eval_example: EvalExample):
         eval_example.run_print_check(example)
 
 
-@pytest.mark.skipif(sys.platform not in {'linux', 'darwin'}, reason='Only both on linux and macos')
+@pytest.mark.skipif(sys.platform not in {'linux', 'darwin'}, reason='Only on linux and macos')
 @pytest.mark.parametrize('example', find_examples('README.md'), ids=str)
 def test_readme(example: CodeExample, eval_example: EvalExample):
     eval_example.set_config(line_length=100, quotes='single')
