@@ -1,9 +1,19 @@
+import json
 import re
+from typing import List
 
 import pytest
 from dirty_equals import IsList
 
-from pydantic_core import PydanticSerializationError, SchemaValidator, ValidationError, to_json, to_jsonable_python
+from pydantic_core import (
+    PydanticSerializationError,
+    SchemaSerializer,
+    SchemaValidator,
+    ValidationError,
+    core_schema,
+    to_json,
+    to_jsonable_python,
+)
 
 from .conftest import Err
 
@@ -92,7 +102,7 @@ def test_float(input_value, expected):
         assert v.validate_json(input_value) == expected
 
 
-def test_model():
+def test_typed_dict():
     v = SchemaValidator(
         {
             'type': 'typed-dict',
@@ -106,6 +116,10 @@ def test_model():
     # language=json
     input_str = '{"field_a": "abc", "field_b": 1}'
     assert v.validate_json(input_str) == {'field_a': 'abc', 'field_b': 1}
+    # language=json
+    input_str = '{"field_a": "a", "field_a": "b", "field_b": 1}'
+    assert v.validate_json(input_str) == {'field_a': 'b', 'field_b': 1}
+    assert v.validate_json(input_str) == {'field_a': 'b', 'field_b': 1}
 
 
 def test_float_no_remainder():
@@ -117,7 +131,6 @@ def test_error_loc():
     v = SchemaValidator(
         {
             'type': 'typed-dict',
-            'return_fields_set': True,
             'fields': {
                 'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'list', 'items_schema': {'type': 'int'}}}
             },
@@ -130,7 +143,7 @@ def test_error_loc():
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json('{"field_a": [1, 2, "wrong"]}')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'int_parsing',
             'loc': ('field_a', 2),
@@ -144,6 +157,10 @@ def test_dict():
     v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'int'}, 'values_schema': {'type': 'int'}})
     assert v.validate_json('{"1": 2, "3": 4}') == {1: 2, 3: 4}
 
+    # duplicate keys, the last value wins, like with python
+    assert json.loads('{"1": 1, "1": 2}') == {'1': 2}
+    assert v.validate_json('{"1": 1, "1": 2}') == {1: 2}
+
 
 def test_dict_any_value():
     v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'str'}})
@@ -155,7 +172,7 @@ def test_json_invalid():
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json('"foobar')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'json_invalid',
             'loc': (),
@@ -166,7 +183,7 @@ def test_json_invalid():
     ]
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json('[1,\n2,\n3,]')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'json_invalid',
             'loc': (),
@@ -221,6 +238,40 @@ def test_to_jsonable_python_fallback():
     assert to_jsonable_python(Foobar(), fallback=fallback_func) == 'fallback:Foobar'
 
 
+def test_to_jsonable_python_schema_serializer():
+    class Foobar:
+        def __init__(self, my_foo: int, my_inners: List['Foobar']):
+            self.my_foo = my_foo
+            self.my_inners = my_inners
+
+    # force a recursive model to ensure we exercise the transfer of definitions from the loaded
+    # serializer
+    c = core_schema.model_schema(
+        Foobar,
+        core_schema.typed_dict_schema(
+            {
+                'my_foo': core_schema.typed_dict_field(core_schema.int_schema(), serialization_alias='myFoo'),
+                'my_inners': core_schema.typed_dict_field(
+                    core_schema.list_schema(core_schema.definition_reference_schema('foobar')),
+                    serialization_alias='myInners',
+                ),
+            }
+        ),
+        ref='foobar',
+    )
+    v = SchemaValidator(c)
+    s = SchemaSerializer(c)
+
+    Foobar.__pydantic_validator__ = v
+    Foobar.__pydantic_serializer__ = s
+
+    instance = Foobar(my_foo=1, my_inners=[Foobar(my_foo=2, my_inners=[])])
+    assert to_jsonable_python(instance) == {'myFoo': 1, 'myInners': [{'myFoo': 2, 'myInners': []}]}
+    assert to_jsonable_python(instance, by_alias=False) == {'my_foo': 1, 'my_inners': [{'my_foo': 2, 'my_inners': []}]}
+    assert to_json(instance) == b'{"myFoo":1,"myInners":[{"myFoo":2,"myInners":[]}]}'
+    assert to_json(instance, by_alias=False) == b'{"my_foo":1,"my_inners":[{"my_foo":2,"my_inners":[]}]}'
+
+
 def test_cycle_same():
     def fallback_func_passthrough(obj):
         return obj
