@@ -16,7 +16,7 @@ def make_5():
 
 
 class MyModel:
-    __slots__ = '__dict__', '__fields_set__'
+    __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
 
 
 def ids_function(val):
@@ -164,6 +164,16 @@ all_schema_functions = [
         args({'foo': core_schema.typed_dict_field({'type': 'int'})}),
         {'type': 'typed-dict', 'fields': {'foo': {'type': 'typed-dict-field', 'schema': {'type': 'int'}}}},
     ),
+    (
+        core_schema.model_field,
+        args({'type': 'int'}, validation_alias='foobar'),
+        {'type': 'model-field', 'schema': {'type': 'int'}, 'validation_alias': 'foobar'},
+    ),
+    (
+        core_schema.model_fields_schema,
+        args({'foo': core_schema.model_field({'type': 'int'})}),
+        {'type': 'model-fields', 'fields': {'foo': {'type': 'model-field', 'schema': {'type': 'int'}}}},
+    ),
     (
         core_schema.model_schema,
         args(MyModel, {'type': 'int'}),
@@ -215,6 +225,11 @@ all_schema_functions = [
         args({'type': 'int'}, {'type': 'int'}),
         {'type': 'lax-or-strict', 'lax_schema': {'type': 'int'}, 'strict_schema': {'type': 'int'}},
     ),
+    (
+        core_schema.json_or_python_schema,
+        args({'type': 'int'}, {'type': 'str'}),
+        {'type': 'json-or-python', 'json_schema': {'type': 'int'}, 'python_schema': {'type': 'str'}},
+    ),
     (core_schema.is_subclass_schema, args(MyModel), {'type': 'is-subclass', 'cls': MyModel}),
     (
         core_schema.definitions_schema,
@@ -245,7 +260,7 @@ def test_schema_functions(function, args_kwargs, expected_schema):
     args, kwargs = args_kwargs
     schema = function(*args, **kwargs)
     assert schema == expected_schema
-    if schema.get('type') in {None, 'definition-ref', 'typed-dict-field'}:
+    if schema.get('type') in {None, 'definition-ref', 'typed-dict-field', 'model-field'}:
         return
 
     v = SchemaValidator(schema)
@@ -267,6 +282,7 @@ def test_all_schema_functions_used():
 
     # isn't a CoreSchema type
     types_used.remove('typed-dict-field')
+    types_used.remove('model-field')
 
     assert all_types == types_used
 
@@ -294,8 +310,8 @@ def repr_function(value, _info):
 def test_expected_serialization_types(json_return_type):
     SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(
-                repr_function, json_return_type=json_return_type
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                repr_function, info_arg=True, json_return_type=json_return_type
             )
         )
     )
