@@ -81,14 +81,15 @@ def test_schema_typing() -> None:
     SchemaValidator(schema)
     schema: CoreSchema = {'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'any'}}
     SchemaValidator(schema)
+    schema: CoreSchema = {
+        'type': 'typed-dict',
+        'fields': {'bar': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
+    }
+    SchemaValidator(schema)
     schema: CoreSchema = {
         'type': 'model',
         'cls': Foo,
-        'schema': {
-            'type': 'typed-dict',
-            'return_fields_set': True,
-            'fields': {'bar': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
-        },
+        'schema': {'type': 'model-fields', 'fields': {'bar': {'type': 'model-field', 'schema': {'type': 'str'}}}},
     }
     SchemaValidator(schema)
     schema: CoreSchema = {
@@ -219,7 +220,7 @@ def test_ser_function_plain():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_plain_serializer_function_ser_schema(f, json_return_type='str')
+            serialization=core_schema.plain_serializer_function_ser_schema(f, info_arg=True, json_return_type='str')
         )
     )
     assert s.to_python(123) == (
@@ -236,8 +237,8 @@ def test_ser_function_wrap():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.general_wrap_serializer_function_ser_schema(
-                f, schema=core_schema.str_schema(), when_used='json'
+            serialization=core_schema.wrap_serializer_function_ser_schema(
+                f, info_arg=True, schema=core_schema.str_schema(), when_used='json'
             )
         )
     )
@@ -259,5 +260,5 @@ def test_error_details() -> None:
     try:
         v.validate_python('not an int')
     except ValidationError as err:
-        for details in err.errors():
+        for details in err.errors(include_url=False):
             act_on_error_details(details)
