@@ -1,6 +1,6 @@
 import dataclasses
 import re
-from typing import Any, Dict, Union
+from typing import Any, Dict, List, Optional, Union
 
 import pytest
 from dirty_equals import IsListOrTuple, IsStr
@@ -10,17 +10,6 @@ from pydantic_core import ArgsKwargs, SchemaValidator, ValidationError, core_sch
 from ..conftest import Err, PyAndJson
 
 
-def test_args_kwargs():
-    ak = ArgsKwargs(('hello', True))
-    assert repr(ak) == "ArgsKwargs(args=('hello', True), kwargs={})"
-    assert ak.args == ('hello', True)
-    assert ak.kwargs is None
-    ak2 = ArgsKwargs((), {'a': 123})
-    assert repr(ak2) == "ArgsKwargs(args=(), kwargs={'a': 123})"
-    assert ak2.args == ()
-    assert ak2.kwargs == {'a': 123}
-
-
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -46,15 +35,8 @@ def test_args_kwargs():
         (
             {'a': 'hello'},
             Err(
-                'Missing required keyword argument',
-                errors=[
-                    {
-                        'type': 'missing_keyword_argument',
-                        'loc': ('b',),
-                        'msg': 'Missing required keyword argument',
-                        'input': {'a': 'hello'},
-                    }
-                ],
+                'Field required',
+                errors=[{'type': 'missing', 'loc': ('b',), 'msg': 'Field required', 'input': {'a': 'hello'}}],
             ),
         ),
     ],
@@ -72,9 +54,9 @@ def test_dataclass_args(py_and_json: PyAndJson, input_value, expected):
         with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
             v.validate_test(input_value)
 
-        # debug(exc_info.value.errors())
+        # debug(exc_info.value.errors(include_url=False))
         if expected.errors is not None:
-            assert exc_info.value.errors() == expected.errors
+            assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -103,15 +85,8 @@ def test_dataclass_args(py_and_json: PyAndJson, input_value, expected):
         (
             {'a': 'hello'},
             Err(
-                'Missing required keyword argument',
-                errors=[
-                    {
-                        'type': 'missing_keyword_argument',
-                        'loc': ('b',),
-                        'msg': 'Missing required keyword argument',
-                        'input': {'a': 'hello'},
-                    }
-                ],
+                'Field required',
+                errors=[{'type': 'missing', 'loc': ('b',), 'msg': 'Field required', 'input': {'a': 'hello'}}],
             ),
         ),
         (
@@ -145,9 +120,9 @@ def test_dataclass_args_init_only(py_and_json: PyAndJson, input_value, expected)
         with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
             v.validate_test(input_value)
 
-        # debug(exc_info.value.errors())
+        # debug(exc_info.value.errors(include_url=False))
         if expected.errors is not None:
-            assert exc_info.value.errors() == expected.errors
+            assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -184,9 +159,9 @@ def test_dataclass_args_init_only_no_fields(py_and_json: PyAndJson, input_value,
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_test(input_value)
 
-        # debug(exc_info.value.errors())
+        # debug(exc_info.value.errors(include_url=False))
         if expected.errors is not None:
-            assert exc_info.value.errors() == expected.errors
+            assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -239,8 +214,8 @@ def test_dataclass():
     with pytest.raises(ValidationError, match='Input should be an instance of FooDataclass') as exc_info:
         v.validate_python({'a': 'hello', 'b': True}, strict=True)
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'model_class_type',
             'loc': (),
@@ -311,9 +286,9 @@ def test_dataclass_subclass(revalidate_instances, input_value, expected):
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             print(v.validate_python(input_value))
 
-        # debug(exc_info.value.errors())
+        # debug(exc_info.value.errors(include_url=False))
         if expected.errors is not None:
-            assert exc_info.value.errors() == expected.errors
+            assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         dc = v.validate_python(input_value)
         assert dataclasses.is_dataclass(dc)
@@ -714,8 +689,8 @@ def test_dataclass_self_init_alias():
     with pytest.raises(ValidationError) as exc_info:
         Foo(aAlias=b'hello', bAlias=['wrong'])
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'bool_parsing',
             'loc': ('bAlias', 0),
@@ -754,8 +729,8 @@ def test_dataclass_self_init_alias_field_name():
     with pytest.raises(ValidationError) as exc_info:
         Foo(aAlias=b'hello', bAlias=['wrong'])
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'bool_parsing',
             'loc': ('b',),
@@ -822,14 +797,14 @@ def test_dataclass_validate_assignment():
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_assignment(foo, 'a', 123)
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'string_type', 'loc': ('a',), 'msg': 'Input should be a valid string', 'input': 123}
     ]
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_assignment(foo, 'c', '123')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'no_such_attribute',
             'loc': ('c',),
@@ -908,8 +883,8 @@ def test_frozen():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_assignment(m, 'f', 'y')
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'frozen_instance', 'loc': (), 'msg': 'Instance is frozen', 'input': 'y'}
     ]
 
@@ -934,8 +909,10 @@ def test_frozen_field():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_assignment(m, 'f', 'y')
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [{'type': 'frozen_field', 'loc': ('f',), 'msg': 'Field is frozen', 'input': 'y'}]
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {'type': 'frozen_field', 'loc': ('f',), 'msg': 'Field is frozen', 'input': 'y'}
+    ]
 
 
 @pytest.mark.parametrize(
@@ -973,7 +950,7 @@ def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], sche
     with pytest.raises(ValidationError) as exc_info:
         v.validate_assignment(m, 'not_f', 'xyz')
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'no_such_attribute',
             'loc': ('not_f',),
@@ -1018,7 +995,7 @@ def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], sche
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_assignment(m, 'not_f', 'xyz')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'no_such_attribute',
             'loc': ('not_f',),
@@ -1064,3 +1041,152 @@ def test_extra_behavior_allow(config: Union[core_schema.CoreConfig, None], schem
 
     v.validate_assignment(m, 'not_f', '123')
     assert getattr(m, 'not_f') == '123'
+
+
+def test_function_validator_wrapping_args_schema_after() -> None:
+    calls: List[Any] = []
+
+    def func(*args: Any) -> Any:
+        calls.append(args)
+        return args[0]
+
+    @dataclasses.dataclass
+    class Model:
+        number: int = 1
+
+    cs = core_schema.dataclass_schema(
+        Model,
+        core_schema.no_info_after_validator_function(
+            func,
+            core_schema.dataclass_args_schema(
+                'Model', [core_schema.dataclass_field('number', core_schema.int_schema())]
+            ),
+        ),
+    )
+
+    v = SchemaValidator(cs)
+
+    instance: Model = v.validate_python({'number': 1})
+    assert instance.number == 1
+    assert calls == [(({'number': 1}, None),)]
+    v.validate_assignment(instance, 'number', 2)
+    assert instance.number == 2
+    assert calls == [(({'number': 1}, None),), (({'number': 2}, None),)]
+
+
+def test_function_validator_wrapping_args_schema_before() -> None:
+    calls: List[Any] = []
+
+    def func(*args: Any) -> Any:
+        calls.append(args)
+        return args[0]
+
+    @dataclasses.dataclass
+    class Model:
+        number: int = 1
+
+    cs = core_schema.dataclass_schema(
+        Model,
+        core_schema.no_info_before_validator_function(
+            func,
+            core_schema.dataclass_args_schema(
+                'Model', [core_schema.dataclass_field('number', core_schema.int_schema())]
+            ),
+        ),
+    )
+
+    v = SchemaValidator(cs)
+
+    instance: Model = v.validate_python({'number': 1})
+    assert instance.number == 1
+    assert calls == [({'number': 1},)]
+    v.validate_assignment(instance, 'number', 2)
+    assert instance.number == 2
+    assert calls == [({'number': 1},), ({'number': 2},)]
+
+
+def test_function_validator_wrapping_args_schema_wrap() -> None:
+    calls: List[Any] = []
+
+    def func(*args: Any) -> Any:
+        assert len(args) == 2
+        input, handler = args
+        output = handler(input)
+        calls.append((input, output))
+        return output
+
+    @dataclasses.dataclass
+    class Model:
+        number: int = 1
+
+    cs = core_schema.dataclass_schema(
+        Model,
+        core_schema.no_info_wrap_validator_function(
+            func,
+            core_schema.dataclass_args_schema(
+                'Model', [core_schema.dataclass_field('number', core_schema.int_schema())]
+            ),
+        ),
+    )
+
+    v = SchemaValidator(cs)
+
+    instance: Model = v.validate_python({'number': 1})
+    assert instance.number == 1
+    assert calls == [({'number': 1}, ({'number': 1}, None))]
+    v.validate_assignment(instance, 'number', 2)
+    assert instance.number == 2
+    assert calls == [({'number': 1}, ({'number': 1}, None)), ({'number': 2}, ({'number': 2}, None))]
+
+
+@dataclasses.dataclass
+class FooParentDataclass:
+    foo: Optional[FooDataclass]
+
+
+def test_custom_dataclass_names():
+    # Note: normally you would use the same values for DataclassArgsSchema.dataclass_name and DataclassSchema.cls_name,
+    # but I have purposely made them different here to show which parts of the errors are affected by which.
+    # I have used square brackets in the names to hint that the most likely reason for using a value different from
+    # cls.__name__ is for use with generic types.
+    schema = core_schema.dataclass_schema(
+        FooParentDataclass,
+        core_schema.dataclass_args_schema(
+            'FooParentDataclass',
+            [
+                core_schema.dataclass_field(
+                    name='foo',
+                    schema=core_schema.union_schema(
+                        [
+                            core_schema.dataclass_schema(
+                                FooDataclass,
+                                core_schema.dataclass_args_schema(
+                                    'FooDataclass[dataclass_args_schema]',
+                                    [
+                                        core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+                                        core_schema.dataclass_field(name='b', schema=core_schema.bool_schema()),
+                                    ],
+                                ),
+                                cls_name='FooDataclass[cls_name]',
+                            ),
+                            core_schema.none_schema(),
+                        ]
+                    ),
+                )
+            ],
+        ),
+    )
+
+    v = SchemaValidator(schema)
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python({'foo': 123})
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'ctx': {'dataclass_name': 'FooDataclass[dataclass_args_schema]'},
+            'input': 123,
+            'loc': ('foo', 'FooDataclass[cls_name]'),
+            'msg': 'Input should be a dictionary or an instance of FooDataclass[dataclass_args_schema]',
+            'type': 'dataclass_type',
+        },
+        {'input': 123, 'loc': ('foo', 'none'), 'msg': 'Input should be None', 'type': 'none_required'},
+    ]
