@@ -156,7 +156,7 @@ def test_date_strict_json_ctx():
     v = SchemaValidator({'type': 'date', 'strict': True})
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json('"foobar"')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'date_parsing',
             'loc': (),
@@ -200,7 +200,7 @@ def test_date_kwargs(kwargs: Dict[str, Any], input_value, expected):
 
 
 def test_invalid_constraint():
-    with pytest.raises(SchemaError, match='date -> gt\n  Input should be a valid date or datetime'):
+    with pytest.raises(SchemaError, match=r'date\.gt\n  Input should be a valid date or datetime'):
         SchemaValidator({'type': 'date', 'gt': 'foobar'})
 
 
