@@ -1,3 +1,4 @@
+import copy
 import json
 import platform
 import re
@@ -185,6 +186,22 @@ def test_tz_comparison():
         SchemaValidator({'type': 'datetime', 'gt': uk_3pm}).validate_python('2022-01-01T16:00:00+01:00')
 
 
+def test_tz_info_deepcopy():
+    output = SchemaValidator({'type': 'datetime'}).validate_python('2023-02-15T16:23:44.037Z')
+    c = copy.deepcopy(output)
+    assert repr(output.tzinfo) == 'TzInfo(UTC)'
+    assert repr(c.tzinfo) == 'TzInfo(UTC)'
+    assert c == output
+
+
+def test_tz_info_copy():
+    output = SchemaValidator({'type': 'datetime'}).validate_python('2023-02-15T16:23:44.037Z')
+    c = copy.copy(output)
+    assert repr(output.tzinfo) == 'TzInfo(UTC)'
+    assert repr(c.tzinfo) == 'TzInfo(UTC)'
+    assert c == output
+
+
 def test_custom_tz():
     class CustomTz(tzinfo):
         def utcoffset(self, _dt):
@@ -223,7 +240,7 @@ def test_custom_invalid_tz():
     else:
         error_message = 'NotImplementedError: a tzinfo subclass must implement utcoffset()'
 
-    assert excinfo.value.errors() == [
+    assert excinfo.value.errors(include_url=False) == [
         {
             'type': 'datetime_object_invalid',
             'loc': (),
@@ -261,7 +278,7 @@ def test_union():
 
 
 def test_invalid_constraint():
-    with pytest.raises(SchemaError, match='datetime -> gt\n  Input should be a valid datetime'):
+    with pytest.raises(SchemaError, match=r'datetime\.gt\n  Input should be a valid datetime'):
         SchemaValidator({'type': 'datetime', 'gt': 'foobar'})
 
 
