@@ -1,9 +1,10 @@
-from typing import Optional
+from dataclasses import dataclass
+from typing import List, Optional
 
 import pytest
-from dirty_equals import AnyThing, HasAttributes, IsInstance, IsList, IsPartialDict, IsStr, IsTuple
+from dirty_equals import AnyThing, HasAttributes, IsList, IsPartialDict, IsStr, IsTuple
 
-from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import SchemaError, SchemaValidator, ValidationError, __version__, core_schema
 
 from ..conftest import Err, plain_repr
 from .test_typed_dict import Cls
@@ -27,13 +28,12 @@ def test_branch_nullable():
             },
         }
     )
-    assert 'return_fields_set:false' in plain_repr(v)
 
     assert v.validate_python({'name': 'root'}) == {'name': 'root', 'sub_branch': None}
     assert plain_repr(v).startswith(
         'SchemaValidator(title="typed-dict",validator=DefinitionRef(DefinitionRefValidator{'
     )
-    assert ',slots=[TypedDict(TypedDictValidator{' in plain_repr(v)
+    assert ',definitions=[TypedDict(TypedDictValidator{' in plain_repr(v)
 
     assert v.validate_python({'name': 'root', 'sub_branch': {'name': 'b1'}}) == (
         {'name': 'root', 'sub_branch': {'name': 'b1', 'sub_branch': None}}
@@ -78,7 +78,7 @@ def test_branch_nullable_definitions():
     assert v.validate_python({'name': 'root', 'sub_branch': {'name': 'b1', 'sub_branch': {'name': 'b2'}}}) == (
         {'name': 'root', 'sub_branch': {'name': 'b1', 'sub_branch': {'name': 'b2', 'sub_branch': None}}}
     )
-    assert ',slots=[TypedDict(TypedDictValidator{' in plain_repr(v)
+    assert ',definitions=[TypedDict(TypedDictValidator{' in plain_repr(v)
 
 
 def test_unused_ref():
@@ -92,9 +92,7 @@ def test_unused_ref():
             },
         }
     )
-    assert plain_repr(v).startswith('SchemaValidator(title="typed-dict",validator=TypedDict(TypedDictValidator')
     assert v.validate_python({'name': 'root', 'other': '4'}) == {'name': 'root', 'other': 4}
-    assert ',slots=[]' in plain_repr(v)
 
 
 def test_nullable_error():
@@ -123,7 +121,7 @@ def test_nullable_error():
     )
     with pytest.raises(ValidationError) as exc_info:
         assert v.validate_python({'width': 123, 'sub_branch': {'width': 'wrong'}})
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'none_required',
             'loc': ('sub_branch', 'none'),
@@ -166,7 +164,7 @@ def test_list():
             'branches': [{'width': 2, 'branches': None}, {'width': 3, 'branches': [{'width': 4, 'branches': None}]}],
         }
     )
-    assert ',slots=[TypedDict(TypedDictValidator{' in plain_repr(v)
+    assert ',definitions=[TypedDict(TypedDictValidator{' in plain_repr(v)
 
 
 def test_multiple_intertwined():
@@ -237,8 +235,8 @@ def test_multiple_intertwined():
 
 def test_model_class():
     class Branch:
-        # this is not required, but it avoids `__fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__fields_set__'
+        # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
         # these are here just as decoration
         width: int
         branch: Optional['Branch']
@@ -249,12 +247,11 @@ def test_model_class():
             'ref': 'Branch',
             'cls': Branch,
             'schema': {
-                'type': 'typed-dict',
-                'return_fields_set': True,
+                'type': 'model-fields',
                 'fields': {
-                    'width': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
+                    'width': {'type': 'model-field', 'schema': {'type': 'int'}},
                     'branch': {
-                        'type': 'typed-dict-field',
+                        'type': 'model-field',
                         'schema': {
                             'type': 'default',
                             'schema': {
@@ -270,14 +267,14 @@ def test_model_class():
     )
     m1: Branch = v.validate_python({'width': '1'})
     assert isinstance(m1, Branch)
-    assert m1.__fields_set__ == {'width'}
+    assert m1.__pydantic_fields_set__ == {'width'}
     assert m1.__dict__ == {'width': 1, 'branch': None}
     assert m1.width == 1
     assert m1.branch is None
 
     m2: Branch = v.validate_python({'width': '10', 'branch': {'width': 20}})
     assert isinstance(m2, Branch)
-    assert m2.__fields_set__ == {'width', 'branch'}
+    assert m2.__pydantic_fields_set__ == {'width', 'branch'}
     assert m2.width == 10
     assert isinstance(m2.branch, Branch)
     assert m2.branch.width == 20
@@ -285,7 +282,7 @@ def test_model_class():
 
 
 def test_invalid_schema():
-    with pytest.raises(SchemaError, match="Slots Error: ref 'Branch' not found"):
+    with pytest.raises(SchemaError, match='Definitions error: attempted to use `Branch` before it was filled'):
         SchemaValidator(
             {
                 'type': 'list',
@@ -332,8 +329,6 @@ def test_outside_parent():
         'tuple1': (1, 1, 'frog'),
         'tuple2': (2, 2, 'toad'),
     }
-    # the definition goes into reusable and gets "inlined" into the schema
-    assert ',slots=[]' in plain_repr(v)
 
 
 def test_recursion_branch():
@@ -355,7 +350,7 @@ def test_recursion_branch():
         },
         {'from_attributes': True},
     )
-    assert ',slots=[TypedDict(TypedDictValidator{' in plain_repr(v)
+    assert ',definitions=[TypedDict(TypedDictValidator{' in plain_repr(v)
 
     assert v.validate_python({'name': 'root'}) == {'name': 'root', 'branch': None}
     assert v.validate_python({'name': 'root', 'branch': {'name': 'b1', 'branch': None}}) == {
@@ -363,16 +358,12 @@ def test_recursion_branch():
         'branch': {'name': 'b1', 'branch': None},
     }
 
-    data = Cls(name='root')
-    data.branch = Cls(name='b1', branch=None)
-    assert v.validate_python(data) == {'name': 'root', 'branch': {'name': 'b1', 'branch': None}}
-
     b = {'name': 'recursive'}
     b['branch'] = b
     with pytest.raises(ValidationError) as exc_info:
         assert v.validate_python(b)
     assert exc_info.value.title == 'typed-dict'
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': ('branch',),
@@ -381,11 +372,45 @@ def test_recursion_branch():
         }
     ]
 
+
+def test_recursion_branch_from_attributes():
+    v = SchemaValidator(
+        {
+            'type': 'model-fields',
+            'ref': 'Branch',
+            'fields': {
+                'name': {'type': 'model-field', 'schema': {'type': 'str'}},
+                'branch': {
+                    'type': 'model-field',
+                    'schema': {
+                        'type': 'default',
+                        'schema': {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 'Branch'}},
+                        'default': None,
+                    },
+                },
+            },
+        },
+        {'from_attributes': True},
+    )
+
+    assert v.validate_python({'name': 'root'}) == ({'name': 'root', 'branch': None}, None, {'name'})
+    model_dict, model_extra, fields_set = v.validate_python({'name': 'root', 'branch': {'name': 'b1', 'branch': None}})
+    assert model_dict == {'name': 'root', 'branch': ({'name': 'b1', 'branch': None}, None, {'name', 'branch'})}
+    assert model_extra is None
+    assert fields_set == {'name', 'branch'}
+
+    data = Cls(name='root')
+    data.branch = Cls(name='b1', branch=None)
+    model_dict, model_extra, fields_set = v.validate_python(data)
+    assert model_dict == {'name': 'root', 'branch': ({'name': 'b1', 'branch': None}, None, {'name', 'branch'})}
+    assert model_extra is None
+    assert fields_set == {'name', 'branch'}
+
     data = Cls(name='root')
     data.branch = data
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(data)
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': ('branch',),
@@ -399,7 +424,7 @@ def test_definition_list():
     v = SchemaValidator(
         {'type': 'list', 'ref': 'the-list', 'items_schema': {'type': 'definition-ref', 'schema_ref': 'the-list'}}
     )
-    assert ',slots=[List(ListValidator{' in plain_repr(v)
+    assert ',definitions=[List(ListValidator{' in plain_repr(v)
     assert v.validate_python([]) == []
     assert v.validate_python([[]]) == [[]]
 
@@ -408,7 +433,7 @@ def test_definition_list():
     with pytest.raises(ValidationError) as exc_info:
         assert v.validate_python(data)
     assert exc_info.value.title == 'list[...]'
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': (0,),
@@ -457,11 +482,8 @@ def multiple_tuple_schema() -> SchemaValidator:
             {'f1': [1, (3, None)], 'f2': [2, (4, (4, (5, None)))]},
             {'f1': (1, (3, None)), 'f2': (2, (4, (4, (5, None))))},
         ),
-        ({'f1': [1, 2]}, Err(r'f1 -> 1\s+Input should be a valid tuple')),
-        (
-            {'f1': [1, (3, None)], 'f2': [2, (4, (4, (5, 6)))]},
-            Err(r'f2 -> 1 -> 1 -> 1 -> 1\s+Input should be a valid tuple'),
-        ),
+        ({'f1': [1, 2]}, Err(r'f1.1\s+Input should be a valid tuple')),
+        ({'f1': [1, (3, None)], 'f2': [2, (4, (4, (5, 6)))]}, Err(r'f2.1.1.1.1\s+Input should be a valid tuple')),
     ],
 )
 def test_multiple_tuple_param(multiple_tuple_schema: SchemaValidator, input_value, expected):
@@ -487,7 +509,7 @@ def test_multiple_tuple_recursion(multiple_tuple_schema: SchemaValidator):
     with pytest.raises(ValidationError) as exc_info:
         multiple_tuple_schema.validate_python({'f1': data, 'f2': data})
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': ('f1', 1),
@@ -509,7 +531,7 @@ def test_multiple_tuple_recursion_once(multiple_tuple_schema: SchemaValidator):
     with pytest.raises(ValidationError) as exc_info:
         multiple_tuple_schema.validate_python({'f1': data, 'f2': data})
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': ('f1', 1),
@@ -549,7 +571,7 @@ def test_definition_wrap():
     t.append(t)
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(t)
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': (1,),
@@ -581,7 +603,7 @@ def test_union_ref_strictness():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'a': 1, 'b': []})
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'int_type', 'loc': ('b', 'int'), 'msg': 'Input should be a valid integer', 'input': []},
         {'type': 'string_type', 'loc': ('b', 'str'), 'msg': 'Input should be a valid string', 'input': []},
     ]
@@ -606,7 +628,7 @@ def test_union_container_strictness():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'a': 1, 'b': []})
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'int_type', 'loc': ('b', 'int'), 'msg': 'Input should be a valid integer', 'input': []},
         {'type': 'string_type', 'loc': ('b', 'str'), 'msg': 'Input should be a valid string', 'input': []},
     ]
@@ -642,7 +664,7 @@ def test_union_cycle(strict: bool):
 
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python(data)
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': ('typed-dict', 'foobar', 0),
@@ -676,7 +698,7 @@ def test_function_name():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('input value')
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': ('function-after[f(), ...]',),
@@ -717,7 +739,7 @@ def test_function_change_id(strict: bool):
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('start-0')
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'recursion_loop',
             'loc': IsTuple(length=(1, 255)),
@@ -754,7 +776,7 @@ def test_many_uses_of_ref():
         'other_names': ['Bob', 'Charlie'],
     }
 
-    with pytest.raises(ValidationError, match=r'other_names -> 2\s+String should have at most 8 characters'):
+    with pytest.raises(ValidationError, match=r'other_names.2\s+String should have at most 8 characters'):
         v.validate_python({'name': 'Anne', 'other_names': ['Bob', 'Charlie', 'Daveeeeee']})
 
     long_input = {'name': 'Anne', 'other_names': [f'p-{i}' for i in range(300)]}
@@ -790,46 +812,102 @@ def test_error_inside_definition_wrapper():
     )
 
 
-def test_model_td_recursive():
-    class Foobar:
-        __slots__ = '__dict__', '__fields_set__'
-
-    v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'ref': '__main__.Foobar',
-            'return_fields_set': True,
-            'fields': {
-                'x': {'type': 'typed-dict-field', 'schema': {'type': 'int'}, 'required': True},
-                'y': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {
-                            'type': 'union',
-                            'choices': [
-                                {
-                                    'type': 'model',
-                                    'cls': Foobar,
-                                    'schema': {'type': 'definition-ref', 'schema_ref': '__main__.Foobar'},
-                                },
-                                {'type': 'none'},
-                            ],
-                        },
-                        'default': None,
-                    },
-                    'required': False,
+def test_recursive_definitions_schema() -> None:
+    s = core_schema.definitions_schema(
+        core_schema.definition_reference_schema(schema_ref='a'),
+        [
+            core_schema.typed_dict_schema(
+                {
+                    'b': core_schema.typed_dict_field(
+                        core_schema.list_schema(core_schema.definition_reference_schema('b'))
+                    )
                 },
-            },
-        }
+                ref='a',
+            ),
+            core_schema.typed_dict_schema(
+                {
+                    'a': core_schema.typed_dict_field(
+                        core_schema.list_schema(core_schema.definition_reference_schema('a'))
+                    )
+                },
+                ref='b',
+            ),
+        ],
     )
-    assert 'return_fields_set:true' in plain_repr(v)
-    d, fields_set = v.validate_python(dict(x=1, y={'x': 2}))
-    assert d == {'x': 1, 'y': IsInstance(Foobar)}
-    assert fields_set == {'y', 'x'}
 
-    f = d['y']
-    assert isinstance(f, Foobar)
-    assert f.x == 2
-    assert f.y is None
-    assert f.__fields_set__ == {'x'}
+    v = SchemaValidator(s)
+
+    assert v.validate_python({'b': [{'a': []}]}) == {'b': [{'a': []}]}
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python({'b': [{'a': {}}]})
+
+    assert exc_info.value.errors() == [
+        {
+            'type': 'list_type',
+            'loc': ('b', 0, 'a'),
+            'msg': 'Input should be a valid list',
+            'input': {},
+            'url': f'https://errors.pydantic.dev/{__version__}/v/list_type',
+        }
+    ]
+
+
+def test_unsorted_definitions_schema() -> None:
+    s = core_schema.definitions_schema(
+        core_schema.definition_reference_schema(schema_ref='td'),
+        [
+            core_schema.typed_dict_schema(
+                {'x': core_schema.typed_dict_field(core_schema.definition_reference_schema('int'))}, ref='td'
+            ),
+            core_schema.int_schema(ref='int'),
+        ],
+    )
+
+    v = SchemaValidator(s)
+
+    assert v.validate_python({'x': 123}) == {'x': 123}
+
+    with pytest.raises(ValidationError):
+        v.validate_python({'x': 'abc'})
+
+
+def test_validate_assignment() -> None:
+    @dataclass
+    class Model:
+        x: List['Model']
+
+    schema = core_schema.dataclass_schema(
+        Model,
+        core_schema.dataclass_args_schema(
+            'Model',
+            [
+                core_schema.dataclass_field(
+                    name='x',
+                    schema=core_schema.list_schema(core_schema.definition_reference_schema('model')),
+                    kw_only=False,
+                )
+            ],
+        ),
+        ref='model',
+    )
+    v = SchemaValidator(schema, config=core_schema.CoreConfig(revalidate_instances='always'))
+
+    data = [Model(x=[Model(x=[])])]
+    instance = Model(x=[])
+    v.validate_assignment(instance, 'x', data)
+    assert instance.x == data
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(instance, 'x', [Model(x=[Model(x=[Model(x=[123])])])])
+
+    assert exc_info.value.errors() == [
+        {
+            'type': 'dataclass_type',
+            'loc': ('x', 0, 'x', 0, 'x', 0, 'x', 0),
+            'msg': 'Input should be a dictionary or an instance of Model',
+            'input': 123,
+            'ctx': {'dataclass_name': 'Model'},
+            'url': f'https://errors.pydantic.dev/{__version__}/v/dataclass_type',
+        }
+    ]
