@@ -30,6 +30,17 @@ def test_function_before():
     assert v.validate_python('input value') == 'input value Changed'
 
 
+def test_function_before_no_info():
+    def f(input_value):
+        return input_value + ' Changed'
+
+    v = SchemaValidator(
+        {'type': 'function-before', 'function': {'type': 'no-info', 'function': f}, 'schema': {'type': 'str'}}
+    )
+
+    assert v.validate_python('input value') == 'input value Changed'
+
+
 def test_function_before_raise():
     def f(input_value, info):
         raise ValueError('foobar')
@@ -41,7 +52,7 @@ def test_function_before_raise():
     with pytest.raises(ValidationError) as exc_info:
         assert v.validate_python('input value') == 'input value Changed'
     # debug(str(exc_info.value))
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'value_error',
             'loc': (),
@@ -67,7 +78,7 @@ def test_function_before_error():
     assert v.validate_python('1234') == '1234x'
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('12345')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'string_too_long',
             'loc': (),
@@ -99,7 +110,7 @@ def test_function_before_error_model():
     assert v.validate_python({'my_field': '1234'}) == {'my_field': '1234x'}
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'my_field': '12345'})
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'string_too_long',
             'loc': ('my_field',),
@@ -143,6 +154,17 @@ def test_function_wrap():
     assert v.validate_python('input value') == 'input value Changed'
 
 
+def test_function_wrap_no_info():
+    def f(input_value, validator):
+        return validator(input_value=input_value) + ' Changed'
+
+    v = SchemaValidator(
+        {'type': 'function-wrap', 'function': {'type': 'no-info', 'function': f}, 'schema': {'type': 'str'}}
+    )
+
+    assert v.validate_python('input value') == 'input value Changed'
+
+
 def test_function_wrap_repr():
     def f(input_value, validator, info):
         assert repr(validator) == str(validator)
@@ -167,14 +189,12 @@ def test_function_wrap_str():
 
 
 def test_function_wrap_not_callable():
-    with pytest.raises(
-        SchemaError, match='function-wrap -> function -> typed-dict -> function\n  Input should be callable'
-    ):
+    with pytest.raises(SchemaError, match='function-wrap.function.typed-dict.function\n  Input should be callable'):
         SchemaValidator(
             {'type': 'function-wrap', 'function': {'type': 'general', 'function': []}, 'schema': {'type': 'str'}}
         )
 
-    with pytest.raises(SchemaError, match='function-wrap -> function\n  Field required'):
+    with pytest.raises(SchemaError, match='function-wrap.function\n  Field required'):
         SchemaValidator({'type': 'function-wrap', 'schema': {'type': 'str'}})
 
 
@@ -194,7 +214,7 @@ def test_wrap_error():
     assert v.validate_python('42') == 84
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('wrong')
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'int_parsing',
             'loc': (),
@@ -215,8 +235,8 @@ def test_function_wrap_location():
     assert v.validate_python(4) == 6
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('wrong')
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'int_parsing',
             'loc': ('foo',),
@@ -249,6 +269,17 @@ def test_function_after():
     assert v.validate_python('input value') == 'input value Changed'
 
 
+def test_function_no_info():
+    def f(input_value):
+        return input_value + ' Changed'
+
+    v = SchemaValidator(
+        {'type': 'function-after', 'function': {'type': 'no-info', 'function': f}, 'schema': {'type': 'str'}}
+    )
+
+    assert v.validate_python('input value') == 'input value Changed'
+
+
 def test_function_after_raise():
     def f(input_value, info):
         raise ValueError('foobar')
@@ -260,7 +291,7 @@ def test_function_after_raise():
     with pytest.raises(ValidationError) as exc_info:
         assert v.validate_python('input value') == 'input value Changed'
     # debug(str(exc_info.value))
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'value_error',
             'loc': (),
@@ -322,7 +353,7 @@ def test_config_no_model():
 
 
 def test_function_plain():
-    def f(input_value, info):
+    def f(input_value, _info):
         return input_value * 2
 
     v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
@@ -331,8 +362,18 @@ def test_function_plain():
     assert v.validate_python('x') == 'xx'
 
 
+def test_function_plain_no_info():
+    def f(input_value):
+        return input_value * 2
+
+    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'no-info', 'function': f}})
+
+    assert v.validate_python(1) == 2
+    assert v.validate_python('x') == 'xx'
+
+
 def test_plain_with_schema():
-    with pytest.raises(SchemaError, match='function-plain -> schema\n  Extra inputs are not permitted'):
+    with pytest.raises(SchemaError, match='function-plain.schema\n  Extra inputs are not permitted'):
         SchemaValidator(
             {
                 'type': 'function-plain',
@@ -343,24 +384,33 @@ def test_plain_with_schema():
 
 
 def test_validate_assignment():
-    def f(input_value, info):
-        input_value['more'] = 'foobar'
+    def f(input_value):
+        input_value.more = 'foobar'
         return input_value
 
-    v = SchemaValidator(
-        {
-            'type': 'function-after',
-            'function': {'type': 'general', 'function': f},
-            'schema': {
-                'type': 'typed-dict',
-                'fields': {'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
-            },
-        }
-    )
+    class Model:
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        field_a: str
 
-    m = {'field_a': 'test', 'more': 'foobar'}
-    assert v.validate_python({'field_a': 'test'}) == m
-    assert v.validate_assignment(m, 'field_a', b'abc') == {'field_a': 'abc', 'more': 'foobar'}
+    v = SchemaValidator(
+        core_schema.no_info_after_validator_function(
+            f,
+            core_schema.model_schema(
+                Model, core_schema.model_fields_schema({'field_a': core_schema.model_field(core_schema.str_schema())})
+            ),
+        )
+    )
+    m = v.validate_python({'field_a': 'test'})
+    assert isinstance(m, Model)
+    assert m.field_a == 'test'
+    assert m.__pydantic_fields_set__ == {'field_a'}
+    assert m.__dict__ == {'field_a': 'test', 'more': 'foobar'}
+
+    m2 = Model()
+    m2.field_a = 'test'
+    assert v.validate_assignment(m2, 'field_a', b'abc') is m2
+    assert m2.__dict__ == {'field_a': 'abc', 'more': 'foobar'}
+    assert not hasattr(m2, '__pydantic_fields_set__')
 
 
 def test_function_wrong_sig():
@@ -411,7 +461,7 @@ def test_class_with_validator():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(True)
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'string_type', 'loc': (), 'msg': 'Input should be a valid string', 'input': True}
     ]
 
@@ -427,7 +477,7 @@ def test_raise_assertion_error():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('input value')
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'assertion_error',
             'loc': (),
@@ -449,7 +499,7 @@ def test_raise_assertion_error_plain():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('input value')
 
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'assertion_error',
             'loc': (),
@@ -493,9 +543,6 @@ def test_model_field_before_validator() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: Any, info: core_schema.FieldValidationInfo) -> Any:
         assert info.field_name == 'x'
         assert info.data == {}
@@ -507,12 +554,8 @@ def test_model_field_before_validator() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
-                {
-                    'x': core_schema.typed_dict_field(
-                        core_schema.field_before_validator_function(f, core_schema.str_schema())
-                    )
-                }
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(core_schema.field_before_validator_function(f, core_schema.str_schema()))}
             ),
         )
     )
@@ -524,9 +567,6 @@ def test_model_field_after_validator() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: str, info: core_schema.FieldValidationInfo) -> Any:
         assert info.field_name == 'x'
         assert info.data == {}
@@ -536,12 +576,8 @@ def test_model_field_after_validator() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
-                {
-                    'x': core_schema.typed_dict_field(
-                        core_schema.field_after_validator_function(f, core_schema.str_schema())
-                    )
-                }
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(core_schema.field_after_validator_function(f, core_schema.str_schema()))}
             ),
         )
     )
@@ -553,9 +589,6 @@ def test_model_field_plain_validator() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: Any, info: core_schema.FieldValidationInfo) -> Any:
         assert info.field_name == 'x'
         assert info.data == {}
@@ -565,8 +598,8 @@ def test_model_field_plain_validator() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
-                {'x': core_schema.typed_dict_field(core_schema.field_plain_validator_function(f))}
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(core_schema.field_plain_validator_function(f))}
             ),
         )
     )
@@ -578,9 +611,6 @@ def test_model_field_wrap_validator() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(
         input_value: Any, val: core_schema.ValidatorFunctionWrapHandler, info: core_schema.FieldValidationInfo
     ) -> Any:
@@ -592,12 +622,8 @@ def test_model_field_wrap_validator() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
-                {
-                    'x': core_schema.typed_dict_field(
-                        core_schema.field_wrap_validator_function(f, core_schema.str_schema())
-                    )
-                }
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(core_schema.field_wrap_validator_function(f, core_schema.str_schema()))}
             ),
         )
     )
@@ -618,9 +644,6 @@ def test_non_model_field_before_validator_tries_to_access_field_info() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: Any, info: core_schema.ValidationInfo) -> Any:
         check_that_info_has_no_model_data(info)
         assert isinstance(input_value, bytes)
@@ -629,9 +652,9 @@ def test_non_model_field_before_validator_tries_to_access_field_info() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'x': core_schema.typed_dict_field(
+                    'x': core_schema.model_field(
                         core_schema.general_before_validator_function(f, core_schema.str_schema())
                     )
                 }
@@ -646,9 +669,6 @@ def test_non_model_field_after_validator_tries_to_access_field_info() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: Any, info: core_schema.ValidationInfo) -> Any:
         check_that_info_has_no_model_data(info)
         return f'input: {input_value}'
@@ -656,9 +676,9 @@ def test_non_model_field_after_validator_tries_to_access_field_info() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
+            core_schema.model_fields_schema(
                 {
-                    'x': core_schema.typed_dict_field(
+                    'x': core_schema.model_field(
                         core_schema.general_after_validator_function(f, core_schema.str_schema())
                     )
                 }
@@ -673,9 +693,6 @@ def test_non_model_field_plain_validator_tries_to_access_field_info() -> None:
     class Model:
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: Any, info: core_schema.ValidationInfo) -> Any:
         check_that_info_has_no_model_data(info)
         assert isinstance(input_value, bytes)
@@ -684,8 +701,8 @@ def test_non_model_field_plain_validator_tries_to_access_field_info() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
-                {'x': core_schema.typed_dict_field(core_schema.general_plain_validator_function(f))}
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(core_schema.general_plain_validator_function(f))}
             ),
         )
     )
@@ -695,11 +712,9 @@ def test_non_model_field_plain_validator_tries_to_access_field_info() -> None:
 
 def test_non_model_field_wrap_validator_tries_to_access_field_info() -> None:
     class Model:
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
         x: str
 
-        def __init__(self, **kwargs: Any) -> None:
-            self.__dict__.update(kwargs)
-
     def f(input_value: Any, val: core_schema.ValidatorFunctionWrapHandler, info: core_schema.ValidationInfo) -> Any:
         check_that_info_has_no_model_data(info)
         return f'input: {val(input_value)}'
@@ -707,12 +722,8 @@ def test_non_model_field_wrap_validator_tries_to_access_field_info() -> None:
     v = SchemaValidator(
         core_schema.model_schema(
             Model,
-            core_schema.typed_dict_schema(
-                {
-                    'x': core_schema.typed_dict_field(
-                        core_schema.general_wrap_validator_function(f, core_schema.str_schema())
-                    )
-                }
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(core_schema.general_wrap_validator_function(f, core_schema.str_schema()))}
             ),
         )
     )
@@ -770,14 +781,20 @@ def test_typed_dict_data() -> None:
     'mode,calls1,calls2',
     [
         ('before', {'value': {'x': b'input', 'y': '123'}}, {'value': {'x': 'different', 'y': 123}}),
-        ('after', {'value': {'x': 'input', 'y': 123}}, {'value': {'x': 'different', 'y': 123}}),
+        (
+            'after',
+            {'value': ({'x': 'input', 'y': 123}, None, {'y', 'x'})},
+            {'value': ({'x': 'different', 'y': 123}, None, {'x'})},
+        ),
         ('wrap', {'value': {'x': b'input', 'y': '123'}}, {'value': {'x': 'different', 'y': 123}}),
     ],
+    ids=('before', 'after', 'wrap'),
 )
 def test_model_root_function_assignment(mode: str, calls1: Any, calls2: Any):
     calls: list[Any] = []
 
     class Model:
+        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
         x: str
         y: int
 
@@ -799,10 +816,10 @@ def test_model_root_function_assignment(mode: str, calls1: Any, calls2: Any):
             {
                 'type': f'function-{mode}',
                 'function': {'type': 'general', 'function': f},
-                'schema': core_schema.typed_dict_schema(
+                'schema': core_schema.model_fields_schema(
                     {
-                        'x': core_schema.typed_dict_field(core_schema.str_schema()),
-                        'y': core_schema.typed_dict_field(core_schema.int_schema()),
+                        'x': core_schema.model_field(core_schema.str_schema()),
+                        'y': core_schema.model_field(core_schema.int_schema()),
                     }
                 ),
             },
@@ -817,3 +834,39 @@ def test_model_root_function_assignment(mode: str, calls1: Any, calls2: Any):
 
     v.validate_assignment(m, 'x', b'different')
     assert calls == [calls1, calls2]
+
+
+def test_function_validation_info_mode():
+    calls: list[str] = []
+
+    def f(v: Any, info: core_schema.ValidationInfo) -> Any:
+        calls.append(info.mode)
+        return v
+
+    v = SchemaValidator(core_schema.general_before_validator_function(f, core_schema.int_schema()))
+    assert v.validate_python(1) == 1
+    assert calls == ['python']
+    calls.clear()
+    assert v.validate_json('1') == 1
+    assert calls == ['json']
+    calls.clear()
+
+    v = SchemaValidator(core_schema.general_after_validator_function(f, core_schema.int_schema()))
+    assert v.validate_python(1) == 1
+    assert calls == ['python']
+    calls.clear()
+    assert v.validate_json('1') == 1
+    assert calls == ['json']
+    calls.clear()
+
+    def f_w(v: Any, handler: core_schema.ValidatorFunctionWrapHandler, info: core_schema.ValidationInfo) -> Any:
+        calls.append(info.mode)
+        return handler(v)
+
+    v = SchemaValidator(core_schema.general_wrap_validator_function(f_w, core_schema.int_schema()))
+    assert v.validate_python(1) == 1
+    assert calls == ['python']
+    calls.clear()
+    assert v.validate_json('1') == 1
+    assert calls == ['json']
+    calls.clear()
