@@ -1,12 +1,13 @@
+import collections.abc
 import re
 from collections import deque
-from collections.abc import Sequence
-from typing import Any, Dict
+from dataclasses import dataclass
+from typing import Any, Dict, Iterator, List, Union
 
 import pytest
-from dirty_equals import HasRepr, IsInstance, IsStr
+from dirty_equals import Contains, HasRepr, IsInstance, IsList, IsStr
 
-from pydantic_core import SchemaValidator, ValidationError
+from pydantic_core import SchemaValidator, ValidationError, core_schema
 
 from ..conftest import Err, PyAndJson, infinite_generator
 
@@ -35,11 +36,16 @@ def test_list_strict():
     assert v.validate_python([1, 2, '33']) == [1, 2, 33]
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python((1, 2, '33'))
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'list_type', 'loc': (), 'msg': 'Input should be a valid list', 'input': (1, 2, '33')}
     ]
 
 
+def test_list_no_copy():
+    v = SchemaValidator({'type': 'list'})
+    assert v.validate_python([1, 2, 3]) is not [1, 2, 3]
+
+
 def gen_ints():
     yield 1
     yield 2
@@ -52,9 +58,9 @@ def gen_ints():
         ([1, 2, '3'], [1, 2, 3]),
         ((1, 2, '3'), [1, 2, 3]),
         (deque((1, 2, '3')), [1, 2, 3]),
-        ({1, 2, '3'}, Err('Input should be a valid list [type=list_type,')),
+        ({1, 2, '3'}, IsList(1, 2, 3, check_order=False)),
         (gen_ints(), [1, 2, 3]),
-        (frozenset({1, 2, '3'}), Err('Input should be a valid list [type=list_type,')),
+        (frozenset({1, 2, '3'}), IsList(1, 2, 3, check_order=False)),
         ({1: 10, 2: 20, '3': '30'}.keys(), [1, 2, 3]),
         ({1: 10, 2: 20, '3': '30'}.values(), [10, 20, 30]),
         ({1: 10, 2: 20, '3': '30'}, Err('Input should be a valid list [type=list_type,')),
@@ -80,8 +86,8 @@ def test_list_json():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json('1')
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'list_type', 'loc': (), 'msg': 'Input should be a valid array', 'input': 1}
     ]
 
@@ -91,11 +97,11 @@ def test_list_json():
     [
         ([], []),
         ([1, '2', b'3'], [1, '2', b'3']),
-        (frozenset([1, '2', b'3']), Err('Input should be a valid list [type=list_type,')),
+        (frozenset([1, '2', b'3']), IsList(1, '2', b'3', check_order=False)),
         ((), []),
         ((1, '2', b'3'), [1, '2', b'3']),
         (deque([1, '2', b'3']), [1, '2', b'3']),
-        ({1, '2', b'3'}, Err('Input should be a valid list [type=list_type,')),
+        ({1, '2', b'3'}, IsList(1, '2', b'3', check_order=False)),
     ],
 )
 def test_list_any(input_value, expected):
@@ -122,7 +128,7 @@ def test_list_error(input_value, index):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(input_value)
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'int_parsing',
             'loc': (index,),
@@ -145,6 +151,11 @@ def test_list_error(input_value, index):
             [1, 2, 3, 4],
             Err('List should have at most 3 items after validation, not 4 [type=too_long,'),
         ),
+        (
+            {'max_length': 3},
+            [1, 2, 3, 4, 5, 6, 7],
+            Err('List should have at most 3 items after validation, not 7 [type=too_long,'),
+        ),
         ({'max_length': 1}, [1, 2], Err('List should have at most 1 item after validation, not 2 [type=too_long,')),
         (
             {'max_length': 44},
@@ -189,8 +200,8 @@ def test_length_ctx():
     v = SchemaValidator({'type': 'list', 'min_length': 2, 'max_length': 3})
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python([1])
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'too_short',
             'loc': (),
@@ -203,8 +214,8 @@ def test_length_ctx():
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python([1, 2, 3, 4])
 
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'too_long',
             'loc': (),
@@ -236,7 +247,7 @@ def test_list_function_val_error():
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python([1, 2])
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'value_error', 'loc': (0,), 'msg': 'Value error, error 1', 'input': 1, 'ctx': {'error': 'error 1'}},
         {'type': 'value_error', 'loc': (1,), 'msg': 'Value error, error 2', 'input': 2, 'ctx': {'error': 'error 2'}},
     ]
@@ -267,7 +278,7 @@ def test_generator_error():
     assert v.validate_python(gen(False)) == [1, 2, 3]
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(gen(True))
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'iteration_error',
             'loc': (2,),
@@ -303,68 +314,6 @@ def test_list_from_dict_items(input_value, items_schema, expected):
     assert output == expected
 
 
-@pytest.fixture(scope='session', name='MySequence')
-def my_sequence():
-    class MySequence(Sequence):
-        def __init__(self):
-            self._data = [1, 2, 3]
-
-        def __getitem__(self, index):
-            return self._data[index]
-
-        def __len__(self):
-            return len(self._data)
-
-        def count(self, value):
-            return self._data.count(value)
-
-    assert isinstance(MySequence(), Sequence)
-    return MySequence
-
-
-def test_sequence(MySequence):
-    v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
-    with pytest.raises(ValidationError) as exc_info:
-        v.validate_python(MySequence())
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
-        {'type': 'list_type', 'loc': (), 'msg': 'Input should be a valid list', 'input': IsInstance(MySequence)}
-    ]
-
-
-@pytest.mark.parametrize(
-    'input_value,expected',
-    [
-        ([1, 2, 3], [1, 2, 3]),
-        ((1, 2, 3), [1, 2, 3]),
-        (range(3), [0, 1, 2]),
-        (gen_ints(), [1, 2, 3]),
-        ({1: 2, 3: 4}, [1, 3]),
-        ('123', [1, 2, 3]),
-        (
-            123,
-            Err(
-                '1 validation error for list[int]',
-                [{'type': 'list_type', 'loc': (), 'msg': 'Input should be a valid list', 'input': 123}],
-            ),
-        ),
-    ],
-)
-def test_allow_any_iter(input_value, expected):
-    v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}, 'allow_any_iter': True})
-    if isinstance(expected, Err):
-        with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
-            v.validate_python(input_value)
-        assert exc_info.value.errors() == expected.errors
-    else:
-        assert v.validate_python(input_value) == expected
-
-
-def test_sequence_allow_any_iter(MySequence):
-    v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}, 'allow_any_iter': True})
-    assert v.validate_python(MySequence()) == [1, 2, 3]
-
-
 @pytest.mark.parametrize('items_schema', ['int', 'any'])
 def test_bad_iter(items_schema):
     class BadIter:
@@ -387,12 +336,12 @@ def test_bad_iter(items_schema):
             else:
                 raise RuntimeError('broken')
 
-    v = SchemaValidator({'type': 'list', 'items_schema': {'type': items_schema}, 'allow_any_iter': True})
+    v = SchemaValidator({'type': 'list', 'items_schema': {'type': items_schema}})
     assert v.validate_python(BadIter(True)) == [1]
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(BadIter(False))
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'iteration_error',
             'loc': (1,),
@@ -401,3 +350,144 @@ def test_bad_iter(items_schema):
             'ctx': {'error': 'RuntimeError: broken'},
         }
     ]
+
+
+@pytest.mark.parametrize('error_in_func', [True, False])
+def test_max_length_fail_fast(error_in_func: bool) -> None:
+    calls: list[int] = []
+
+    def f(v: int) -> int:
+        calls.append(v)
+        if error_in_func:
+            assert v < 10
+        return v
+
+    s = core_schema.list_schema(
+        core_schema.no_info_after_validator_function(f, core_schema.int_schema()), max_length=10
+    )
+
+    v = SchemaValidator(s)
+
+    data = list(range(15))
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python(data)
+
+    assert len(calls) <= 11, len(calls)  # we still run validation on the "extra" item
+
+    assert exc_info.value.errors(include_url=False) == Contains(
+        {
+            'type': 'too_long',
+            'loc': (),
+            'msg': 'List should have at most 10 items after validation, not 11',
+            'input': data,
+            'ctx': {'field_type': 'List', 'max_length': 10, 'actual_length': 11},
+        }
+    )
+
+
+class MySequence(collections.abc.Sequence):
+    def __init__(self, data: List[Any]):
+        self._data = data
+
+    def __getitem__(self, index: int) -> Any:
+        return self._data[index]
+
+    def __len__(self):
+        return len(self._data)
+
+    def __repr__(self) -> str:
+        return f'MySequence({repr(self._data)})'
+
+
+class MyMapping(collections.abc.Mapping):
+    def __init__(self, data: Dict[Any, Any]) -> None:
+        self._data = data
+
+    def __getitem__(self, key: Any) -> Any:
+        return self._data[key]
+
+    def __iter__(self) -> Iterator[Any]:
+        return iter(self._data)
+
+    def __len__(self) -> int:
+        return len(self._data)
+
+    def __repr__(self) -> str:
+        return f'MyMapping({repr(self._data)})'
+
+
+@dataclass
+class ListInputTestCase:
+    input: Any
+    output: Union[Any, Err]
+    strict: Union[bool, None] = None
+
+
+LAX_MODE_INPUTS: List[Any] = [
+    (1, 2, 3),
+    frozenset((1, 2, 3)),
+    set((1, 2, 3)),
+    deque([1, 2, 3]),
+    {1: 'a', 2: 'b', 3: 'c'}.keys(),
+    {'a': 1, 'b': 2, 'c': 3}.values(),
+    MySequence([1, 2, 3]),
+    MyMapping({1: 'a', 2: 'b', 3: 'c'}).keys(),
+    MyMapping({'a': 1, 'b': 2, 'c': 3}).values(),
+    (x for x in [1, 2, 3]),
+]
+
+
+@pytest.mark.parametrize(
+    'testcase',
+    [
+        *[ListInputTestCase([1, 2, 3], [1, 2, 3], strict) for strict in (True, False, None)],
+        *[
+            ListInputTestCase(inp, Err('Input should be a valid list [type=list_type,'), True)
+            for inp in [*LAX_MODE_INPUTS, '123', b'123']
+        ],
+        *[ListInputTestCase(inp, [1, 2, 3], False) for inp in LAX_MODE_INPUTS],
+        *[
+            ListInputTestCase(inp, Err('Input should be a valid list [type=list_type,'), False)
+            for inp in ['123', b'123', MyMapping({1: 'a', 2: 'b', 3: 'c'}), {1: 'a', 2: 'b', 3: 'c'}]
+        ],
+    ],
+    ids=repr,
+)
+def test_list_allowed_inputs_python(testcase: ListInputTestCase):
+    v = SchemaValidator(core_schema.list_schema(core_schema.int_schema(), strict=testcase.strict))
+    if isinstance(testcase.output, Err):
+        with pytest.raises(ValidationError, match=re.escape(testcase.output.message)):
+            v.validate_python(testcase.input)
+    else:
+        output = v.validate_python(testcase.input)
+        assert output == testcase.output
+        assert output is not testcase.input
+
+
+@pytest.mark.parametrize(
+    'testcase',
+    [
+        ListInputTestCase({1: 1, 2: 2, 3: 3}.items(), Err('Input should be a valid list [type=list_type,'), True),
+        ListInputTestCase(
+            MyMapping({1: 1, 2: 2, 3: 3}).items(), Err('Input should be a valid list [type=list_type,'), True
+        ),
+        ListInputTestCase({1: 1, 2: 2, 3: 3}.items(), [(1, 1), (2, 2), (3, 3)], False),
+        ListInputTestCase(MyMapping({1: 1, 2: 2, 3: 3}).items(), [(1, 1), (2, 2), (3, 3)], False),
+    ],
+    ids=repr,
+)
+def test_list_dict_items_input(testcase: ListInputTestCase) -> None:
+    v = SchemaValidator(
+        core_schema.list_schema(
+            core_schema.tuple_positional_schema([core_schema.int_schema(), core_schema.int_schema()]),
+            strict=testcase.strict,
+        )
+    )
+    if isinstance(testcase.output, Err):
+        with pytest.raises(ValidationError, match=re.escape(testcase.output.message)):
+            v.validate_python(testcase.input)
+    else:
+        output = v.validate_python(testcase.input)
+        assert output == testcase.output
+        assert output is not testcase.input
