@@ -108,8 +108,8 @@ def test_simple_tagged_union(py_and_json: PyAndJson, input_value, expected):
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_test(input_value)
-        # debug(exc_info.value.errors())
-        assert exc_info.value.errors() == expected.errors
+        # debug(exc_info.value.errors(include_url=False))
+        assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -184,8 +184,8 @@ def test_int_choice_keys(py_and_json: PyAndJson, input_value, expected):
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_test(input_value)
-        # debug(exc_info.value.errors())
-        assert exc_info.value.errors() == expected.errors
+        # debug(exc_info.value.errors(include_url=False))
+        assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -228,7 +228,7 @@ def test_enum_keys():
     assert v.validate_python({'foo': BarEnum.ONE, 'bar': '123'}) == {'foo': 1, 'bar': 123}
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'foo': FooEnum.APPLE, 'spam': [1, 2, '3']})
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'union_tag_invalid',
             'loc': (),
@@ -266,7 +266,7 @@ def test_discriminator_path(py_and_json: PyAndJson):
     assert v.validate_test({'menu': ['x', 'banana'], 'c': 'C', 'd': [1, '2']}) == {'c': 'C', 'd': [1, 2]}
     with pytest.raises(ValidationError) as exc_info:
         v.validate_test({})
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'union_tag_not_found',
             'loc': (),
@@ -358,8 +358,8 @@ def test_discriminator_function(py_and_json: PyAndJson, input_value, expected):
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_python(input_value)
-        # debug(exc_info.value.errors())
-        assert exc_info.value.errors() == expected.errors
+        # debug(exc_info.value.errors(include_url=False))
+        assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -424,8 +424,8 @@ def test_int_discriminator_function(py_and_json: PyAndJson, input_value, expecte
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_python(input_value)
-        # debug(exc_info.value.errors())
-        assert exc_info.value.errors() == expected.errors
+        # debug(exc_info.value.errors(include_url=False))
+        assert exc_info.value.errors(include_url=False) == expected.errors
     else:
         assert v.validate_test(input_value) == expected
 
@@ -437,27 +437,35 @@ def test_from_attributes():
             'discriminator': 'foobar',
             'choices': {
                 'apple': {
-                    'type': 'typed-dict',
+                    'type': 'model-fields',
                     'fields': {
-                        'a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                        'b': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
+                        'a': {'type': 'model-field', 'schema': {'type': 'str'}},
+                        'b': {'type': 'model-field', 'schema': {'type': 'int'}},
                     },
                 },
                 'banana': {
-                    'type': 'typed-dict',
+                    'type': 'model-fields',
                     'fields': {
-                        'c': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                        'd': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
+                        'c': {'type': 'model-field', 'schema': {'type': 'str'}},
+                        'd': {'type': 'model-field', 'schema': {'type': 'int'}},
                     },
                 },
             },
         },
         {'from_attributes': True},
     )
-    assert v.validate_python({'foobar': 'apple', 'a': 'apple', 'b': '13'}) == {'a': 'apple', 'b': 13}
-    assert v.validate_python(Cls(foobar='apple', a='apple', b='13')) == {'a': 'apple', 'b': 13}
-    assert v.validate_python({'foobar': 'banana', 'c': 'banana', 'd': '31'}) == {'c': 'banana', 'd': 31}
-    assert v.validate_python(Cls(foobar='banana', c='banana', d='31')) == {'c': 'banana', 'd': 31}
+    assert v.validate_python({'foobar': 'apple', 'a': 'apple', 'b': '13'}) == (
+        {'a': 'apple', 'b': 13},
+        None,
+        {'a', 'b'},
+    )
+    assert v.validate_python(Cls(foobar='apple', a='apple', b='13')) == ({'a': 'apple', 'b': 13}, None, {'a', 'b'})
+    assert v.validate_python({'foobar': 'banana', 'c': 'banana', 'd': '31'}) == (
+        {'c': 'banana', 'd': 31},
+        None,
+        {'c', 'd'},
+    )
+    assert v.validate_python(Cls(foobar='banana', c='banana', d='31')) == ({'c': 'banana', 'd': 31}, None, {'c', 'd'})
 
 
 def test_use_ref():
@@ -489,7 +497,7 @@ def test_downcast_error():
     v = SchemaValidator({'type': 'tagged-union', 'discriminator': lambda x: 123, 'choices': {'str': {'type': 'str'}}})
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('x')
-        assert exc_info.value.errors() == [
+        assert exc_info.value.errors(include_url=False) == [
             {
                 'type': 'union_tag_invalid',
                 'loc': (),
@@ -530,13 +538,13 @@ def test_custom_error():
     assert v.validate_python({'foo': 'apple', 'bar': '123'}) == {'foo': 'apple', 'bar': 123}
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'spam': 'apple', 'bar': 'Bar'})
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'snap', 'loc': (), 'msg': 'Input should be a foo or bar', 'input': {'spam': 'apple', 'bar': 'Bar'}}
     ]
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'foo': 'other', 'bar': 'Bar'})
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {'type': 'snap', 'loc': (), 'msg': 'Input should be a foo or bar', 'input': {'foo': 'other', 'bar': 'Bar'}}
     ]
 
@@ -571,8 +579,8 @@ def test_custom_error_type():
     assert v.validate_python({'foo': 'apple', 'bar': '123'}) == {'foo': 'apple', 'bar': 123}
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'spam': 'apple', 'bar': 'Bar'})
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'finite_number',
             'loc': (),
@@ -582,7 +590,7 @@ def test_custom_error_type():
     ]
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python({'foo': 'other', 'bar': 'Bar'})
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'finite_number',
             'loc': (),
@@ -659,11 +667,11 @@ def test_tag_repeated(py_and_json: PyAndJson, input_value, expected):
         }
     )
     if isinstance(expected, Err):
-        # insert_assert(exc_info.value.errors())
+        # insert_assert(exc_info.value.errors(include_url=False))
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_test(input_value)
-        # debug(exc_info.value.errors())
-        assert exc_info.value.errors() == expected.errors
+        # debug(exc_info.value.errors(include_url=False))
+        assert exc_info.value.errors(include_url=False) == expected.errors
 
     else:
         assert v.validate_test(input_value) == expected
