@@ -1,5 +1,6 @@
 import re
-from typing import Optional, Union
+from copy import deepcopy
+from typing import Dict, Optional, Union
 
 import pytest
 from dirty_equals import HasRepr, IsInstance
@@ -70,7 +71,7 @@ def url_test_case_helper(
             else:  # validator_mode == MULTI_URL_CLASS_MODE:
                 MultiHostUrl(url)
         assert exc_info.value.error_count() == 1
-        error = exc_info.value.errors()[0]
+        error = exc_info.value.errors(include_url=False)[0]
         assert error['type'] == 'url_parsing'
         assert error['ctx']['error'] == expected.message
     else:
@@ -391,7 +392,7 @@ def test_url_error(strict_url_validator, url, expected):
         with pytest.raises(ValidationError) as exc_info:
             strict_url_validator.validate_python(url)
         assert exc_info.value.error_count() == 1
-        error = exc_info.value.errors()[0]
+        error = exc_info.value.errors(include_url=False)[0]
         assert error['ctx']['error'] == expected.message
         assert error['type'] == expected.errors
     else:
@@ -425,8 +426,8 @@ def test_max_length():
     assert str(v.validate_python('https://example.com')) == 'https://example.com/'
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('https://example.com/foo/bar')
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'url_too_long',
             'loc': (),
@@ -450,8 +451,8 @@ def test_allowed_schemes_error():
     v = SchemaValidator(core_schema.url_schema(allowed_schemes=['http', 'https']))
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('unix:/run/foo.socket')
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'url_scheme',
             'loc': (),
@@ -466,8 +467,8 @@ def test_allowed_schemes_errors():
     v = SchemaValidator(core_schema.url_schema(allowed_schemes=['a', 'b', 'c']))
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('unix:/run/foo.socket')
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'url_scheme',
             'loc': (),
@@ -519,8 +520,8 @@ def test_url_to_constraint():
 
     with pytest.raises(ValidationError) as exc_info:
         v2.validate_python(url)
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'url_too_long',
             'loc': (),
@@ -534,7 +535,7 @@ def test_url_to_constraint():
 
     with pytest.raises(ValidationError) as exc_info:
         v3.validate_python(url)
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'url_scheme',
             'loc': (),
@@ -888,7 +889,7 @@ def test_multi_url_cases(multi_host_url_validator, url, expected):
         with pytest.raises(ValidationError) as exc_info:
             multi_host_url_validator.validate_python(url)
         assert exc_info.value.error_count() == 1
-        error = exc_info.value.errors()[0]
+        error = exc_info.value.errors(include_url=False)[0]
         assert error['type'] == 'url_parsing'
         assert error['ctx']['error'] == expected.message
     else:
@@ -944,7 +945,7 @@ def test_multi_url_cases_strict(strict_multi_host_url_validator, url, expected):
         with pytest.raises(ValidationError) as exc_info:
             strict_multi_host_url_validator.validate_python(url)
         assert exc_info.value.error_count() == 1
-        error = exc_info.value.errors()[0]
+        error = exc_info.value.errors(include_url=False)[0]
         assert error['type'] == expected.errors
         assert error['ctx']['error'] == expected.message
     else:
@@ -1140,3 +1141,75 @@ def test_url_vulnerabilities(url_validator, url, expected):
             else:
                 output_parts[key] = getattr(output_url, key)
         assert output_parts == expected
+
+
+def test_multi_host_url_comparison() -> None:
+    assert MultiHostUrl('http://example.com,www.example.com') == MultiHostUrl('http://example.com,www.example.com')
+    assert MultiHostUrl('http://example.com,www.example.com') == MultiHostUrl('http://example.com,www.example.com/')
+    assert MultiHostUrl('http://example.com,www.example.com') != MultiHostUrl('http://example.com,www.example.com/123')
+    assert MultiHostUrl('http://example.com,www.example.com/123') > MultiHostUrl('http://example.com,www.example.com')
+    assert MultiHostUrl('http://example.com,www.example.com/123') >= MultiHostUrl('http://example.com,www.example.com')
+    assert MultiHostUrl('http://example.com,www.example.com') >= MultiHostUrl('http://example.com,www.example.com')
+    assert MultiHostUrl('http://example.com,www.example.com') < MultiHostUrl('http://example.com,www.example.com/123')
+    assert MultiHostUrl('http://example.com,www.example.com') <= MultiHostUrl('http://example.com,www.example.com/123')
+    assert MultiHostUrl('http://example.com,www.example.com') <= MultiHostUrl('http://example.com')
+
+
+def test_multi_host_url_bool() -> None:
+    assert bool(MultiHostUrl('http://example.com,www.example.com')) is True
+
+
+def test_multi_host_url_hash() -> None:
+    data: Dict[MultiHostUrl, int] = {}
+
+    data[MultiHostUrl('http://example.com,www.example.com')] = 1
+    assert data == {MultiHostUrl('http://example.com,www.example.com/'): 1}
+
+    data[MultiHostUrl('http://example.com,www.example.com/123')] = 2
+    assert data == {
+        MultiHostUrl('http://example.com,www.example.com/'): 1,
+        MultiHostUrl('http://example.com,www.example.com/123'): 2,
+    }
+
+    data[MultiHostUrl('http://example.com,www.example.com')] = 3
+    assert data == {
+        MultiHostUrl('http://example.com,www.example.com/'): 3,
+        MultiHostUrl('http://example.com,www.example.com/123'): 2,
+    }
+
+
+def test_multi_host_url_deepcopy() -> None:
+    assert deepcopy(MultiHostUrl('http://example.com')) == MultiHostUrl('http://example.com/')
+
+
+def test_url_comparison() -> None:
+    assert Url('http://example.com') == Url('http://example.com')
+    assert Url('http://example.com') == Url('http://example.com/')
+    assert Url('http://example.com') != Url('http://example.com/123')
+    assert Url('http://example.com/123') > Url('http://example.com')
+    assert Url('http://example.com/123') >= Url('http://example.com')
+    assert Url('http://example.com') >= Url('http://example.com')
+    assert Url('http://example.com') < Url('http://example.com/123')
+    assert Url('http://example.com') <= Url('http://example.com/123')
+    assert Url('http://example.com') <= Url('http://example.com')
+
+
+def test_url_bool() -> None:
+    assert bool(Url('http://example.com')) is True
+
+
+def test_url_hash() -> None:
+    data: Dict[Url, int] = {}
+
+    data[Url('http://example.com')] = 1
+    assert data == {Url('http://example.com/'): 1}
+
+    data[Url('http://example.com/123')] = 2
+    assert data == {Url('http://example.com/'): 1, Url('http://example.com/123'): 2}
+
+    data[Url('http://example.com')] = 3
+    assert data == {Url('http://example.com/'): 3, Url('http://example.com/123'): 2}
+
+
+def test_url_deepcopy() -> None:
+    assert deepcopy(Url('http://example.com')) == Url('http://example.com/')
